//  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *  //
//                                                                                   //
//                       / /   / /    __    \ \  / /                                 //
//                      / /__ / /   / _ \    \ \/ /                                  //
//                     /  ___  /   | |/_/    / /\ \                                  //
//                    / /   / /    \_\      / /  \ \                                 //
//                                                                                   //
//                                                                                   //
//  Copyright (c) 2015, Jakub Benda, Charles University in Prague                    //
//                                                                                   //
// MIT License:                                                                      //
//                                                                                   //
//  Permission is hereby granted, free of charge, to any person obtaining a          //
// copy of this software and associated documentation files (the "Software"),        //
// to deal in the Software without restriction, including without limitation         //
// the rights to use, copy, modify, merge, publish, distribute, sublicense,          //
// and/or sell copies of the Software, and to permit persons to whom the             //
// Software is furnished to do so, subject to the following conditions:              //
//                                                                                   //
//  The above copyright notice and this permission notice shall be included          //
// in all copies or substantial portions of the Software.                            //
//                                                                                   //
//  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS          //
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,       //
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE       //
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, //
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF         //
// OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.  //
//                                                                                   //
//  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *  //

#include <map>
#include <vector>

#include "hex-spgrid.h"

const std::vector<unsigned> spgrid::vtk_cell = {
     1, // 0-D: vertex (not used)
     3, // 1-D: line
     9, // 2-D: quad
    12  // 3-D: hexahedron
};

// The nodes and weights are from the web page http://sparse-grids.de/#Nodes.
// It is a subset of the whole data for 1 <= dim <= 6 and 1 <= l <= 5.

const std::map<spgrid::SparseGridId, spgrid::SparseGridNodes> spgrid::nodes = {
    {
        d1l1n1,
        {
            /* dim = */ 1,
            /* n = */ 1,
            /* x = */ std::vector<double>
            ({
                0.5
            }),
            /* w = */ std::vector<double>
            ({
                 1
            })
        }
    },
    {
        d1l2n3,
        {
            /* dim = */ 1,
            /* n = */ 3,
            /* x = */ std::vector<double>
            ({
                0.1127017, 
                0.5, 
                0.8872983
            }),
            /* w = */ std::vector<double>
            ({
                 0.2777778, 
                 0.4444444, 
                 0.2777778
            })
        }
    },
    {
        d1l3n3,
        {
            /* dim = */ 1,
            /* n = */ 3,
            /* x = */ std::vector<double>
            ({
                0.1127017, 
                0.5, 
                0.8872983
            }),
            /* w = */ std::vector<double>
            ({
                 0.2777778, 
                 0.4444444, 
                 0.2777778
            })
        }
    },
    {
        d1l4n7,
        {
            /* dim = */ 1,
            /* n = */ 7,
            /* x = */ std::vector<double>
            ({
                0.0197543999999999, 
                0.1127017, 
                0.2828781, 
                0.5, 
                0.7171219, 
                0.8872983, 
                0.9802456
            }),
            /* w = */ std::vector<double>
            ({
                 0.0523281052328105, 
                 0.134244013424401, 
                 0.200698720069872, 
                 0.225458322545832, 
                 0.200698720069872, 
                 0.134244013424401, 
                 0.0523281052328105
            })
        }
    },
    {
        d1l5n7,
        {
            /* dim = */ 1,
            /* n = */ 7,
            /* x = */ std::vector<double>
            ({
                0.0197543999999999, 
                0.1127017, 
                0.2828781, 
                0.5, 
                0.7171219, 
                0.8872983, 
                0.9802456
            }),
            /* w = */ std::vector<double>
            ({
                 0.0523281052328105, 
                 0.134244013424401, 
                 0.200698720069872, 
                 0.225458322545832, 
                 0.200698720069872, 
                 0.134244013424401, 
                 0.0523281052328105
            })
        }
    },
    {
        d2l1n1,
        {
            /* dim = */ 2,
            /* n = */ 1,
            /* x = */ std::vector<double>
            ({
                0.5, 0.5
            }),
            /* w = */ std::vector<double>
            ({
                 1
            })
        }
    },
    {
        d2l2n5,
        {
            /* dim = */ 2,
            /* n = */ 5,
            /* x = */ std::vector<double>
            ({
                0.1127017, 0.5, 
                0.5, 0.1127017, 
                0.5, 0.5, 
                0.5, 0.8872983, 
                0.8872983, 0.5
            }),
            /* w = */ std::vector<double>
            ({
                 0.2777778, 
                 0.2777778, 
                -0.1111112, 
                 0.2777778, 
                 0.2777778
            })
        }
    },
    {
        d2l3n9,
        {
            /* dim = */ 2,
            /* n = */ 9,
            /* x = */ std::vector<double>
            ({
                0.1127017, 0.1127017, 
                0.1127017, 0.5, 
                0.1127017, 0.8872983, 
                0.5, 0.1127017, 
                0.5, 0.5, 
                0.5, 0.8872983, 
                0.8872983, 0.1127017, 
                0.8872983, 0.5, 
                0.8872983, 0.8872983
            }),
            /* w = */ std::vector<double>
            ({
                 0.07716050617284, 
                 0.12345678765432, 
                 0.07716050617284, 
                 0.12345678765432, 
                 0.19753082469136, 
                 0.12345678765432, 
                 0.07716050617284, 
                 0.12345678765432, 
                 0.07716050617284
            })
        }
    },
    {
        d2l4n17,
        {
            /* dim = */ 2,
            /* n = */ 17,
            /* x = */ std::vector<double>
            ({
                0.0197543999999999, 0.5, 
                0.1127017, 0.1127017, 
                0.1127017, 0.5, 
                0.1127017, 0.8872983, 
                0.2828781, 0.5, 
                0.5, 0.0197543999999999, 
                0.5, 0.1127017, 
                0.5, 0.2828781, 
                0.5, 0.5, 
                0.5, 0.7171219, 
                0.5, 0.8872983, 
                0.5, 0.9802456, 
                0.7171219, 0.5, 
                0.8872983, 0.1127017, 
                0.8872983, 0.5, 
                0.8872983, 0.8872983, 
                0.9802456, 0.5
            }),
            /* w = */ std::vector<double>
            ({
                 0.0523281052328105, 
                 0.07716050617284, 
                -0.0200769989212786, 
                 0.07716050617284, 
                 0.200698720069872, 
                 0.0523281052328105, 
                -0.0200769989212787, 
                 0.200698720069872, 
                -0.240441330216975, 
                 0.200698720069872, 
                -0.0200769989212787, 
                 0.0523281052328105, 
                 0.200698720069872, 
                 0.07716050617284, 
                -0.0200769989212786, 
                 0.07716050617284, 
                 0.0523281052328105
            })
        }
    },
    {
        d2l5n33,
        {
            /* dim = */ 2,
            /* n = */ 33,
            /* x = */ std::vector<double>
            ({
                0.0197543999999999, 0.1127017, 
                0.0197543999999999, 0.5, 
                0.0197543999999999, 0.8872983, 
                0.1127017, 0.0197543999999999, 
                0.1127017, 0.1127017, 
                0.1127017, 0.2828781, 
                0.1127017, 0.5, 
                0.1127017, 0.7171219, 
                0.1127017, 0.8872983, 
                0.1127017, 0.9802456, 
                0.2828781, 0.1127017, 
                0.2828781, 0.5, 
                0.2828781, 0.8872983, 
                0.5, 0.0197543999999999, 
                0.5, 0.1127017, 
                0.5, 0.2828781, 
                0.5, 0.5, 
                0.5, 0.7171219, 
                0.5, 0.8872983, 
                0.5, 0.9802456, 
                0.7171219, 0.1127017, 
                0.7171219, 0.5, 
                0.7171219, 0.8872983, 
                0.8872983, 0.0197543999999999, 
                0.8872983, 0.1127017, 
                0.8872983, 0.2828781, 
                0.8872983, 0.5, 
                0.8872983, 0.7171219, 
                0.8872983, 0.8872983, 
                0.8872983, 0.9802456, 
                0.9802456, 0.1127017, 
                0.9802456, 0.5, 
                0.9802456, 0.8872983
            }),
            /* w = */ std::vector<double>
            ({
                 0.0145355859497386, 
                 0.0232569333333333, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                -0.00258049274843866, 
                 0.0557496489238249, 
                -0.00116547082584836, 
                 0.0557496489238249, 
                -0.00258049274843866, 
                 0.0145355859497386, 
                 0.0557496489238249, 
                 0.0891994222222222, 
                 0.0557496489238249, 
                 0.0232569333333333, 
                -0.00116547082584831, 
                 0.0891994222222222, 
                 0.00287655308641768, 
                 0.0891994222222222, 
                -0.00116547082584831, 
                 0.0232569333333333, 
                 0.0557496489238249, 
                 0.0891994222222222, 
                 0.0557496489238249, 
                 0.0145355859497386, 
                -0.00258049274843866, 
                 0.0557496489238249, 
                -0.00116547082584836, 
                 0.0557496489238249, 
                -0.00258049274843866, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0232569333333333, 
                 0.0145355859497386
            })
        }
    },
    {
        d2l6n33,
        {
            /* dim = */ 2,
            /* n = */ 33,
            /* x = */ std::vector<double>
            ({
                0.0197543999999999, 0.1127017, 
                0.0197543999999999, 0.5, 
                0.0197543999999999, 0.8872983, 
                0.1127017, 0.0197543999999999, 
                0.1127017, 0.1127017, 
                0.1127017, 0.2828781, 
                0.1127017, 0.5, 
                0.1127017, 0.7171219, 
                0.1127017, 0.8872983, 
                0.1127017, 0.9802456, 
                0.2828781, 0.1127017, 
                0.2828781, 0.5, 
                0.2828781, 0.8872983, 
                0.5, 0.0197543999999999, 
                0.5, 0.1127017, 
                0.5, 0.2828781, 
                0.5, 0.5, 
                0.5, 0.7171219, 
                0.5, 0.8872983, 
                0.5, 0.9802456, 
                0.7171219, 0.1127017, 
                0.7171219, 0.5, 
                0.7171219, 0.8872983, 
                0.8872983, 0.0197543999999999, 
                0.8872983, 0.1127017, 
                0.8872983, 0.2828781, 
                0.8872983, 0.5, 
                0.8872983, 0.7171219, 
                0.8872983, 0.8872983, 
                0.8872983, 0.9802456, 
                0.9802456, 0.1127017, 
                0.9802456, 0.5, 
                0.9802456, 0.8872983
            }),
            /* w = */ std::vector<double>
            ({
                 0.0145355859497386, 
                 0.0232569333333333, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                -0.00258049274843866, 
                 0.0557496489238249, 
                -0.00116547082584836, 
                 0.0557496489238249, 
                -0.00258049274843866, 
                 0.0145355859497386, 
                 0.0557496489238249, 
                 0.0891994222222222, 
                 0.0557496489238249, 
                 0.0232569333333333, 
                -0.00116547082584831, 
                 0.0891994222222222, 
                 0.00287655308641768, 
                 0.0891994222222222, 
                -0.00116547082584831, 
                 0.0232569333333333, 
                 0.0557496489238249, 
                 0.0891994222222222, 
                 0.0557496489238249, 
                 0.0145355859497386, 
                -0.00258049274843866, 
                 0.0557496489238249, 
                -0.00116547082584836, 
                 0.0557496489238249, 
                -0.00258049274843866, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0232569333333333, 
                 0.0145355859497386
            })
        }
    },
    {
        d2l7n65,
        {
            /* dim = */ 2,
            /* n = */ 65,
            /* x = */ std::vector<double>
            ({
                .0030839999999999756, .5,
                .01975439999999995, .01975439999999995,
                .01975439999999995, .11270170000000002,
                .01975439999999995, .28287810000000002,
                .01975439999999995, .5,
                .01975439999999995, .71712189999999998,
                .01975439999999995, .88729829999999998,
                .01975439999999995, .98024560000000005,
                .055770399999999998, .5,
                .11270170000000002, .01975439999999995,
                .11270170000000002, .11270170000000002,
                .11270170000000002, .28287810000000002,
                .11270170000000002, .5,
                .11270170000000002, .71712189999999998,
                .11270170000000002, .88729829999999998,
                .11270170000000002, .98024560000000005,
                .18944850000000002, .5,
                .28287810000000002, .01975439999999995,
                .28287810000000002, .11270170000000002,
                .28287810000000002, .28287810000000002,
                .28287810000000002, .5,
                .28287810000000002, .71712189999999998,
                .28287810000000002, .88729829999999998,
                .28287810000000002, .98024560000000005,
                .3883067, .5,
                .5, .0030839999999999756,
                .5, .01975439999999995,
                .5, .055770399999999998,
                .5, .11270170000000002,
                .5, .18944850000000002,
                .5, .28287810000000002,
                .5, .3883067,
                .5, .5,
                .5, .6116933,
                .5, .71712189999999998,
                .5, .81055149999999998,
                .5, .88729829999999998,
                .5, .9442296,
                .5, .98024560000000005,
                .5, .99691600000000002,
                .6116933, .5,
                .71712189999999998, .01975439999999995,
                .71712189999999998, .11270170000000002,
                .71712189999999998, .28287810000000002,
                .71712189999999998, .5,
                .71712189999999998, .71712189999999998,
                .71712189999999998, .88729829999999998,
                .71712189999999998, .98024560000000005,
                .81055149999999998, .5,
                .88729829999999998, .01975439999999995,
                .88729829999999998, .11270170000000002,
                .88729829999999998, .28287810000000002,
                .88729829999999998, .5,
                .88729829999999998, .71712189999999998,
                .88729829999999998, .88729829999999998,
                .88729829999999998, .98024560000000005,
                .9442296, .5,
                .98024560000000005, .01975439999999995,
                .98024560000000005, .11270170000000002,
                .98024560000000005, .28287810000000002,
                .98024560000000005, .5,
                .98024560000000005, .71712189999999998,
                .98024560000000005, .88729829999999998,
                .98024560000000005, .98024560000000005,
                .99691600000000002, .5
            }),
            /* w = */ std::vector<double>
            ({
                 0.0085008999999999987,
                 0.0027382305972560913,
                 0.0070247348613468995,
                 0.010502183743906641,
                -0.014728698405019263,
                 0.010502183743906641,
                 0.0070247348613468995,
                 0.0027382305972560913,
                 0.046463599999999994,
                 0.0070247348613468995,
                 0.018021455140290844,
                 0.026942601671320059,
                -0.036769983345915601,
                 0.026942601671320059,
                 0.018021455140290844,
                 0.0070247348613468995,
                 0.085755999999999985,
                 0.010502183743906641,
                 0.026942601671320059,
                 0.040279976237684832,
                -0.055135223305823086,
                 0.040279976237684832,
                 0.026942601671320059,
                 0.010502183743906641,
                 0.10957839999999998,
                 0.0085008999999999987,
                -0.01472869840501927,
                 0.046463599999999994,
                -0.03676998334591558,
                 0.085755999999999985,
                -0.055135223305823086,
                 0.10957839999999998,
                -0.17457478988648398,
                 0.10957839999999998,
                -0.055135223305823086,
                 0.085755999999999985,
                -0.03676998334591558,
                 0.046463599999999994,
                -0.01472869840501927,
                 0.0085008999999999987,                                                                                                                                                                                                                                                          
                 0.10957839999999998,
                 0.010502183743906641,
                 0.026942601671320059,
                 0.040279976237684832,
                -0.055135223305823086,
                 0.040279976237684832,
                 0.026942601671320059,
                 0.010502183743906641,
                 0.085755999999999985,
                 0.0070247348613468995,
                 0.018021455140290844,
                 0.026942601671320059,
                -0.036769983345915601,
                 0.026942601671320059,
                 0.018021455140290844,
                 0.0070247348613468995,
                 0.046463599999999994,
                 0.0027382305972560913,
                 0.0070247348613468995,
                 0.010502183743906641,
                -0.014728698405019263,
                 0.010502183743906641,
                 0.0070247348613468995,
                 0.0027382305972560913,
                 0.0085008999999999987
            })
        }
    },
    {
        d3l1n1,
        {
            /* dim = */ 3,
            /* n = */ 1,
            /* x = */ std::vector<double>
            ({
                0.5, 0.5, 0.5
            }),
            /* w = */ std::vector<double>
            ({
                 1
            })
        }
    },
    {
        d3l2n7,
        {
            /* dim = */ 3,
            /* n = */ 7,
            /* x = */ std::vector<double>
            ({
                0.1127017, 0.5, 0.5, 
                0.5, 0.1127017, 0.5, 
                0.5, 0.5, 0.1127017, 
                0.5, 0.5, 0.5, 
                0.5, 0.5, 0.8872983, 
                0.5, 0.8872983, 0.5, 
                0.8872983, 0.5, 0.5
            }),
            /* w = */ std::vector<double>
            ({
                 0.2777778, 
                 0.2777778, 
                 0.2777778, 
                -0.6666668, 
                 0.2777778, 
                 0.2777778, 
                 0.2777778
            })
        }
    },
    {
        d3l3n19,
        {
            /* dim = */ 3,
            /* n = */ 19,
            /* x = */ std::vector<double>
            ({
                0.1127017, 0.1127017, 0.5, 
                0.1127017, 0.5, 0.1127017, 
                0.1127017, 0.5, 0.5, 
                0.1127017, 0.5, 0.8872983, 
                0.1127017, 0.8872983, 0.5, 
                0.5, 0.1127017, 0.1127017, 
                0.5, 0.1127017, 0.5, 
                0.5, 0.1127017, 0.8872983, 
                0.5, 0.5, 0.1127017, 
                0.5, 0.5, 0.5, 
                0.5, 0.5, 0.8872983, 
                0.5, 0.8872983, 0.1127017, 
                0.5, 0.8872983, 0.5, 
                0.5, 0.8872983, 0.8872983, 
                0.8872983, 0.1127017, 0.5, 
                0.8872983, 0.5, 0.1127017, 
                0.8872983, 0.5, 0.5, 
                0.8872983, 0.5, 0.8872983, 
                0.8872983, 0.8872983, 0.5
            }),
            /* w = */ std::vector<double>
            ({
                 0.07716050617284, 
                 0.07716050617284, 
                -0.03086422469136, 
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                -0.03086422469136, 
                 0.07716050617284, 
                -0.03086422469136, 
                 0.25925927407408, 
                -0.03086422469136, 
                 0.07716050617284, 
                -0.03086422469136, 
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                -0.03086422469136, 
                 0.07716050617284, 
                 0.07716050617284
            })
        }
    },
    {
        d3l4n39,
        {
            /* dim = */ 3,
            /* n = */ 39,
            /* x = */ std::vector<double>
            ({
                0.0197543999999999, 0.5, 0.5, 
                0.1127017, 0.1127017, 0.1127017, 
                0.1127017, 0.1127017, 0.5, 
                0.1127017, 0.1127017, 0.8872983, 
                0.1127017, 0.5, 0.1127017, 
                0.1127017, 0.5, 0.5, 
                0.1127017, 0.5, 0.8872983, 
                0.1127017, 0.8872983, 0.1127017, 
                0.1127017, 0.8872983, 0.5, 
                0.1127017, 0.8872983, 0.8872983, 
                0.2828781, 0.5, 0.5, 
                0.5, 0.0197543999999999, 0.5, 
                0.5, 0.1127017, 0.1127017, 
                0.5, 0.1127017, 0.5, 
                0.5, 0.1127017, 0.8872983, 
                0.5, 0.2828781, 0.5, 
                0.5, 0.5, 0.0197543999999999, 
                0.5, 0.5, 0.1127017, 
                0.5, 0.5, 0.2828781, 
                0.5, 0.5, 0.5, 
                0.5, 0.5, 0.7171219, 
                0.5, 0.5, 0.8872983, 
                0.5, 0.5, 0.9802456, 
                0.5, 0.7171219, 0.5, 
                0.5, 0.8872983, 0.1127017, 
                0.5, 0.8872983, 0.5, 
                0.5, 0.8872983, 0.8872983, 
                0.5, 0.9802456, 0.5, 
                0.7171219, 0.5, 0.5, 
                0.8872983, 0.1127017, 0.1127017, 
                0.8872983, 0.1127017, 0.5, 
                0.8872983, 0.1127017, 0.8872983, 
                0.8872983, 0.5, 0.1127017, 
                0.8872983, 0.5, 0.5, 
                0.8872983, 0.5, 0.8872983, 
                0.8872983, 0.8872983, 0.1127017, 
                0.8872983, 0.8872983, 0.5, 
                0.8872983, 0.8872983, 0.8872983, 
                0.9802456, 0.5, 0.5
            }),
            /* w = */ std::vector<double>
            ({
                 0.0523281052328105, 
                 0.0214334756515779, 
                 0.0342935548696842, 
                 0.0214334756515779, 
                 0.0342935548696842, 
                -0.0886641086606471, 
                 0.0342935548696842, 
                 0.0214334756515779, 
                 0.0342935548696842, 
                 0.0214334756515779, 
                 0.200698720069872, 
                 0.0523281052328105, 
                 0.0342935548696842, 
                -0.0886641086606471, 
                 0.0342935548696842, 
                 0.200698720069872, 
                 0.0523281052328105, 
                -0.088664108660647, 
                 0.200698720069872, 
                -0.569166763501047, 
                 0.200698720069872, 
                -0.088664108660647, 
                 0.0523281052328105, 
                 0.200698720069872, 
                 0.0342935548696842, 
                -0.0886641086606471, 
                 0.0342935548696842, 
                 0.0523281052328105, 
                 0.200698720069872, 
                 0.0214334756515779, 
                 0.0342935548696842, 
                 0.0214334756515779, 
                 0.0342935548696842, 
                -0.0886641086606471, 
                 0.0342935548696842, 
                 0.0214334756515779, 
                 0.0342935548696842, 
                 0.0214334756515779, 
                 0.0523281052328105
            })
        }
    },
    {
        d3l5n87,
        {
            /* dim = */ 3,
            /* n = */ 87,
            /* x = */ std::vector<double>
            ({
                0.0197543999999999, 0.1127017, 0.5, 
                0.0197543999999999, 0.5, 0.1127017, 
                0.0197543999999999, 0.5, 0.5, 
                0.0197543999999999, 0.5, 0.8872983, 
                0.0197543999999999, 0.8872983, 0.5, 
                0.1127017, 0.0197543999999999, 0.5, 
                0.1127017, 0.1127017, 0.1127017, 
                0.1127017, 0.1127017, 0.5, 
                0.1127017, 0.1127017, 0.8872983, 
                0.1127017, 0.2828781, 0.5, 
                0.1127017, 0.5, 0.0197543999999999, 
                0.1127017, 0.5, 0.1127017, 
                0.1127017, 0.5, 0.2828781, 
                0.1127017, 0.5, 0.5, 
                0.1127017, 0.5, 0.7171219, 
                0.1127017, 0.5, 0.8872983, 
                0.1127017, 0.5, 0.9802456, 
                0.1127017, 0.7171219, 0.5, 
                0.1127017, 0.8872983, 0.1127017, 
                0.1127017, 0.8872983, 0.5, 
                0.1127017, 0.8872983, 0.8872983, 
                0.1127017, 0.9802456, 0.5, 
                0.2828781, 0.1127017, 0.5, 
                0.2828781, 0.5, 0.1127017, 
                0.2828781, 0.5, 0.5, 
                0.2828781, 0.5, 0.8872983, 
                0.2828781, 0.8872983, 0.5, 
                0.5, 0.0197543999999999, 0.1127017, 
                0.5, 0.0197543999999999, 0.5, 
                0.5, 0.0197543999999999, 0.8872983, 
                0.5, 0.1127017, 0.0197543999999999, 
                0.5, 0.1127017, 0.1127017, 
                0.5, 0.1127017, 0.2828781, 
                0.5, 0.1127017, 0.5, 
                0.5, 0.1127017, 0.7171219, 
                0.5, 0.1127017, 0.8872983, 
                0.5, 0.1127017, 0.9802456, 
                0.5, 0.2828781, 0.1127017, 
                0.5, 0.2828781, 0.5, 
                0.5, 0.2828781, 0.8872983, 
                0.5, 0.5, 0.0197543999999999, 
                0.5, 0.5, 0.1127017, 
                0.5, 0.5, 0.2828781, 
                0.5, 0.5, 0.5, 
                0.5, 0.5, 0.7171219, 
                0.5, 0.5, 0.8872983, 
                0.5, 0.5, 0.9802456, 
                0.5, 0.7171219, 0.1127017, 
                0.5, 0.7171219, 0.5, 
                0.5, 0.7171219, 0.8872983, 
                0.5, 0.8872983, 0.0197543999999999, 
                0.5, 0.8872983, 0.1127017, 
                0.5, 0.8872983, 0.2828781, 
                0.5, 0.8872983, 0.5, 
                0.5, 0.8872983, 0.7171219, 
                0.5, 0.8872983, 0.8872983, 
                0.5, 0.8872983, 0.9802456, 
                0.5, 0.9802456, 0.1127017, 
                0.5, 0.9802456, 0.5, 
                0.5, 0.9802456, 0.8872983, 
                0.7171219, 0.1127017, 0.5, 
                0.7171219, 0.5, 0.1127017, 
                0.7171219, 0.5, 0.5, 
                0.7171219, 0.5, 0.8872983, 
                0.7171219, 0.8872983, 0.5, 
                0.8872983, 0.0197543999999999, 0.5, 
                0.8872983, 0.1127017, 0.1127017, 
                0.8872983, 0.1127017, 0.5, 
                0.8872983, 0.1127017, 0.8872983, 
                0.8872983, 0.2828781, 0.5, 
                0.8872983, 0.5, 0.0197543999999999, 
                0.8872983, 0.5, 0.1127017, 
                0.8872983, 0.5, 0.2828781, 
                0.8872983, 0.5, 0.5, 
                0.8872983, 0.5, 0.7171219, 
                0.8872983, 0.5, 0.8872983, 
                0.8872983, 0.5, 0.9802456, 
                0.8872983, 0.7171219, 0.5, 
                0.8872983, 0.8872983, 0.1127017, 
                0.8872983, 0.8872983, 0.5, 
                0.8872983, 0.8872983, 0.8872983, 
                0.8872983, 0.9802456, 0.5, 
                0.9802456, 0.1127017, 0.5, 
                0.9802456, 0.5, 0.1127017, 
                0.9802456, 0.5, 0.5, 
                0.9802456, 0.5, 0.8872983, 
                0.9802456, 0.8872983, 0.5
            }),
            /* w = */ std::vector<double>
            ({
                 0.0145355859497386, 
                 0.0145355859497386, 
                -0.00581423856614385, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0214334756515779, 
                -0.0454474440515945, 
                 0.0214334756515779, 
                 0.0557496489238249, 
                 0.0145355859497386, 
                -0.0454474440515945, 
                 0.0557496489238249, 
                -0.0508410524697862, 
                 0.0557496489238249, 
                -0.0454474440515945, 
                 0.0145355859497386, 
                 0.0557496489238249, 
                 0.0214334756515779, 
                -0.0454474440515945, 
                 0.0214334756515779, 
                 0.0145355859497386, 
                 0.0557496489238249, 
                 0.0557496489238249, 
                -0.0222998756254276, 
                 0.0557496489238249, 
                 0.0557496489238249, 
                 0.0145355859497386, 
                -0.00581423856614385, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                -0.0454474440515945, 
                 0.0557496489238249, 
                -0.0508410524697862, 
                 0.0557496489238249, 
                -0.0454474440515945, 
                 0.0145355859497386, 
                 0.0557496489238249, 
                -0.0222998756254276, 
                 0.0557496489238249, 
                -0.00581423856614384, 
                -0.0508410524697864, 
                -0.0222998756254276, 
                 0.160786886409133, 
                -0.0222998756254276, 
                -0.0508410524697864, 
                -0.00581423856614384, 
                 0.0557496489238249, 
                -0.0222998756254276, 
                 0.0557496489238249, 
                 0.0145355859497386, 
                -0.0454474440515945, 
                 0.0557496489238249, 
                -0.0508410524697862, 
                 0.0557496489238249, 
                -0.0454474440515945, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                -0.00581423856614385, 
                 0.0145355859497386, 
                 0.0557496489238249, 
                 0.0557496489238249, 
                -0.0222998756254276, 
                 0.0557496489238249, 
                 0.0557496489238249, 
                 0.0145355859497386, 
                 0.0214334756515779, 
                -0.0454474440515945, 
                 0.0214334756515779, 
                 0.0557496489238249, 
                 0.0145355859497386, 
                -0.0454474440515945, 
                 0.0557496489238249, 
                -0.0508410524697862, 
                 0.0557496489238249, 
                -0.0454474440515945, 
                 0.0145355859497386, 
                 0.0557496489238249, 
                 0.0214334756515779, 
                -0.0454474440515945, 
                 0.0214334756515779, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                -0.00581423856614385, 
                 0.0145355859497386, 
                 0.0145355859497386
            })
        }
    },
    {
        d3l6n135,
        {
            /* dim = */ 3,
            /* n = */ 135,
            /* x = */ std::vector<double>
            ({
                .01975439999999995, .11270170000000002, .11270170000000002,
                .01975439999999995, .11270170000000002, .5,
                .01975439999999995, .11270170000000002, .88729829999999998,
                .01975439999999995, .5, .11270170000000002,
                .01975439999999995, .5, .5,
                .01975439999999995, .5, .88729829999999998,
                .01975439999999995, .88729829999999998, .11270170000000002,
                .01975439999999995, .88729829999999998, .5,
                .01975439999999995, .88729829999999998, .88729829999999998,
                .11270170000000002, .01975439999999995, .11270170000000002,
                .11270170000000002, .01975439999999995, .5,
                .11270170000000002, .01975439999999995, .88729829999999998,
                .11270170000000002, .11270170000000002, .01975439999999995,
                .11270170000000002, .11270170000000002, .11270170000000002,
                .11270170000000002, .11270170000000002, .28287810000000002,
                .11270170000000002, .11270170000000002, .5,
                .11270170000000002, .11270170000000002, .71712189999999998,
                .11270170000000002, .11270170000000002, .88729829999999998,
                .11270170000000002, .11270170000000002, .98024560000000005,
                .11270170000000002, .28287810000000002, .11270170000000002,
                .11270170000000002, .28287810000000002, .5,
                .11270170000000002, .28287810000000002, .88729829999999998,
                .11270170000000002, .5, .01975439999999995,
                .11270170000000002, .5, .11270170000000002,
                .11270170000000002, .5, .28287810000000002,
                .11270170000000002, .5, .5,
                .11270170000000002, .5, .71712189999999998,
                .11270170000000002, .5, .88729829999999998,
                .11270170000000002, .5, .98024560000000005,
                .11270170000000002, .71712189999999998, .11270170000000002,
                .11270170000000002, .71712189999999998, .5,
                .11270170000000002, .71712189999999998, .88729829999999998,
                .11270170000000002, .88729829999999998, .01975439999999995,
                .11270170000000002, .88729829999999998, .11270170000000002,
                .11270170000000002, .88729829999999998, .28287810000000002,
                .11270170000000002, .88729829999999998, .5,
                .11270170000000002, .88729829999999998, .71712189999999998,
                .11270170000000002, .88729829999999998, .88729829999999998,
                .11270170000000002, .88729829999999998, .98024560000000005,
                .11270170000000002, .98024560000000005, .11270170000000002,
                .11270170000000002, .98024560000000005, .5,
                .11270170000000002, .98024560000000005, .88729829999999998,
                .28287810000000002, .11270170000000002, .11270170000000002,
                .28287810000000002, .11270170000000002, .5,
                .28287810000000002, .11270170000000002, .88729829999999998,
                .28287810000000002, .5, .11270170000000002,
                .28287810000000002, .5, .5,
                .28287810000000002, .5, .88729829999999998,
                .28287810000000002, .88729829999999998, .11270170000000002,
                .28287810000000002, .88729829999999998, .5,
                .28287810000000002, .88729829999999998, .88729829999999998,
                .5, .01975439999999995, .11270170000000002,
                .5, .01975439999999995, .5,
                .5, .01975439999999995, .88729829999999998,
                .5, .11270170000000002, .01975439999999995,
                .5, .11270170000000002, .11270170000000002,
                .5, .11270170000000002, .28287810000000002,
                .5, .11270170000000002, .5,
                .5, .11270170000000002, .71712189999999998,
                .5, .11270170000000002, .88729829999999998,
                .5, .11270170000000002, .98024560000000005,
                .5, .28287810000000002, .11270170000000002,
                .5, .28287810000000002, .5,
                .5, .28287810000000002, .88729829999999998,
                .5, .5, .01975439999999995,
                .5, .5, .11270170000000002,
                .5, .5, .28287810000000002,
                .5, .5, .5,
                .5, .5, .71712189999999998,
                .5, .5, .88729829999999998,
                .5, .5, .98024560000000005,
                .5, .71712189999999998, .11270170000000002,
                .5, .71712189999999998, .5,
                .5, .71712189999999998, .88729829999999998,
                .5, .88729829999999998, .01975439999999995,
                .5, .88729829999999998, .11270170000000002,
                .5, .88729829999999998, .28287810000000002,
                .5, .88729829999999998, .5,
                .5, .88729829999999998, .71712189999999998,
                .5, .88729829999999998, .88729829999999998,
                .5, .88729829999999998, .98024560000000005,
                .5, .98024560000000005, .11270170000000002,
                .5, .98024560000000005, .5,
                .5, .98024560000000005, .88729829999999998,
                .71712189999999998, .11270170000000002, .11270170000000002,
                .71712189999999998, .11270170000000002, .5,
                .71712189999999998, .11270170000000002, .88729829999999998,
                .71712189999999998, .5, .11270170000000002,
                .71712189999999998, .5, .5,
                .71712189999999998, .5, .88729829999999998,
                .71712189999999998, .88729829999999998, .11270170000000002,
                .71712189999999998, .88729829999999998, .5,
                .71712189999999998, .88729829999999998, .88729829999999998,
                .88729829999999998, .01975439999999995, .11270170000000002,
                .88729829999999998, .01975439999999995, .5,
                .88729829999999998, .01975439999999995, .88729829999999998,
                .88729829999999998, .11270170000000002, .01975439999999995,
                .88729829999999998, .11270170000000002, .11270170000000002,
                .88729829999999998, .11270170000000002, .28287810000000002,
                .88729829999999998, .11270170000000002, .5,
                .88729829999999998, .11270170000000002, .71712189999999998,
                .88729829999999998, .11270170000000002, .88729829999999998,
                .88729829999999998, .11270170000000002, .98024560000000005,
                .88729829999999998, .28287810000000002, .11270170000000002,
                .88729829999999998, .28287810000000002, .5,
                .88729829999999998, .28287810000000002, .88729829999999998,
                .88729829999999998, .5, .01975439999999995,
                .88729829999999998, .5, .11270170000000002,
                .88729829999999998, .5, .28287810000000002,
                .88729829999999998, .5, .5,
                .88729829999999998, .5, .71712189999999998,
                .88729829999999998, .5, .88729829999999998,
                .88729829999999998, .5, .98024560000000005,
                .88729829999999998, .71712189999999998, .11270170000000002,
                .88729829999999998, .71712189999999998, .5,
                .88729829999999998, .71712189999999998, .88729829999999998,
                .88729829999999998, .88729829999999998, .01975439999999995,
                .88729829999999998, .88729829999999998, .11270170000000002,
                .88729829999999998, .88729829999999998, .28287810000000002,
                .88729829999999998, .88729829999999998, .5,
                .88729829999999998, .88729829999999998, .71712189999999998,
                .88729829999999998, .88729829999999998, .88729829999999998,
                .88729829999999998, .88729829999999998, .98024560000000005,
                .88729829999999998, .98024560000000005, .11270170000000002,
                .88729829999999998, .98024560000000005, .5,
                .88729829999999998, .98024560000000005, .88729829999999998,
                .98024560000000005, .11270170000000002, .11270170000000002,
                .98024560000000005, .11270170000000002, .5,
                .98024560000000005, .11270170000000002, .88729829999999998,
                .98024560000000005, .5, .11270170000000002,
                .98024560000000005, .5, .5,
                .98024560000000005, .5, .88729829999999998,
                .98024560000000005, .88729829999999998, .11270170000000002,
                .98024560000000005, .88729829999999998, .5,
                .98024560000000005, .88729829999999998, .88729829999999998
            }),
            /* w = */ std::vector<double>
            ({
                 .0040376630868293002,
                .0064602597760800027,
                .0040376630868293002,
                .0064602597760800053,
                .010336413781173323,
                .0064602597760800053,
                .0040376630868293002,
                .0064602597760800027,
                .0040376630868293002,
                .0040376630868293002,
                .0064602597760800027,
                .0040376630868293002,
                .0040376630868293002,
                -.011791943223654825,
                .015486014828832457,
                -.018043962132452507,
                .015486014828832457,
                -.011791943223654825,
                .0040376630868293002,
                .015486014828832457,
                .024777619266160018,
                .015486014828832457,
                .0064602597760800027,
                -.018043962132452514,
                .024777619266160018,
                -.027553304645423583,
                .024777619266160018,
                -.018043962132452514,
                .0064602597760800027,
                .015486014828832457,
                .024777619266160018,
                .015486014828832457,
                .0040376630868293002,
                -.011791943223654825,
                .015486014828832457,
                -.018043962132452507,
                .015486014828832457,
                -.011791943223654825,
                .0040376630868293002,
                .0040376630868293002,
                .0064602597760800027,
                .0040376630868293002,
                .015486014828832457,
                .024777619266160015,
                .015486014828832457,
                .024777619266160015,
                .039644183689902246,
                .024777619266160015,
                .015486014828832457,
                .024777619266160015,
                .015486014828832457,
                .0064602597760800053,
                .010336413781173351,
                .0064602597760800053,
                .0064602597760800044,
                -.018043962132452494,
                .024777619266160015,
                -.027553304645423268,
                .024777619266160015,
                -.018043962132452494,
                .0064602597760800044,
                .024777619266160015,
                .039644183689902246,
                .024777619266160015,
                .010336413781173327,
                -.027553304645423368,
                .039644183689902329,
                -.041978032564886886,
                .039644183689902329,
                -.027553304645423368,
                .010336413781173327,
                .024777619266160015,
                .039644183689902246,
                .024777619266160015,
                .0064602597760800044,
                -.018043962132452494,
                .024777619266160015,
                -.027553304645423268,
                .024777619266160015,
                -.018043962132452494,
                .0064602597760800044,
                .0064602597760800053,
                .010336413781173351,
                .0064602597760800053,
                .015486014828832457,
                .024777619266160015,
                .015486014828832457,
                .024777619266160015,
                .039644183689902246,
                .024777619266160015,
                .015486014828832457,
                .024777619266160015,
                .015486014828832457,
                .0040376630868293002,
                .0064602597760800027,
                .0040376630868293002,
                .0040376630868293002,
                -.011791943223654825,
                .015486014828832457,
                -.018043962132452507,
                .015486014828832457,
                -.011791943223654825,
                .0040376630868293002,
                .015486014828832457,
                .024777619266160018,
                .015486014828832457,
                .0064602597760800027,
                -.018043962132452514,
                .024777619266160018,
                -.027553304645423583,
                .024777619266160018,
                -.018043962132452514,
                .0064602597760800027,
                .015486014828832457,
                .024777619266160018,
                .015486014828832457,
                .0040376630868293002,
                -.011791943223654825,
                .015486014828832457,
                -.018043962132452507,
                .015486014828832457,
                -.011791943223654825,
                .0040376630868293002,
                .0040376630868293002,
                .0064602597760800027,
                .0040376630868293002,
                .0040376630868293002,
                .0064602597760800027,
                .0040376630868293002,
                .0064602597760800053,
                .010336413781173323,
                .0064602597760800053,
                .0040376630868293002,
                .0064602597760800027,
                .0040376630868293002
            })
        }
    },
    {
        d4l1n1,
        {
            /* dim = */ 4,
            /* n = */ 1,
            /* x = */ std::vector<double>
            ({
                0.5, 0.5, 0.5, 0.5
            }),
            /* w = */ std::vector<double>
            ({
                 1
            })
        }
    },
    {
        d4l2n9,
        {
            /* dim = */ 4,
            /* n = */ 9,
            /* x = */ std::vector<double>
            ({
                0.1127017, 0.5, 0.5, 0.5, 
                0.5, 0.1127017, 0.5, 0.5, 
                0.5, 0.5, 0.1127017, 0.5, 
                0.5, 0.5, 0.5, 0.1127017, 
                0.5, 0.5, 0.5, 0.5, 
                0.5, 0.5, 0.5, 0.8872983, 
                0.5, 0.5, 0.8872983, 0.5, 
                0.5, 0.8872983, 0.5, 0.5, 
                0.8872983, 0.5, 0.5, 0.5
            }),
            /* w = */ std::vector<double>
            ({
                 0.2777778, 
                 0.2777778, 
                 0.2777778, 
                 0.2777778, 
                -1.2222224, 
                 0.2777778, 
                 0.2777778, 
                 0.2777778, 
                 0.2777778
            })
        }
    },
    {
        d4l3n33,
        {
            /* dim = */ 4,
            /* n = */ 33,
            /* x = */ std::vector<double>
            ({
                0.1127017, 0.1127017, 0.5, 0.5, 
                0.1127017, 0.5, 0.1127017, 0.5, 
                0.1127017, 0.5, 0.5, 0.1127017, 
                0.1127017, 0.5, 0.5, 0.5, 
                0.1127017, 0.5, 0.5, 0.8872983, 
                0.1127017, 0.5, 0.8872983, 0.5, 
                0.1127017, 0.8872983, 0.5, 0.5, 
                0.5, 0.1127017, 0.1127017, 0.5, 
                0.5, 0.1127017, 0.5, 0.1127017, 
                0.5, 0.1127017, 0.5, 0.5, 
                0.5, 0.1127017, 0.5, 0.8872983, 
                0.5, 0.1127017, 0.8872983, 0.5, 
                0.5, 0.5, 0.1127017, 0.1127017, 
                0.5, 0.5, 0.1127017, 0.5, 
                0.5, 0.5, 0.1127017, 0.8872983, 
                0.5, 0.5, 0.5, 0.1127017, 
                0.5, 0.5, 0.5, 0.5, 
                0.5, 0.5, 0.5, 0.8872983, 
                0.5, 0.5, 0.8872983, 0.1127017, 
                0.5, 0.5, 0.8872983, 0.5, 
                0.5, 0.5, 0.8872983, 0.8872983, 
                0.5, 0.8872983, 0.1127017, 0.5, 
                0.5, 0.8872983, 0.5, 0.1127017, 
                0.5, 0.8872983, 0.5, 0.5, 
                0.5, 0.8872983, 0.5, 0.8872983, 
                0.5, 0.8872983, 0.8872983, 0.5, 
                0.8872983, 0.1127017, 0.5, 0.5, 
                0.8872983, 0.5, 0.1127017, 0.5, 
                0.8872983, 0.5, 0.5, 0.1127017, 
                0.8872983, 0.5, 0.5, 0.5, 
                0.8872983, 0.5, 0.5, 0.8872983, 
                0.8872983, 0.5, 0.8872983, 0.5, 
                0.8872983, 0.8872983, 0.5, 0.5
            }),
            /* w = */ std::vector<double>
            ({
                 0.0771605061728401, 
                 0.0771605061728401, 
                 0.0771605061728401, 
                -0.18518523703704, 
                 0.0771605061728401, 
                 0.0771605061728401, 
                 0.0771605061728401, 
                 0.0771605061728401, 
                 0.0771605061728401, 
                -0.18518523703704, 
                 0.0771605061728401, 
                 0.0771605061728401, 
                 0.0771605061728401, 
                -0.18518523703704, 
                 0.0771605061728401, 
                -0.18518523703704, 
                 0.629629748148161, 
                -0.18518523703704, 
                 0.0771605061728401, 
                -0.18518523703704, 
                 0.0771605061728401, 
                 0.0771605061728401, 
                 0.0771605061728401, 
                -0.18518523703704, 
                 0.0771605061728401, 
                 0.0771605061728401, 
                 0.0771605061728401, 
                 0.0771605061728401, 
                 0.0771605061728401, 
                -0.18518523703704, 
                 0.0771605061728401, 
                 0.0771605061728401, 
                 0.0771605061728401
            })
        }
    },
    {
        d4l4n81,
        {
            /* dim = */ 4,
            /* n = */ 81,
            /* x = */ std::vector<double>
            ({
                0.0197543999999999, 0.5, 0.5, 0.5, 
                0.1127017, 0.1127017, 0.1127017, 0.5, 
                0.1127017, 0.1127017, 0.5, 0.1127017, 
                0.1127017, 0.1127017, 0.5, 0.5, 
                0.1127017, 0.1127017, 0.5, 0.8872983, 
                0.1127017, 0.1127017, 0.8872983, 0.5, 
                0.1127017, 0.5, 0.1127017, 0.1127017, 
                0.1127017, 0.5, 0.1127017, 0.5, 
                0.1127017, 0.5, 0.1127017, 0.8872983, 
                0.1127017, 0.5, 0.5, 0.1127017, 
                0.1127017, 0.5, 0.5, 0.5, 
                0.1127017, 0.5, 0.5, 0.8872983, 
                0.1127017, 0.5, 0.8872983, 0.1127017, 
                0.1127017, 0.5, 0.8872983, 0.5, 
                0.1127017, 0.5, 0.8872983, 0.8872983, 
                0.1127017, 0.8872983, 0.1127017, 0.5, 
                0.1127017, 0.8872983, 0.5, 0.1127017, 
                0.1127017, 0.8872983, 0.5, 0.5, 
                0.1127017, 0.8872983, 0.5, 0.8872983, 
                0.1127017, 0.8872983, 0.8872983, 0.5, 
                0.2828781, 0.5, 0.5, 0.5, 
                0.5, 0.0197543999999999, 0.5, 0.5, 
                0.5, 0.1127017, 0.1127017, 0.1127017, 
                0.5, 0.1127017, 0.1127017, 0.5, 
                0.5, 0.1127017, 0.1127017, 0.8872983, 
                0.5, 0.1127017, 0.5, 0.1127017, 
                0.5, 0.1127017, 0.5, 0.5, 
                0.5, 0.1127017, 0.5, 0.8872983, 
                0.5, 0.1127017, 0.8872983, 0.1127017, 
                0.5, 0.1127017, 0.8872983, 0.5, 
                0.5, 0.1127017, 0.8872983, 0.8872983, 
                0.5, 0.2828781, 0.5, 0.5, 
                0.5, 0.5, 0.0197543999999999, 0.5, 
                0.5, 0.5, 0.1127017, 0.1127017, 
                0.5, 0.5, 0.1127017, 0.5, 
                0.5, 0.5, 0.1127017, 0.8872983, 
                0.5, 0.5, 0.2828781, 0.5, 
                0.5, 0.5, 0.5, 0.0197543999999999, 
                0.5, 0.5, 0.5, 0.1127017, 
                0.5, 0.5, 0.5, 0.2828781, 
                0.5, 0.5, 0.5, 0.5, 
                0.5, 0.5, 0.5, 0.7171219, 
                0.5, 0.5, 0.5, 0.8872983, 
                0.5, 0.5, 0.5, 0.9802456, 
                0.5, 0.5, 0.7171219, 0.5, 
                0.5, 0.5, 0.8872983, 0.1127017, 
                0.5, 0.5, 0.8872983, 0.5, 
                0.5, 0.5, 0.8872983, 0.8872983, 
                0.5, 0.5, 0.9802456, 0.5, 
                0.5, 0.7171219, 0.5, 0.5, 
                0.5, 0.8872983, 0.1127017, 0.1127017, 
                0.5, 0.8872983, 0.1127017, 0.5, 
                0.5, 0.8872983, 0.1127017, 0.8872983, 
                0.5, 0.8872983, 0.5, 0.1127017, 
                0.5, 0.8872983, 0.5, 0.5, 
                0.5, 0.8872983, 0.5, 0.8872983, 
                0.5, 0.8872983, 0.8872983, 0.1127017, 
                0.5, 0.8872983, 0.8872983, 0.5, 
                0.5, 0.8872983, 0.8872983, 0.8872983, 
                0.5, 0.9802456, 0.5, 0.5, 
                0.7171219, 0.5, 0.5, 0.5, 
                0.8872983, 0.1127017, 0.1127017, 0.5, 
                0.8872983, 0.1127017, 0.5, 0.1127017, 
                0.8872983, 0.1127017, 0.5, 0.5, 
                0.8872983, 0.1127017, 0.5, 0.8872983, 
                0.8872983, 0.1127017, 0.8872983, 0.5, 
                0.8872983, 0.5, 0.1127017, 0.1127017, 
                0.8872983, 0.5, 0.1127017, 0.5, 
                0.8872983, 0.5, 0.1127017, 0.8872983, 
                0.8872983, 0.5, 0.5, 0.1127017, 
                0.8872983, 0.5, 0.5, 0.5, 
                0.8872983, 0.5, 0.5, 0.8872983, 
                0.8872983, 0.5, 0.8872983, 0.1127017, 
                0.8872983, 0.5, 0.8872983, 0.5, 
                0.8872983, 0.5, 0.8872983, 0.8872983, 
                0.8872983, 0.8872983, 0.1127017, 0.5, 
                0.8872983, 0.8872983, 0.5, 0.1127017, 
                0.8872983, 0.8872983, 0.5, 0.5, 
                0.8872983, 0.8872983, 0.5, 0.8872983, 
                0.8872983, 0.8872983, 0.8872983, 0.5, 
                0.9802456, 0.5, 0.5, 0.5
            }),
            /* w = */ std::vector<double>
            ({
                 0.0523281052328105, 
                 0.0214334756515779, 
                 0.0214334756515779, 
                -0.00857339643347166, 
                 0.0214334756515779, 
                 0.0214334756515779, 
                 0.0214334756515779, 
                -0.00857339643347166, 
                 0.0214334756515779, 
                -0.00857339643347166, 
                -0.0715173157937036, 
                -0.00857339643347166, 
                 0.0214334756515779, 
                -0.00857339643347166, 
                 0.0214334756515779, 
                 0.0214334756515779, 
                 0.0214334756515779, 
                -0.00857339643347166, 
                 0.0214334756515779, 
                 0.0214334756515779, 
                 0.200698720069872, 
                 0.0523281052328105, 
                 0.0214334756515779, 
                -0.00857339643347164, 
                 0.0214334756515779, 
                -0.00857339643347166, 
                -0.0715173157937037, 
                -0.00857339643347166, 
                 0.0214334756515779, 
                -0.00857339643347164, 
                 0.0214334756515779, 
                 0.200698720069872, 
                 0.0523281052328105, 
                -0.00857339643347169, 
                -0.0715173157937038, 
                -0.00857339643347169, 
                 0.200698720069872, 
                 0.0523281052328105, 
                -0.0715173157937035, 
                 0.200698720069872, 
                -0.932185782519005, 
                 0.200698720069872, 
                -0.0715173157937035, 
                 0.0523281052328105, 
                 0.200698720069872, 
                -0.00857339643347169, 
                -0.0715173157937038, 
                -0.00857339643347169, 
                 0.0523281052328105, 
                 0.200698720069872, 
                 0.0214334756515779, 
                -0.00857339643347164, 
                 0.0214334756515779, 
                -0.00857339643347166, 
                -0.0715173157937037, 
                -0.00857339643347166, 
                 0.0214334756515779, 
                -0.00857339643347164, 
                 0.0214334756515779, 
                 0.0523281052328105, 
                 0.200698720069872, 
                 0.0214334756515779, 
                 0.0214334756515779, 
                -0.00857339643347166, 
                 0.0214334756515779, 
                 0.0214334756515779, 
                 0.0214334756515779, 
                -0.00857339643347166, 
                 0.0214334756515779, 
                -0.00857339643347166, 
                -0.0715173157937036, 
                -0.00857339643347166, 
                 0.0214334756515779, 
                -0.00857339643347166, 
                 0.0214334756515779, 
                 0.0214334756515779, 
                 0.0214334756515779, 
                -0.00857339643347166, 
                 0.0214334756515779, 
                 0.0214334756515779, 
                 0.0523281052328105
            })
        }
    },
    {
        d4l5n193,
        {
            /* dim = */ 4,
            /* n = */ 193,
            /* x = */ std::vector<double>
            ({
                0.0197543999999999, 0.1127017, 0.5, 0.5, 
                0.0197543999999999, 0.5, 0.1127017, 0.5, 
                0.0197543999999999, 0.5, 0.5, 0.1127017, 
                0.0197543999999999, 0.5, 0.5, 0.5, 
                0.0197543999999999, 0.5, 0.5, 0.8872983, 
                0.0197543999999999, 0.5, 0.8872983, 0.5, 
                0.0197543999999999, 0.8872983, 0.5, 0.5, 
                0.1127017, 0.0197543999999999, 0.5, 0.5, 
                0.1127017, 0.1127017, 0.1127017, 0.1127017, 
                0.1127017, 0.1127017, 0.1127017, 0.5, 
                0.1127017, 0.1127017, 0.1127017, 0.8872983, 
                0.1127017, 0.1127017, 0.5, 0.1127017, 
                0.1127017, 0.1127017, 0.5, 0.5, 
                0.1127017, 0.1127017, 0.5, 0.8872983, 
                0.1127017, 0.1127017, 0.8872983, 0.1127017, 
                0.1127017, 0.1127017, 0.8872983, 0.5, 
                0.1127017, 0.1127017, 0.8872983, 0.8872983, 
                0.1127017, 0.2828781, 0.5, 0.5, 
                0.1127017, 0.5, 0.0197543999999999, 0.5, 
                0.1127017, 0.5, 0.1127017, 0.1127017, 
                0.1127017, 0.5, 0.1127017, 0.5, 
                0.1127017, 0.5, 0.1127017, 0.8872983, 
                0.1127017, 0.5, 0.2828781, 0.5, 
                0.1127017, 0.5, 0.5, 0.0197543999999999, 
                0.1127017, 0.5, 0.5, 0.1127017, 
                0.1127017, 0.5, 0.5, 0.2828781, 
                0.1127017, 0.5, 0.5, 0.5, 
                0.1127017, 0.5, 0.5, 0.7171219, 
                0.1127017, 0.5, 0.5, 0.8872983, 
                0.1127017, 0.5, 0.5, 0.9802456, 
                0.1127017, 0.5, 0.7171219, 0.5, 
                0.1127017, 0.5, 0.8872983, 0.1127017, 
                0.1127017, 0.5, 0.8872983, 0.5, 
                0.1127017, 0.5, 0.8872983, 0.8872983, 
                0.1127017, 0.5, 0.9802456, 0.5, 
                0.1127017, 0.7171219, 0.5, 0.5, 
                0.1127017, 0.8872983, 0.1127017, 0.1127017, 
                0.1127017, 0.8872983, 0.1127017, 0.5, 
                0.1127017, 0.8872983, 0.1127017, 0.8872983, 
                0.1127017, 0.8872983, 0.5, 0.1127017, 
                0.1127017, 0.8872983, 0.5, 0.5, 
                0.1127017, 0.8872983, 0.5, 0.8872983, 
                0.1127017, 0.8872983, 0.8872983, 0.1127017, 
                0.1127017, 0.8872983, 0.8872983, 0.5, 
                0.1127017, 0.8872983, 0.8872983, 0.8872983, 
                0.1127017, 0.9802456, 0.5, 0.5, 
                0.2828781, 0.1127017, 0.5, 0.5, 
                0.2828781, 0.5, 0.1127017, 0.5, 
                0.2828781, 0.5, 0.5, 0.1127017, 
                0.2828781, 0.5, 0.5, 0.5, 
                0.2828781, 0.5, 0.5, 0.8872983, 
                0.2828781, 0.5, 0.8872983, 0.5, 
                0.2828781, 0.8872983, 0.5, 0.5, 
                0.5, 0.0197543999999999, 0.1127017, 0.5, 
                0.5, 0.0197543999999999, 0.5, 0.1127017, 
                0.5, 0.0197543999999999, 0.5, 0.5, 
                0.5, 0.0197543999999999, 0.5, 0.8872983, 
                0.5, 0.0197543999999999, 0.8872983, 0.5, 
                0.5, 0.1127017, 0.0197543999999999, 0.5, 
                0.5, 0.1127017, 0.1127017, 0.1127017, 
                0.5, 0.1127017, 0.1127017, 0.5, 
                0.5, 0.1127017, 0.1127017, 0.8872983, 
                0.5, 0.1127017, 0.2828781, 0.5, 
                0.5, 0.1127017, 0.5, 0.0197543999999999, 
                0.5, 0.1127017, 0.5, 0.1127017, 
                0.5, 0.1127017, 0.5, 0.2828781, 
                0.5, 0.1127017, 0.5, 0.5, 
                0.5, 0.1127017, 0.5, 0.7171219, 
                0.5, 0.1127017, 0.5, 0.8872983, 
                0.5, 0.1127017, 0.5, 0.9802456, 
                0.5, 0.1127017, 0.7171219, 0.5, 
                0.5, 0.1127017, 0.8872983, 0.1127017, 
                0.5, 0.1127017, 0.8872983, 0.5, 
                0.5, 0.1127017, 0.8872983, 0.8872983, 
                0.5, 0.1127017, 0.9802456, 0.5, 
                0.5, 0.2828781, 0.1127017, 0.5, 
                0.5, 0.2828781, 0.5, 0.1127017, 
                0.5, 0.2828781, 0.5, 0.5, 
                0.5, 0.2828781, 0.5, 0.8872983, 
                0.5, 0.2828781, 0.8872983, 0.5, 
                0.5, 0.5, 0.0197543999999999, 0.1127017, 
                0.5, 0.5, 0.0197543999999999, 0.5, 
                0.5, 0.5, 0.0197543999999999, 0.8872983, 
                0.5, 0.5, 0.1127017, 0.0197543999999999, 
                0.5, 0.5, 0.1127017, 0.1127017, 
                0.5, 0.5, 0.1127017, 0.2828781, 
                0.5, 0.5, 0.1127017, 0.5, 
                0.5, 0.5, 0.1127017, 0.7171219, 
                0.5, 0.5, 0.1127017, 0.8872983, 
                0.5, 0.5, 0.1127017, 0.9802456, 
                0.5, 0.5, 0.2828781, 0.1127017, 
                0.5, 0.5, 0.2828781, 0.5, 
                0.5, 0.5, 0.2828781, 0.8872983, 
                0.5, 0.5, 0.5, 0.0197543999999999, 
                0.5, 0.5, 0.5, 0.1127017, 
                0.5, 0.5, 0.5, 0.2828781, 
                0.5, 0.5, 0.5, 0.5, 
                0.5, 0.5, 0.5, 0.7171219, 
                0.5, 0.5, 0.5, 0.8872983, 
                0.5, 0.5, 0.5, 0.9802456, 
                0.5, 0.5, 0.7171219, 0.1127017, 
                0.5, 0.5, 0.7171219, 0.5, 
                0.5, 0.5, 0.7171219, 0.8872983, 
                0.5, 0.5, 0.8872983, 0.0197543999999999, 
                0.5, 0.5, 0.8872983, 0.1127017, 
                0.5, 0.5, 0.8872983, 0.2828781, 
                0.5, 0.5, 0.8872983, 0.5, 
                0.5, 0.5, 0.8872983, 0.7171219, 
                0.5, 0.5, 0.8872983, 0.8872983, 
                0.5, 0.5, 0.8872983, 0.9802456, 
                0.5, 0.5, 0.9802456, 0.1127017, 
                0.5, 0.5, 0.9802456, 0.5, 
                0.5, 0.5, 0.9802456, 0.8872983, 
                0.5, 0.7171219, 0.1127017, 0.5, 
                0.5, 0.7171219, 0.5, 0.1127017, 
                0.5, 0.7171219, 0.5, 0.5, 
                0.5, 0.7171219, 0.5, 0.8872983, 
                0.5, 0.7171219, 0.8872983, 0.5, 
                0.5, 0.8872983, 0.0197543999999999, 0.5, 
                0.5, 0.8872983, 0.1127017, 0.1127017, 
                0.5, 0.8872983, 0.1127017, 0.5, 
                0.5, 0.8872983, 0.1127017, 0.8872983, 
                0.5, 0.8872983, 0.2828781, 0.5, 
                0.5, 0.8872983, 0.5, 0.0197543999999999, 
                0.5, 0.8872983, 0.5, 0.1127017, 
                0.5, 0.8872983, 0.5, 0.2828781, 
                0.5, 0.8872983, 0.5, 0.5, 
                0.5, 0.8872983, 0.5, 0.7171219, 
                0.5, 0.8872983, 0.5, 0.8872983, 
                0.5, 0.8872983, 0.5, 0.9802456, 
                0.5, 0.8872983, 0.7171219, 0.5, 
                0.5, 0.8872983, 0.8872983, 0.1127017, 
                0.5, 0.8872983, 0.8872983, 0.5, 
                0.5, 0.8872983, 0.8872983, 0.8872983, 
                0.5, 0.8872983, 0.9802456, 0.5, 
                0.5, 0.9802456, 0.1127017, 0.5, 
                0.5, 0.9802456, 0.5, 0.1127017, 
                0.5, 0.9802456, 0.5, 0.5, 
                0.5, 0.9802456, 0.5, 0.8872983, 
                0.5, 0.9802456, 0.8872983, 0.5, 
                0.7171219, 0.1127017, 0.5, 0.5, 
                0.7171219, 0.5, 0.1127017, 0.5, 
                0.7171219, 0.5, 0.5, 0.1127017, 
                0.7171219, 0.5, 0.5, 0.5, 
                0.7171219, 0.5, 0.5, 0.8872983, 
                0.7171219, 0.5, 0.8872983, 0.5, 
                0.7171219, 0.8872983, 0.5, 0.5, 
                0.8872983, 0.0197543999999999, 0.5, 0.5, 
                0.8872983, 0.1127017, 0.1127017, 0.1127017, 
                0.8872983, 0.1127017, 0.1127017, 0.5, 
                0.8872983, 0.1127017, 0.1127017, 0.8872983, 
                0.8872983, 0.1127017, 0.5, 0.1127017, 
                0.8872983, 0.1127017, 0.5, 0.5, 
                0.8872983, 0.1127017, 0.5, 0.8872983, 
                0.8872983, 0.1127017, 0.8872983, 0.1127017, 
                0.8872983, 0.1127017, 0.8872983, 0.5, 
                0.8872983, 0.1127017, 0.8872983, 0.8872983, 
                0.8872983, 0.2828781, 0.5, 0.5, 
                0.8872983, 0.5, 0.0197543999999999, 0.5, 
                0.8872983, 0.5, 0.1127017, 0.1127017, 
                0.8872983, 0.5, 0.1127017, 0.5, 
                0.8872983, 0.5, 0.1127017, 0.8872983, 
                0.8872983, 0.5, 0.2828781, 0.5, 
                0.8872983, 0.5, 0.5, 0.0197543999999999, 
                0.8872983, 0.5, 0.5, 0.1127017, 
                0.8872983, 0.5, 0.5, 0.2828781, 
                0.8872983, 0.5, 0.5, 0.5, 
                0.8872983, 0.5, 0.5, 0.7171219, 
                0.8872983, 0.5, 0.5, 0.8872983, 
                0.8872983, 0.5, 0.5, 0.9802456, 
                0.8872983, 0.5, 0.7171219, 0.5, 
                0.8872983, 0.5, 0.8872983, 0.1127017, 
                0.8872983, 0.5, 0.8872983, 0.5, 
                0.8872983, 0.5, 0.8872983, 0.8872983, 
                0.8872983, 0.5, 0.9802456, 0.5, 
                0.8872983, 0.7171219, 0.5, 0.5, 
                0.8872983, 0.8872983, 0.1127017, 0.1127017, 
                0.8872983, 0.8872983, 0.1127017, 0.5, 
                0.8872983, 0.8872983, 0.1127017, 0.8872983, 
                0.8872983, 0.8872983, 0.5, 0.1127017, 
                0.8872983, 0.8872983, 0.5, 0.5, 
                0.8872983, 0.8872983, 0.5, 0.8872983, 
                0.8872983, 0.8872983, 0.8872983, 0.1127017, 
                0.8872983, 0.8872983, 0.8872983, 0.5, 
                0.8872983, 0.8872983, 0.8872983, 0.8872983, 
                0.8872983, 0.9802456, 0.5, 0.5, 
                0.9802456, 0.1127017, 0.5, 0.5, 
                0.9802456, 0.5, 0.1127017, 0.5, 
                0.9802456, 0.5, 0.5, 0.1127017, 
                0.9802456, 0.5, 0.5, 0.5, 
                0.9802456, 0.5, 0.5, 0.8872983, 
                0.9802456, 0.5, 0.8872983, 0.5, 
                0.9802456, 0.8872983, 0.5, 0.5
            }),
            /* w = */ std::vector<double>
            ({
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                -0.034885410465621, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.00595374371284887, 
                 0.00952598822588015, 
                 0.00595374371284887, 
                 0.00952598822588015, 
                -0.0644994205033547, 
                 0.00952598822588015, 
                 0.00595374371284887, 
                 0.00952598822588015, 
                 0.00595374371284887, 
                 0.0557496489238248, 
                 0.0145355859497386, 
                 0.00952598822588015, 
                -0.0644994205033548, 
                 0.00952598822588015, 
                 0.0557496489238248, 
                 0.0145355859497386, 
                -0.0644994205033548, 
                 0.0557496489238248, 
                -0.0624126812102034, 
                 0.0557496489238248, 
                -0.0644994205033548, 
                 0.0145355859497386, 
                 0.0557496489238248, 
                 0.00952598822588015, 
                -0.0644994205033548, 
                 0.00952598822588015, 
                 0.0145355859497386, 
                 0.0557496489238248, 
                 0.00595374371284887, 
                 0.00952598822588015, 
                 0.00595374371284887, 
                 0.00952598822588015, 
                -0.0644994205033547, 
                 0.00952598822588015, 
                 0.00595374371284887, 
                 0.00952598822588015, 
                 0.00595374371284887, 
                 0.0145355859497386, 
                 0.0557496489238248, 
                 0.0557496489238248, 
                 0.0557496489238248, 
                -0.133799173473077, 
                 0.0557496489238248, 
                 0.0557496489238248, 
                 0.0557496489238248, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                -0.034885410465621, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.00952598822588015, 
                -0.0644994205033547, 
                 0.00952598822588015, 
                 0.0557496489238248, 
                 0.0145355859497386, 
                -0.0644994205033548, 
                 0.0557496489238248, 
                -0.0624126812102033, 
                 0.0557496489238248, 
                -0.0644994205033548, 
                 0.0145355859497386, 
                 0.0557496489238248, 
                 0.00952598822588015, 
                -0.0644994205033547, 
                 0.00952598822588015, 
                 0.0145355859497386, 
                 0.0557496489238248, 
                 0.0557496489238248, 
                -0.133799173473077, 
                 0.0557496489238248, 
                 0.0557496489238248, 
                 0.0145355859497386, 
                -0.034885410465621, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                -0.0644994205033546, 
                 0.0557496489238248, 
                -0.062412681210204, 
                 0.0557496489238248, 
                -0.0644994205033546, 
                 0.0145355859497386, 
                 0.0557496489238248, 
                -0.133799173473077, 
                 0.0557496489238248, 
                -0.034885410465621, 
                -0.0624126812102037, 
                -0.133799173473077, 
                 0.622981416706939, 
                -0.133799173473077, 
                -0.0624126812102037, 
                -0.034885410465621, 
                 0.0557496489238248, 
                -0.133799173473077, 
                 0.0557496489238248, 
                 0.0145355859497386, 
                -0.0644994205033546, 
                 0.0557496489238248, 
                -0.062412681210204, 
                 0.0557496489238248, 
                -0.0644994205033546, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                -0.034885410465621, 
                 0.0145355859497386, 
                 0.0557496489238248, 
                 0.0557496489238248, 
                -0.133799173473077, 
                 0.0557496489238248, 
                 0.0557496489238248, 
                 0.0145355859497386, 
                 0.00952598822588015, 
                -0.0644994205033547, 
                 0.00952598822588015, 
                 0.0557496489238248, 
                 0.0145355859497386, 
                -0.0644994205033548, 
                 0.0557496489238248, 
                -0.0624126812102033, 
                 0.0557496489238248, 
                -0.0644994205033548, 
                 0.0145355859497386, 
                 0.0557496489238248, 
                 0.00952598822588015, 
                -0.0644994205033547, 
                 0.00952598822588015, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                -0.034885410465621, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0557496489238248, 
                 0.0557496489238248, 
                 0.0557496489238248, 
                -0.133799173473077, 
                 0.0557496489238248, 
                 0.0557496489238248, 
                 0.0557496489238248, 
                 0.0145355859497386, 
                 0.00595374371284887, 
                 0.00952598822588015, 
                 0.00595374371284887, 
                 0.00952598822588015, 
                -0.0644994205033547, 
                 0.00952598822588015, 
                 0.00595374371284887, 
                 0.00952598822588015, 
                 0.00595374371284887, 
                 0.0557496489238248, 
                 0.0145355859497386, 
                 0.00952598822588015, 
                -0.0644994205033548, 
                 0.00952598822588015, 
                 0.0557496489238248, 
                 0.0145355859497386, 
                -0.0644994205033548, 
                 0.0557496489238248, 
                -0.0624126812102034, 
                 0.0557496489238248, 
                -0.0644994205033548, 
                 0.0145355859497386, 
                 0.0557496489238248, 
                 0.00952598822588015, 
                -0.0644994205033548, 
                 0.00952598822588015, 
                 0.0145355859497386, 
                 0.0557496489238248, 
                 0.00595374371284887, 
                 0.00952598822588015, 
                 0.00595374371284887, 
                 0.00952598822588015, 
                -0.0644994205033547, 
                 0.00952598822588015, 
                 0.00595374371284887, 
                 0.00952598822588015, 
                 0.00595374371284887, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                -0.034885410465621, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0145355859497386
            })
        }
    },
    {
        d5l1n1,
        {
            /* dim = */ 5,
            /* n = */ 1,
            /* x = */ std::vector<double>
            ({
                0.5, 0.5, 0.5, 0.5, 0.5
            }),
            /* w = */ std::vector<double>
            ({
                 1
            })
        }
    },
    {
        d5l2n11,
        {
            /* dim = */ 5,
            /* n = */ 11,
            /* x = */ std::vector<double>
            ({
                0.1127017, 0.5, 0.5, 0.5, 0.5, 
                0.5, 0.1127017, 0.5, 0.5, 0.5, 
                0.5, 0.5, 0.1127017, 0.5, 0.5, 
                0.5, 0.5, 0.5, 0.1127017, 0.5, 
                0.5, 0.5, 0.5, 0.5, 0.1127017, 
                0.5, 0.5, 0.5, 0.5, 0.5, 
                0.5, 0.5, 0.5, 0.5, 0.8872983, 
                0.5, 0.5, 0.5, 0.8872983, 0.5, 
                0.5, 0.5, 0.8872983, 0.5, 0.5, 
                0.5, 0.8872983, 0.5, 0.5, 0.5, 
                0.8872983, 0.5, 0.5, 0.5, 0.5
            }),
            /* w = */ std::vector<double>
            ({
                 0.2777778, 
                 0.2777778, 
                 0.2777778, 
                 0.2777778, 
                 0.2777778, 
                -1.777778, 
                 0.2777778, 
                 0.2777778, 
                 0.2777778, 
                 0.2777778, 
                 0.2777778
            })
        }
    },
    {
        d5l3n51,
        {
            /* dim = */ 5,
            /* n = */ 51,
            /* x = */ std::vector<double>
            ({
                0.1127017, 0.1127017, 0.5, 0.5, 0.5, 
                0.1127017, 0.5, 0.1127017, 0.5, 0.5, 
                0.1127017, 0.5, 0.5, 0.1127017, 0.5, 
                0.1127017, 0.5, 0.5, 0.5, 0.1127017, 
                0.1127017, 0.5, 0.5, 0.5, 0.5, 
                0.1127017, 0.5, 0.5, 0.5, 0.8872983, 
                0.1127017, 0.5, 0.5, 0.8872983, 0.5, 
                0.1127017, 0.5, 0.8872983, 0.5, 0.5, 
                0.1127017, 0.8872983, 0.5, 0.5, 0.5, 
                0.5, 0.1127017, 0.1127017, 0.5, 0.5, 
                0.5, 0.1127017, 0.5, 0.1127017, 0.5, 
                0.5, 0.1127017, 0.5, 0.5, 0.1127017, 
                0.5, 0.1127017, 0.5, 0.5, 0.5, 
                0.5, 0.1127017, 0.5, 0.5, 0.8872983, 
                0.5, 0.1127017, 0.5, 0.8872983, 0.5, 
                0.5, 0.1127017, 0.8872983, 0.5, 0.5, 
                0.5, 0.5, 0.1127017, 0.1127017, 0.5, 
                0.5, 0.5, 0.1127017, 0.5, 0.1127017, 
                0.5, 0.5, 0.1127017, 0.5, 0.5, 
                0.5, 0.5, 0.1127017, 0.5, 0.8872983, 
                0.5, 0.5, 0.1127017, 0.8872983, 0.5, 
                0.5, 0.5, 0.5, 0.1127017, 0.1127017, 
                0.5, 0.5, 0.5, 0.1127017, 0.5, 
                0.5, 0.5, 0.5, 0.1127017, 0.8872983, 
                0.5, 0.5, 0.5, 0.5, 0.1127017, 
                0.5, 0.5, 0.5, 0.5, 0.5, 
                0.5, 0.5, 0.5, 0.5, 0.8872983, 
                0.5, 0.5, 0.5, 0.8872983, 0.1127017, 
                0.5, 0.5, 0.5, 0.8872983, 0.5, 
                0.5, 0.5, 0.5, 0.8872983, 0.8872983, 
                0.5, 0.5, 0.8872983, 0.1127017, 0.5, 
                0.5, 0.5, 0.8872983, 0.5, 0.1127017, 
                0.5, 0.5, 0.8872983, 0.5, 0.5, 
                0.5, 0.5, 0.8872983, 0.5, 0.8872983, 
                0.5, 0.5, 0.8872983, 0.8872983, 0.5, 
                0.5, 0.8872983, 0.1127017, 0.5, 0.5, 
                0.5, 0.8872983, 0.5, 0.1127017, 0.5, 
                0.5, 0.8872983, 0.5, 0.5, 0.1127017, 
                0.5, 0.8872983, 0.5, 0.5, 0.5, 
                0.5, 0.8872983, 0.5, 0.5, 0.8872983, 
                0.5, 0.8872983, 0.5, 0.8872983, 0.5, 
                0.5, 0.8872983, 0.8872983, 0.5, 0.5, 
                0.8872983, 0.1127017, 0.5, 0.5, 0.5, 
                0.8872983, 0.5, 0.1127017, 0.5, 0.5, 
                0.8872983, 0.5, 0.5, 0.1127017, 0.5, 
                0.8872983, 0.5, 0.5, 0.5, 0.1127017, 
                0.8872983, 0.5, 0.5, 0.5, 0.5, 
                0.8872983, 0.5, 0.5, 0.5, 0.8872983, 
                0.8872983, 0.5, 0.5, 0.8872983, 0.5, 
                0.8872983, 0.5, 0.8872983, 0.5, 0.5, 
                0.8872983, 0.8872983, 0.5, 0.5, 0.5
            }),
            /* w = */ std::vector<double>
            ({
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                -0.33950624938272, 
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                -0.33950624938272, 
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                -0.33950624938272, 
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                -0.33950624938272, 
                 0.07716050617284, 
                -0.33950624938272, 
                 1.3086422469136, 
                -0.33950624938272, 
                 0.07716050617284, 
                -0.33950624938272, 
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                -0.33950624938272, 
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                -0.33950624938272, 
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                -0.33950624938272, 
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284
            })
        }
    },
    {
        d5l4n151,
        {
            /* dim = */ 5,
            /* n = */ 151,
            /* x = */ std::vector<double>
            ({
                0.0197543999999999, 0.5, 0.5, 0.5, 0.5, 
                0.1127017, 0.1127017, 0.1127017, 0.5, 0.5, 
                0.1127017, 0.1127017, 0.5, 0.1127017, 0.5, 
                0.1127017, 0.1127017, 0.5, 0.5, 0.1127017, 
                0.1127017, 0.1127017, 0.5, 0.5, 0.5, 
                0.1127017, 0.1127017, 0.5, 0.5, 0.8872983, 
                0.1127017, 0.1127017, 0.5, 0.8872983, 0.5, 
                0.1127017, 0.1127017, 0.8872983, 0.5, 0.5, 
                0.1127017, 0.5, 0.1127017, 0.1127017, 0.5, 
                0.1127017, 0.5, 0.1127017, 0.5, 0.1127017, 
                0.1127017, 0.5, 0.1127017, 0.5, 0.5, 
                0.1127017, 0.5, 0.1127017, 0.5, 0.8872983, 
                0.1127017, 0.5, 0.1127017, 0.8872983, 0.5, 
                0.1127017, 0.5, 0.5, 0.1127017, 0.1127017, 
                0.1127017, 0.5, 0.5, 0.1127017, 0.5, 
                0.1127017, 0.5, 0.5, 0.1127017, 0.8872983, 
                0.1127017, 0.5, 0.5, 0.5, 0.1127017, 
                0.1127017, 0.5, 0.5, 0.5, 0.5, 
                0.1127017, 0.5, 0.5, 0.5, 0.8872983, 
                0.1127017, 0.5, 0.5, 0.8872983, 0.1127017, 
                0.1127017, 0.5, 0.5, 0.8872983, 0.5, 
                0.1127017, 0.5, 0.5, 0.8872983, 0.8872983, 
                0.1127017, 0.5, 0.8872983, 0.1127017, 0.5, 
                0.1127017, 0.5, 0.8872983, 0.5, 0.1127017, 
                0.1127017, 0.5, 0.8872983, 0.5, 0.5, 
                0.1127017, 0.5, 0.8872983, 0.5, 0.8872983, 
                0.1127017, 0.5, 0.8872983, 0.8872983, 0.5, 
                0.1127017, 0.8872983, 0.1127017, 0.5, 0.5, 
                0.1127017, 0.8872983, 0.5, 0.1127017, 0.5, 
                0.1127017, 0.8872983, 0.5, 0.5, 0.1127017, 
                0.1127017, 0.8872983, 0.5, 0.5, 0.5, 
                0.1127017, 0.8872983, 0.5, 0.5, 0.8872983, 
                0.1127017, 0.8872983, 0.5, 0.8872983, 0.5, 
                0.1127017, 0.8872983, 0.8872983, 0.5, 0.5, 
                0.2828781, 0.5, 0.5, 0.5, 0.5, 
                0.5, 0.0197543999999999, 0.5, 0.5, 0.5, 
                0.5, 0.1127017, 0.1127017, 0.1127017, 0.5, 
                0.5, 0.1127017, 0.1127017, 0.5, 0.1127017, 
                0.5, 0.1127017, 0.1127017, 0.5, 0.5, 
                0.5, 0.1127017, 0.1127017, 0.5, 0.8872983, 
                0.5, 0.1127017, 0.1127017, 0.8872983, 0.5, 
                0.5, 0.1127017, 0.5, 0.1127017, 0.1127017, 
                0.5, 0.1127017, 0.5, 0.1127017, 0.5, 
                0.5, 0.1127017, 0.5, 0.1127017, 0.8872983, 
                0.5, 0.1127017, 0.5, 0.5, 0.1127017, 
                0.5, 0.1127017, 0.5, 0.5, 0.5, 
                0.5, 0.1127017, 0.5, 0.5, 0.8872983, 
                0.5, 0.1127017, 0.5, 0.8872983, 0.1127017, 
                0.5, 0.1127017, 0.5, 0.8872983, 0.5, 
                0.5, 0.1127017, 0.5, 0.8872983, 0.8872983, 
                0.5, 0.1127017, 0.8872983, 0.1127017, 0.5, 
                0.5, 0.1127017, 0.8872983, 0.5, 0.1127017, 
                0.5, 0.1127017, 0.8872983, 0.5, 0.5, 
                0.5, 0.1127017, 0.8872983, 0.5, 0.8872983, 
                0.5, 0.1127017, 0.8872983, 0.8872983, 0.5, 
                0.5, 0.2828781, 0.5, 0.5, 0.5, 
                0.5, 0.5, 0.0197543999999999, 0.5, 0.5, 
                0.5, 0.5, 0.1127017, 0.1127017, 0.1127017, 
                0.5, 0.5, 0.1127017, 0.1127017, 0.5, 
                0.5, 0.5, 0.1127017, 0.1127017, 0.8872983, 
                0.5, 0.5, 0.1127017, 0.5, 0.1127017, 
                0.5, 0.5, 0.1127017, 0.5, 0.5, 
                0.5, 0.5, 0.1127017, 0.5, 0.8872983, 
                0.5, 0.5, 0.1127017, 0.8872983, 0.1127017, 
                0.5, 0.5, 0.1127017, 0.8872983, 0.5, 
                0.5, 0.5, 0.1127017, 0.8872983, 0.8872983, 
                0.5, 0.5, 0.2828781, 0.5, 0.5, 
                0.5, 0.5, 0.5, 0.0197543999999999, 0.5, 
                0.5, 0.5, 0.5, 0.1127017, 0.1127017, 
                0.5, 0.5, 0.5, 0.1127017, 0.5, 
                0.5, 0.5, 0.5, 0.1127017, 0.8872983, 
                0.5, 0.5, 0.5, 0.2828781, 0.5, 
                0.5, 0.5, 0.5, 0.5, 0.0197543999999999, 
                0.5, 0.5, 0.5, 0.5, 0.1127017, 
                0.5, 0.5, 0.5, 0.5, 0.2828781, 
                0.5, 0.5, 0.5, 0.5, 0.5, 
                0.5, 0.5, 0.5, 0.5, 0.7171219, 
                0.5, 0.5, 0.5, 0.5, 0.8872983, 
                0.5, 0.5, 0.5, 0.5, 0.9802456, 
                0.5, 0.5, 0.5, 0.7171219, 0.5, 
                0.5, 0.5, 0.5, 0.8872983, 0.1127017, 
                0.5, 0.5, 0.5, 0.8872983, 0.5, 
                0.5, 0.5, 0.5, 0.8872983, 0.8872983, 
                0.5, 0.5, 0.5, 0.9802456, 0.5, 
                0.5, 0.5, 0.7171219, 0.5, 0.5, 
                0.5, 0.5, 0.8872983, 0.1127017, 0.1127017, 
                0.5, 0.5, 0.8872983, 0.1127017, 0.5, 
                0.5, 0.5, 0.8872983, 0.1127017, 0.8872983, 
                0.5, 0.5, 0.8872983, 0.5, 0.1127017, 
                0.5, 0.5, 0.8872983, 0.5, 0.5, 
                0.5, 0.5, 0.8872983, 0.5, 0.8872983, 
                0.5, 0.5, 0.8872983, 0.8872983, 0.1127017, 
                0.5, 0.5, 0.8872983, 0.8872983, 0.5, 
                0.5, 0.5, 0.8872983, 0.8872983, 0.8872983, 
                0.5, 0.5, 0.9802456, 0.5, 0.5, 
                0.5, 0.7171219, 0.5, 0.5, 0.5, 
                0.5, 0.8872983, 0.1127017, 0.1127017, 0.5, 
                0.5, 0.8872983, 0.1127017, 0.5, 0.1127017, 
                0.5, 0.8872983, 0.1127017, 0.5, 0.5, 
                0.5, 0.8872983, 0.1127017, 0.5, 0.8872983, 
                0.5, 0.8872983, 0.1127017, 0.8872983, 0.5, 
                0.5, 0.8872983, 0.5, 0.1127017, 0.1127017, 
                0.5, 0.8872983, 0.5, 0.1127017, 0.5, 
                0.5, 0.8872983, 0.5, 0.1127017, 0.8872983, 
                0.5, 0.8872983, 0.5, 0.5, 0.1127017, 
                0.5, 0.8872983, 0.5, 0.5, 0.5, 
                0.5, 0.8872983, 0.5, 0.5, 0.8872983, 
                0.5, 0.8872983, 0.5, 0.8872983, 0.1127017, 
                0.5, 0.8872983, 0.5, 0.8872983, 0.5, 
                0.5, 0.8872983, 0.5, 0.8872983, 0.8872983, 
                0.5, 0.8872983, 0.8872983, 0.1127017, 0.5, 
                0.5, 0.8872983, 0.8872983, 0.5, 0.1127017, 
                0.5, 0.8872983, 0.8872983, 0.5, 0.5, 
                0.5, 0.8872983, 0.8872983, 0.5, 0.8872983, 
                0.5, 0.8872983, 0.8872983, 0.8872983, 0.5, 
                0.5, 0.9802456, 0.5, 0.5, 0.5, 
                0.7171219, 0.5, 0.5, 0.5, 0.5, 
                0.8872983, 0.1127017, 0.1127017, 0.5, 0.5, 
                0.8872983, 0.1127017, 0.5, 0.1127017, 0.5, 
                0.8872983, 0.1127017, 0.5, 0.5, 0.1127017, 
                0.8872983, 0.1127017, 0.5, 0.5, 0.5, 
                0.8872983, 0.1127017, 0.5, 0.5, 0.8872983, 
                0.8872983, 0.1127017, 0.5, 0.8872983, 0.5, 
                0.8872983, 0.1127017, 0.8872983, 0.5, 0.5, 
                0.8872983, 0.5, 0.1127017, 0.1127017, 0.5, 
                0.8872983, 0.5, 0.1127017, 0.5, 0.1127017, 
                0.8872983, 0.5, 0.1127017, 0.5, 0.5, 
                0.8872983, 0.5, 0.1127017, 0.5, 0.8872983, 
                0.8872983, 0.5, 0.1127017, 0.8872983, 0.5, 
                0.8872983, 0.5, 0.5, 0.1127017, 0.1127017, 
                0.8872983, 0.5, 0.5, 0.1127017, 0.5, 
                0.8872983, 0.5, 0.5, 0.1127017, 0.8872983, 
                0.8872983, 0.5, 0.5, 0.5, 0.1127017, 
                0.8872983, 0.5, 0.5, 0.5, 0.5, 
                0.8872983, 0.5, 0.5, 0.5, 0.8872983, 
                0.8872983, 0.5, 0.5, 0.8872983, 0.1127017, 
                0.8872983, 0.5, 0.5, 0.8872983, 0.5, 
                0.8872983, 0.5, 0.5, 0.8872983, 0.8872983, 
                0.8872983, 0.5, 0.8872983, 0.1127017, 0.5, 
                0.8872983, 0.5, 0.8872983, 0.5, 0.1127017, 
                0.8872983, 0.5, 0.8872983, 0.5, 0.5, 
                0.8872983, 0.5, 0.8872983, 0.5, 0.8872983, 
                0.8872983, 0.5, 0.8872983, 0.8872983, 0.5, 
                0.8872983, 0.8872983, 0.1127017, 0.5, 0.5, 
                0.8872983, 0.8872983, 0.5, 0.1127017, 0.5, 
                0.8872983, 0.8872983, 0.5, 0.5, 0.1127017, 
                0.8872983, 0.8872983, 0.5, 0.5, 0.5, 
                0.8872983, 0.8872983, 0.5, 0.5, 0.8872983, 
                0.8872983, 0.8872983, 0.5, 0.8872983, 0.5, 
                0.8872983, 0.8872983, 0.8872983, 0.5, 0.5, 
                0.9802456, 0.5, 0.5, 0.5, 0.5
            }),
            /* w = */ std::vector<double>
            ({
                 0.0523281052328106, 
                 0.0214334756515779, 
                 0.0214334756515779, 
                 0.0214334756515779, 
                -0.0514403477366276, 
                 0.0214334756515779, 
                 0.0214334756515779, 
                 0.0214334756515779, 
                 0.0214334756515779, 
                 0.0214334756515779, 
                -0.0514403477366276, 
                 0.0214334756515779, 
                 0.0214334756515779, 
                 0.0214334756515779, 
                -0.0514403477366276, 
                 0.0214334756515779, 
                -0.0514403477366276, 
                 0.0313633796795514, 
                -0.0514403477366276, 
                 0.0214334756515779, 
                -0.0514403477366276, 
                 0.0214334756515779, 
                 0.0214334756515779, 
                 0.0214334756515779, 
                -0.0514403477366276, 
                 0.0214334756515779, 
                 0.0214334756515779, 
                 0.0214334756515779, 
                 0.0214334756515779, 
                 0.0214334756515779, 
                -0.0514403477366276, 
                 0.0214334756515779, 
                 0.0214334756515779, 
                 0.0214334756515779, 
                 0.200698720069872, 
                 0.0523281052328106, 
                 0.0214334756515779, 
                 0.0214334756515779, 
                -0.0514403477366276, 
                 0.0214334756515779, 
                 0.0214334756515779, 
                 0.0214334756515779, 
                -0.0514403477366276, 
                 0.0214334756515779, 
                -0.0514403477366276, 
                 0.0313633796795514, 
                -0.0514403477366276, 
                 0.0214334756515779, 
                -0.0514403477366276, 
                 0.0214334756515779, 
                 0.0214334756515779, 
                 0.0214334756515779, 
                -0.0514403477366276, 
                 0.0214334756515779, 
                 0.0214334756515779, 
                 0.200698720069872, 
                 0.0523281052328106, 
                 0.0214334756515779, 
                -0.0514403477366275, 
                 0.0214334756515779, 
                -0.0514403477366276, 
                 0.031363379679551, 
                -0.0514403477366276, 
                 0.0214334756515779, 
                -0.0514403477366275, 
                 0.0214334756515779, 
                 0.200698720069872, 
                 0.0523281052328106, 
                -0.0514403477366276, 
                 0.0313633796795514, 
                -0.0514403477366276, 
                 0.200698720069872, 
                 0.0523281052328106, 
                 0.0313633796795517, 
                 0.200698720069872, 
                -1.50096619248348, 
                 0.200698720069872, 
                 0.0313633796795517, 
                 0.0523281052328106, 
                 0.200698720069872, 
                -0.0514403477366276, 
                 0.0313633796795514, 
                -0.0514403477366276, 
                 0.0523281052328106, 
                 0.200698720069872, 
                 0.0214334756515779, 
                -0.0514403477366275, 
                 0.0214334756515779, 
                -0.0514403477366276, 
                 0.031363379679551, 
                -0.0514403477366276, 
                 0.0214334756515779, 
                -0.0514403477366275, 
                 0.0214334756515779, 
                 0.0523281052328106, 
                 0.200698720069872, 
                 0.0214334756515779, 
                 0.0214334756515779, 
                -0.0514403477366276, 
                 0.0214334756515779, 
                 0.0214334756515779, 
                 0.0214334756515779, 
                -0.0514403477366276, 
                 0.0214334756515779, 
                -0.0514403477366276, 
                 0.0313633796795514, 
                -0.0514403477366276, 
                 0.0214334756515779, 
                -0.0514403477366276, 
                 0.0214334756515779, 
                 0.0214334756515779, 
                 0.0214334756515779, 
                -0.0514403477366276, 
                 0.0214334756515779, 
                 0.0214334756515779, 
                 0.0523281052328106, 
                 0.200698720069872, 
                 0.0214334756515779, 
                 0.0214334756515779, 
                 0.0214334756515779, 
                -0.0514403477366276, 
                 0.0214334756515779, 
                 0.0214334756515779, 
                 0.0214334756515779, 
                 0.0214334756515779, 
                 0.0214334756515779, 
                -0.0514403477366276, 
                 0.0214334756515779, 
                 0.0214334756515779, 
                 0.0214334756515779, 
                -0.0514403477366276, 
                 0.0214334756515779, 
                -0.0514403477366276, 
                 0.0313633796795514, 
                -0.0514403477366276, 
                 0.0214334756515779, 
                -0.0514403477366276, 
                 0.0214334756515779, 
                 0.0214334756515779, 
                 0.0214334756515779, 
                -0.0514403477366276, 
                 0.0214334756515779, 
                 0.0214334756515779, 
                 0.0214334756515779, 
                 0.0214334756515779, 
                 0.0214334756515779, 
                -0.0514403477366276, 
                 0.0214334756515779, 
                 0.0214334756515779, 
                 0.0214334756515779, 
                 0.0523281052328106
            })
        }
    },
    {
        d5l5n391,
        {
            /* dim = */ 5,
            /* n = */ 391,
            /* x = */ std::vector<double>
            ({
                0.0197543999999999, 0.1127017, 0.5, 0.5, 0.5, 
                0.0197543999999999, 0.5, 0.1127017, 0.5, 0.5, 
                0.0197543999999999, 0.5, 0.5, 0.1127017, 0.5, 
                0.0197543999999999, 0.5, 0.5, 0.5, 0.1127017, 
                0.0197543999999999, 0.5, 0.5, 0.5, 0.5, 
                0.0197543999999999, 0.5, 0.5, 0.5, 0.8872983, 
                0.0197543999999999, 0.5, 0.5, 0.8872983, 0.5, 
                0.0197543999999999, 0.5, 0.8872983, 0.5, 0.5, 
                0.0197543999999999, 0.8872983, 0.5, 0.5, 0.5, 
                0.1127017, 0.0197543999999999, 0.5, 0.5, 0.5, 
                0.1127017, 0.1127017, 0.1127017, 0.1127017, 0.5, 
                0.1127017, 0.1127017, 0.1127017, 0.5, 0.1127017, 
                0.1127017, 0.1127017, 0.1127017, 0.5, 0.5, 
                0.1127017, 0.1127017, 0.1127017, 0.5, 0.8872983, 
                0.1127017, 0.1127017, 0.1127017, 0.8872983, 0.5, 
                0.1127017, 0.1127017, 0.5, 0.1127017, 0.1127017, 
                0.1127017, 0.1127017, 0.5, 0.1127017, 0.5, 
                0.1127017, 0.1127017, 0.5, 0.1127017, 0.8872983, 
                0.1127017, 0.1127017, 0.5, 0.5, 0.1127017, 
                0.1127017, 0.1127017, 0.5, 0.5, 0.5, 
                0.1127017, 0.1127017, 0.5, 0.5, 0.8872983, 
                0.1127017, 0.1127017, 0.5, 0.8872983, 0.1127017, 
                0.1127017, 0.1127017, 0.5, 0.8872983, 0.5, 
                0.1127017, 0.1127017, 0.5, 0.8872983, 0.8872983, 
                0.1127017, 0.1127017, 0.8872983, 0.1127017, 0.5, 
                0.1127017, 0.1127017, 0.8872983, 0.5, 0.1127017, 
                0.1127017, 0.1127017, 0.8872983, 0.5, 0.5, 
                0.1127017, 0.1127017, 0.8872983, 0.5, 0.8872983, 
                0.1127017, 0.1127017, 0.8872983, 0.8872983, 0.5, 
                0.1127017, 0.2828781, 0.5, 0.5, 0.5, 
                0.1127017, 0.5, 0.0197543999999999, 0.5, 0.5, 
                0.1127017, 0.5, 0.1127017, 0.1127017, 0.1127017, 
                0.1127017, 0.5, 0.1127017, 0.1127017, 0.5, 
                0.1127017, 0.5, 0.1127017, 0.1127017, 0.8872983, 
                0.1127017, 0.5, 0.1127017, 0.5, 0.1127017, 
                0.1127017, 0.5, 0.1127017, 0.5, 0.5, 
                0.1127017, 0.5, 0.1127017, 0.5, 0.8872983, 
                0.1127017, 0.5, 0.1127017, 0.8872983, 0.1127017, 
                0.1127017, 0.5, 0.1127017, 0.8872983, 0.5, 
                0.1127017, 0.5, 0.1127017, 0.8872983, 0.8872983, 
                0.1127017, 0.5, 0.2828781, 0.5, 0.5, 
                0.1127017, 0.5, 0.5, 0.0197543999999999, 0.5, 
                0.1127017, 0.5, 0.5, 0.1127017, 0.1127017, 
                0.1127017, 0.5, 0.5, 0.1127017, 0.5, 
                0.1127017, 0.5, 0.5, 0.1127017, 0.8872983, 
                0.1127017, 0.5, 0.5, 0.2828781, 0.5, 
                0.1127017, 0.5, 0.5, 0.5, 0.0197543999999999, 
                0.1127017, 0.5, 0.5, 0.5, 0.1127017, 
                0.1127017, 0.5, 0.5, 0.5, 0.2828781, 
                0.1127017, 0.5, 0.5, 0.5, 0.5, 
                0.1127017, 0.5, 0.5, 0.5, 0.7171219, 
                0.1127017, 0.5, 0.5, 0.5, 0.8872983, 
                0.1127017, 0.5, 0.5, 0.5, 0.9802456, 
                0.1127017, 0.5, 0.5, 0.7171219, 0.5, 
                0.1127017, 0.5, 0.5, 0.8872983, 0.1127017, 
                0.1127017, 0.5, 0.5, 0.8872983, 0.5, 
                0.1127017, 0.5, 0.5, 0.8872983, 0.8872983, 
                0.1127017, 0.5, 0.5, 0.9802456, 0.5, 
                0.1127017, 0.5, 0.7171219, 0.5, 0.5, 
                0.1127017, 0.5, 0.8872983, 0.1127017, 0.1127017, 
                0.1127017, 0.5, 0.8872983, 0.1127017, 0.5, 
                0.1127017, 0.5, 0.8872983, 0.1127017, 0.8872983, 
                0.1127017, 0.5, 0.8872983, 0.5, 0.1127017, 
                0.1127017, 0.5, 0.8872983, 0.5, 0.5, 
                0.1127017, 0.5, 0.8872983, 0.5, 0.8872983, 
                0.1127017, 0.5, 0.8872983, 0.8872983, 0.1127017, 
                0.1127017, 0.5, 0.8872983, 0.8872983, 0.5, 
                0.1127017, 0.5, 0.8872983, 0.8872983, 0.8872983, 
                0.1127017, 0.5, 0.9802456, 0.5, 0.5, 
                0.1127017, 0.7171219, 0.5, 0.5, 0.5, 
                0.1127017, 0.8872983, 0.1127017, 0.1127017, 0.5, 
                0.1127017, 0.8872983, 0.1127017, 0.5, 0.1127017, 
                0.1127017, 0.8872983, 0.1127017, 0.5, 0.5, 
                0.1127017, 0.8872983, 0.1127017, 0.5, 0.8872983, 
                0.1127017, 0.8872983, 0.1127017, 0.8872983, 0.5, 
                0.1127017, 0.8872983, 0.5, 0.1127017, 0.1127017, 
                0.1127017, 0.8872983, 0.5, 0.1127017, 0.5, 
                0.1127017, 0.8872983, 0.5, 0.1127017, 0.8872983, 
                0.1127017, 0.8872983, 0.5, 0.5, 0.1127017, 
                0.1127017, 0.8872983, 0.5, 0.5, 0.5, 
                0.1127017, 0.8872983, 0.5, 0.5, 0.8872983, 
                0.1127017, 0.8872983, 0.5, 0.8872983, 0.1127017, 
                0.1127017, 0.8872983, 0.5, 0.8872983, 0.5, 
                0.1127017, 0.8872983, 0.5, 0.8872983, 0.8872983, 
                0.1127017, 0.8872983, 0.8872983, 0.1127017, 0.5, 
                0.1127017, 0.8872983, 0.8872983, 0.5, 0.1127017, 
                0.1127017, 0.8872983, 0.8872983, 0.5, 0.5, 
                0.1127017, 0.8872983, 0.8872983, 0.5, 0.8872983, 
                0.1127017, 0.8872983, 0.8872983, 0.8872983, 0.5, 
                0.1127017, 0.9802456, 0.5, 0.5, 0.5, 
                0.2828781, 0.1127017, 0.5, 0.5, 0.5, 
                0.2828781, 0.5, 0.1127017, 0.5, 0.5, 
                0.2828781, 0.5, 0.5, 0.1127017, 0.5, 
                0.2828781, 0.5, 0.5, 0.5, 0.1127017, 
                0.2828781, 0.5, 0.5, 0.5, 0.5, 
                0.2828781, 0.5, 0.5, 0.5, 0.8872983, 
                0.2828781, 0.5, 0.5, 0.8872983, 0.5, 
                0.2828781, 0.5, 0.8872983, 0.5, 0.5, 
                0.2828781, 0.8872983, 0.5, 0.5, 0.5, 
                0.5, 0.0197543999999999, 0.1127017, 0.5, 0.5, 
                0.5, 0.0197543999999999, 0.5, 0.1127017, 0.5, 
                0.5, 0.0197543999999999, 0.5, 0.5, 0.1127017, 
                0.5, 0.0197543999999999, 0.5, 0.5, 0.5, 
                0.5, 0.0197543999999999, 0.5, 0.5, 0.8872983, 
                0.5, 0.0197543999999999, 0.5, 0.8872983, 0.5, 
                0.5, 0.0197543999999999, 0.8872983, 0.5, 0.5, 
                0.5, 0.1127017, 0.0197543999999999, 0.5, 0.5, 
                0.5, 0.1127017, 0.1127017, 0.1127017, 0.1127017, 
                0.5, 0.1127017, 0.1127017, 0.1127017, 0.5, 
                0.5, 0.1127017, 0.1127017, 0.1127017, 0.8872983, 
                0.5, 0.1127017, 0.1127017, 0.5, 0.1127017, 
                0.5, 0.1127017, 0.1127017, 0.5, 0.5, 
                0.5, 0.1127017, 0.1127017, 0.5, 0.8872983, 
                0.5, 0.1127017, 0.1127017, 0.8872983, 0.1127017, 
                0.5, 0.1127017, 0.1127017, 0.8872983, 0.5, 
                0.5, 0.1127017, 0.1127017, 0.8872983, 0.8872983, 
                0.5, 0.1127017, 0.2828781, 0.5, 0.5, 
                0.5, 0.1127017, 0.5, 0.0197543999999999, 0.5, 
                0.5, 0.1127017, 0.5, 0.1127017, 0.1127017, 
                0.5, 0.1127017, 0.5, 0.1127017, 0.5, 
                0.5, 0.1127017, 0.5, 0.1127017, 0.8872983, 
                0.5, 0.1127017, 0.5, 0.2828781, 0.5, 
                0.5, 0.1127017, 0.5, 0.5, 0.0197543999999999, 
                0.5, 0.1127017, 0.5, 0.5, 0.1127017, 
                0.5, 0.1127017, 0.5, 0.5, 0.2828781, 
                0.5, 0.1127017, 0.5, 0.5, 0.5, 
                0.5, 0.1127017, 0.5, 0.5, 0.7171219, 
                0.5, 0.1127017, 0.5, 0.5, 0.8872983, 
                0.5, 0.1127017, 0.5, 0.5, 0.9802456, 
                0.5, 0.1127017, 0.5, 0.7171219, 0.5, 
                0.5, 0.1127017, 0.5, 0.8872983, 0.1127017, 
                0.5, 0.1127017, 0.5, 0.8872983, 0.5, 
                0.5, 0.1127017, 0.5, 0.8872983, 0.8872983, 
                0.5, 0.1127017, 0.5, 0.9802456, 0.5, 
                0.5, 0.1127017, 0.7171219, 0.5, 0.5, 
                0.5, 0.1127017, 0.8872983, 0.1127017, 0.1127017, 
                0.5, 0.1127017, 0.8872983, 0.1127017, 0.5, 
                0.5, 0.1127017, 0.8872983, 0.1127017, 0.8872983, 
                0.5, 0.1127017, 0.8872983, 0.5, 0.1127017, 
                0.5, 0.1127017, 0.8872983, 0.5, 0.5, 
                0.5, 0.1127017, 0.8872983, 0.5, 0.8872983, 
                0.5, 0.1127017, 0.8872983, 0.8872983, 0.1127017, 
                0.5, 0.1127017, 0.8872983, 0.8872983, 0.5, 
                0.5, 0.1127017, 0.8872983, 0.8872983, 0.8872983, 
                0.5, 0.1127017, 0.9802456, 0.5, 0.5, 
                0.5, 0.2828781, 0.1127017, 0.5, 0.5, 
                0.5, 0.2828781, 0.5, 0.1127017, 0.5, 
                0.5, 0.2828781, 0.5, 0.5, 0.1127017, 
                0.5, 0.2828781, 0.5, 0.5, 0.5, 
                0.5, 0.2828781, 0.5, 0.5, 0.8872983, 
                0.5, 0.2828781, 0.5, 0.8872983, 0.5, 
                0.5, 0.2828781, 0.8872983, 0.5, 0.5, 
                0.5, 0.5, 0.0197543999999999, 0.1127017, 0.5, 
                0.5, 0.5, 0.0197543999999999, 0.5, 0.1127017, 
                0.5, 0.5, 0.0197543999999999, 0.5, 0.5, 
                0.5, 0.5, 0.0197543999999999, 0.5, 0.8872983, 
                0.5, 0.5, 0.0197543999999999, 0.8872983, 0.5, 
                0.5, 0.5, 0.1127017, 0.0197543999999999, 0.5, 
                0.5, 0.5, 0.1127017, 0.1127017, 0.1127017, 
                0.5, 0.5, 0.1127017, 0.1127017, 0.5, 
                0.5, 0.5, 0.1127017, 0.1127017, 0.8872983, 
                0.5, 0.5, 0.1127017, 0.2828781, 0.5, 
                0.5, 0.5, 0.1127017, 0.5, 0.0197543999999999, 
                0.5, 0.5, 0.1127017, 0.5, 0.1127017, 
                0.5, 0.5, 0.1127017, 0.5, 0.2828781, 
                0.5, 0.5, 0.1127017, 0.5, 0.5, 
                0.5, 0.5, 0.1127017, 0.5, 0.7171219, 
                0.5, 0.5, 0.1127017, 0.5, 0.8872983, 
                0.5, 0.5, 0.1127017, 0.5, 0.9802456, 
                0.5, 0.5, 0.1127017, 0.7171219, 0.5, 
                0.5, 0.5, 0.1127017, 0.8872983, 0.1127017, 
                0.5, 0.5, 0.1127017, 0.8872983, 0.5, 
                0.5, 0.5, 0.1127017, 0.8872983, 0.8872983, 
                0.5, 0.5, 0.1127017, 0.9802456, 0.5, 
                0.5, 0.5, 0.2828781, 0.1127017, 0.5, 
                0.5, 0.5, 0.2828781, 0.5, 0.1127017, 
                0.5, 0.5, 0.2828781, 0.5, 0.5, 
                0.5, 0.5, 0.2828781, 0.5, 0.8872983, 
                0.5, 0.5, 0.2828781, 0.8872983, 0.5, 
                0.5, 0.5, 0.5, 0.0197543999999999, 0.1127017, 
                0.5, 0.5, 0.5, 0.0197543999999999, 0.5, 
                0.5, 0.5, 0.5, 0.0197543999999999, 0.8872983, 
                0.5, 0.5, 0.5, 0.1127017, 0.0197543999999999, 
                0.5, 0.5, 0.5, 0.1127017, 0.1127017, 
                0.5, 0.5, 0.5, 0.1127017, 0.2828781, 
                0.5, 0.5, 0.5, 0.1127017, 0.5, 
                0.5, 0.5, 0.5, 0.1127017, 0.7171219, 
                0.5, 0.5, 0.5, 0.1127017, 0.8872983, 
                0.5, 0.5, 0.5, 0.1127017, 0.9802456, 
                0.5, 0.5, 0.5, 0.2828781, 0.1127017, 
                0.5, 0.5, 0.5, 0.2828781, 0.5, 
                0.5, 0.5, 0.5, 0.2828781, 0.8872983, 
                0.5, 0.5, 0.5, 0.5, 0.0197543999999999, 
                0.5, 0.5, 0.5, 0.5, 0.1127017, 
                0.5, 0.5, 0.5, 0.5, 0.2828781, 
                0.5, 0.5, 0.5, 0.5, 0.5, 
                0.5, 0.5, 0.5, 0.5, 0.7171219, 
                0.5, 0.5, 0.5, 0.5, 0.8872983, 
                0.5, 0.5, 0.5, 0.5, 0.9802456, 
                0.5, 0.5, 0.5, 0.7171219, 0.1127017, 
                0.5, 0.5, 0.5, 0.7171219, 0.5, 
                0.5, 0.5, 0.5, 0.7171219, 0.8872983, 
                0.5, 0.5, 0.5, 0.8872983, 0.0197543999999999, 
                0.5, 0.5, 0.5, 0.8872983, 0.1127017, 
                0.5, 0.5, 0.5, 0.8872983, 0.2828781, 
                0.5, 0.5, 0.5, 0.8872983, 0.5, 
                0.5, 0.5, 0.5, 0.8872983, 0.7171219, 
                0.5, 0.5, 0.5, 0.8872983, 0.8872983, 
                0.5, 0.5, 0.5, 0.8872983, 0.9802456, 
                0.5, 0.5, 0.5, 0.9802456, 0.1127017, 
                0.5, 0.5, 0.5, 0.9802456, 0.5, 
                0.5, 0.5, 0.5, 0.9802456, 0.8872983, 
                0.5, 0.5, 0.7171219, 0.1127017, 0.5, 
                0.5, 0.5, 0.7171219, 0.5, 0.1127017, 
                0.5, 0.5, 0.7171219, 0.5, 0.5, 
                0.5, 0.5, 0.7171219, 0.5, 0.8872983, 
                0.5, 0.5, 0.7171219, 0.8872983, 0.5, 
                0.5, 0.5, 0.8872983, 0.0197543999999999, 0.5, 
                0.5, 0.5, 0.8872983, 0.1127017, 0.1127017, 
                0.5, 0.5, 0.8872983, 0.1127017, 0.5, 
                0.5, 0.5, 0.8872983, 0.1127017, 0.8872983, 
                0.5, 0.5, 0.8872983, 0.2828781, 0.5, 
                0.5, 0.5, 0.8872983, 0.5, 0.0197543999999999, 
                0.5, 0.5, 0.8872983, 0.5, 0.1127017, 
                0.5, 0.5, 0.8872983, 0.5, 0.2828781, 
                0.5, 0.5, 0.8872983, 0.5, 0.5, 
                0.5, 0.5, 0.8872983, 0.5, 0.7171219, 
                0.5, 0.5, 0.8872983, 0.5, 0.8872983, 
                0.5, 0.5, 0.8872983, 0.5, 0.9802456, 
                0.5, 0.5, 0.8872983, 0.7171219, 0.5, 
                0.5, 0.5, 0.8872983, 0.8872983, 0.1127017, 
                0.5, 0.5, 0.8872983, 0.8872983, 0.5, 
                0.5, 0.5, 0.8872983, 0.8872983, 0.8872983, 
                0.5, 0.5, 0.8872983, 0.9802456, 0.5, 
                0.5, 0.5, 0.9802456, 0.1127017, 0.5, 
                0.5, 0.5, 0.9802456, 0.5, 0.1127017, 
                0.5, 0.5, 0.9802456, 0.5, 0.5, 
                0.5, 0.5, 0.9802456, 0.5, 0.8872983, 
                0.5, 0.5, 0.9802456, 0.8872983, 0.5, 
                0.5, 0.7171219, 0.1127017, 0.5, 0.5, 
                0.5, 0.7171219, 0.5, 0.1127017, 0.5, 
                0.5, 0.7171219, 0.5, 0.5, 0.1127017, 
                0.5, 0.7171219, 0.5, 0.5, 0.5, 
                0.5, 0.7171219, 0.5, 0.5, 0.8872983, 
                0.5, 0.7171219, 0.5, 0.8872983, 0.5, 
                0.5, 0.7171219, 0.8872983, 0.5, 0.5, 
                0.5, 0.8872983, 0.0197543999999999, 0.5, 0.5, 
                0.5, 0.8872983, 0.1127017, 0.1127017, 0.1127017, 
                0.5, 0.8872983, 0.1127017, 0.1127017, 0.5, 
                0.5, 0.8872983, 0.1127017, 0.1127017, 0.8872983, 
                0.5, 0.8872983, 0.1127017, 0.5, 0.1127017, 
                0.5, 0.8872983, 0.1127017, 0.5, 0.5, 
                0.5, 0.8872983, 0.1127017, 0.5, 0.8872983, 
                0.5, 0.8872983, 0.1127017, 0.8872983, 0.1127017, 
                0.5, 0.8872983, 0.1127017, 0.8872983, 0.5, 
                0.5, 0.8872983, 0.1127017, 0.8872983, 0.8872983, 
                0.5, 0.8872983, 0.2828781, 0.5, 0.5, 
                0.5, 0.8872983, 0.5, 0.0197543999999999, 0.5, 
                0.5, 0.8872983, 0.5, 0.1127017, 0.1127017, 
                0.5, 0.8872983, 0.5, 0.1127017, 0.5, 
                0.5, 0.8872983, 0.5, 0.1127017, 0.8872983, 
                0.5, 0.8872983, 0.5, 0.2828781, 0.5, 
                0.5, 0.8872983, 0.5, 0.5, 0.0197543999999999, 
                0.5, 0.8872983, 0.5, 0.5, 0.1127017, 
                0.5, 0.8872983, 0.5, 0.5, 0.2828781, 
                0.5, 0.8872983, 0.5, 0.5, 0.5, 
                0.5, 0.8872983, 0.5, 0.5, 0.7171219, 
                0.5, 0.8872983, 0.5, 0.5, 0.8872983, 
                0.5, 0.8872983, 0.5, 0.5, 0.9802456, 
                0.5, 0.8872983, 0.5, 0.7171219, 0.5, 
                0.5, 0.8872983, 0.5, 0.8872983, 0.1127017, 
                0.5, 0.8872983, 0.5, 0.8872983, 0.5, 
                0.5, 0.8872983, 0.5, 0.8872983, 0.8872983, 
                0.5, 0.8872983, 0.5, 0.9802456, 0.5, 
                0.5, 0.8872983, 0.7171219, 0.5, 0.5, 
                0.5, 0.8872983, 0.8872983, 0.1127017, 0.1127017, 
                0.5, 0.8872983, 0.8872983, 0.1127017, 0.5, 
                0.5, 0.8872983, 0.8872983, 0.1127017, 0.8872983, 
                0.5, 0.8872983, 0.8872983, 0.5, 0.1127017, 
                0.5, 0.8872983, 0.8872983, 0.5, 0.5, 
                0.5, 0.8872983, 0.8872983, 0.5, 0.8872983, 
                0.5, 0.8872983, 0.8872983, 0.8872983, 0.1127017, 
                0.5, 0.8872983, 0.8872983, 0.8872983, 0.5, 
                0.5, 0.8872983, 0.8872983, 0.8872983, 0.8872983, 
                0.5, 0.8872983, 0.9802456, 0.5, 0.5, 
                0.5, 0.9802456, 0.1127017, 0.5, 0.5, 
                0.5, 0.9802456, 0.5, 0.1127017, 0.5, 
                0.5, 0.9802456, 0.5, 0.5, 0.1127017, 
                0.5, 0.9802456, 0.5, 0.5, 0.5, 
                0.5, 0.9802456, 0.5, 0.5, 0.8872983, 
                0.5, 0.9802456, 0.5, 0.8872983, 0.5, 
                0.5, 0.9802456, 0.8872983, 0.5, 0.5, 
                0.7171219, 0.1127017, 0.5, 0.5, 0.5, 
                0.7171219, 0.5, 0.1127017, 0.5, 0.5, 
                0.7171219, 0.5, 0.5, 0.1127017, 0.5, 
                0.7171219, 0.5, 0.5, 0.5, 0.1127017, 
                0.7171219, 0.5, 0.5, 0.5, 0.5, 
                0.7171219, 0.5, 0.5, 0.5, 0.8872983, 
                0.7171219, 0.5, 0.5, 0.8872983, 0.5, 
                0.7171219, 0.5, 0.8872983, 0.5, 0.5, 
                0.7171219, 0.8872983, 0.5, 0.5, 0.5, 
                0.8872983, 0.0197543999999999, 0.5, 0.5, 0.5, 
                0.8872983, 0.1127017, 0.1127017, 0.1127017, 0.5, 
                0.8872983, 0.1127017, 0.1127017, 0.5, 0.1127017, 
                0.8872983, 0.1127017, 0.1127017, 0.5, 0.5, 
                0.8872983, 0.1127017, 0.1127017, 0.5, 0.8872983, 
                0.8872983, 0.1127017, 0.1127017, 0.8872983, 0.5, 
                0.8872983, 0.1127017, 0.5, 0.1127017, 0.1127017, 
                0.8872983, 0.1127017, 0.5, 0.1127017, 0.5, 
                0.8872983, 0.1127017, 0.5, 0.1127017, 0.8872983, 
                0.8872983, 0.1127017, 0.5, 0.5, 0.1127017, 
                0.8872983, 0.1127017, 0.5, 0.5, 0.5, 
                0.8872983, 0.1127017, 0.5, 0.5, 0.8872983, 
                0.8872983, 0.1127017, 0.5, 0.8872983, 0.1127017, 
                0.8872983, 0.1127017, 0.5, 0.8872983, 0.5, 
                0.8872983, 0.1127017, 0.5, 0.8872983, 0.8872983, 
                0.8872983, 0.1127017, 0.8872983, 0.1127017, 0.5, 
                0.8872983, 0.1127017, 0.8872983, 0.5, 0.1127017, 
                0.8872983, 0.1127017, 0.8872983, 0.5, 0.5, 
                0.8872983, 0.1127017, 0.8872983, 0.5, 0.8872983, 
                0.8872983, 0.1127017, 0.8872983, 0.8872983, 0.5, 
                0.8872983, 0.2828781, 0.5, 0.5, 0.5, 
                0.8872983, 0.5, 0.0197543999999999, 0.5, 0.5, 
                0.8872983, 0.5, 0.1127017, 0.1127017, 0.1127017, 
                0.8872983, 0.5, 0.1127017, 0.1127017, 0.5, 
                0.8872983, 0.5, 0.1127017, 0.1127017, 0.8872983, 
                0.8872983, 0.5, 0.1127017, 0.5, 0.1127017, 
                0.8872983, 0.5, 0.1127017, 0.5, 0.5, 
                0.8872983, 0.5, 0.1127017, 0.5, 0.8872983, 
                0.8872983, 0.5, 0.1127017, 0.8872983, 0.1127017, 
                0.8872983, 0.5, 0.1127017, 0.8872983, 0.5, 
                0.8872983, 0.5, 0.1127017, 0.8872983, 0.8872983, 
                0.8872983, 0.5, 0.2828781, 0.5, 0.5, 
                0.8872983, 0.5, 0.5, 0.0197543999999999, 0.5, 
                0.8872983, 0.5, 0.5, 0.1127017, 0.1127017, 
                0.8872983, 0.5, 0.5, 0.1127017, 0.5, 
                0.8872983, 0.5, 0.5, 0.1127017, 0.8872983, 
                0.8872983, 0.5, 0.5, 0.2828781, 0.5, 
                0.8872983, 0.5, 0.5, 0.5, 0.0197543999999999, 
                0.8872983, 0.5, 0.5, 0.5, 0.1127017, 
                0.8872983, 0.5, 0.5, 0.5, 0.2828781, 
                0.8872983, 0.5, 0.5, 0.5, 0.5, 
                0.8872983, 0.5, 0.5, 0.5, 0.7171219, 
                0.8872983, 0.5, 0.5, 0.5, 0.8872983, 
                0.8872983, 0.5, 0.5, 0.5, 0.9802456, 
                0.8872983, 0.5, 0.5, 0.7171219, 0.5, 
                0.8872983, 0.5, 0.5, 0.8872983, 0.1127017, 
                0.8872983, 0.5, 0.5, 0.8872983, 0.5, 
                0.8872983, 0.5, 0.5, 0.8872983, 0.8872983, 
                0.8872983, 0.5, 0.5, 0.9802456, 0.5, 
                0.8872983, 0.5, 0.7171219, 0.5, 0.5, 
                0.8872983, 0.5, 0.8872983, 0.1127017, 0.1127017, 
                0.8872983, 0.5, 0.8872983, 0.1127017, 0.5, 
                0.8872983, 0.5, 0.8872983, 0.1127017, 0.8872983, 
                0.8872983, 0.5, 0.8872983, 0.5, 0.1127017, 
                0.8872983, 0.5, 0.8872983, 0.5, 0.5, 
                0.8872983, 0.5, 0.8872983, 0.5, 0.8872983, 
                0.8872983, 0.5, 0.8872983, 0.8872983, 0.1127017, 
                0.8872983, 0.5, 0.8872983, 0.8872983, 0.5, 
                0.8872983, 0.5, 0.8872983, 0.8872983, 0.8872983, 
                0.8872983, 0.5, 0.9802456, 0.5, 0.5, 
                0.8872983, 0.7171219, 0.5, 0.5, 0.5, 
                0.8872983, 0.8872983, 0.1127017, 0.1127017, 0.5, 
                0.8872983, 0.8872983, 0.1127017, 0.5, 0.1127017, 
                0.8872983, 0.8872983, 0.1127017, 0.5, 0.5, 
                0.8872983, 0.8872983, 0.1127017, 0.5, 0.8872983, 
                0.8872983, 0.8872983, 0.1127017, 0.8872983, 0.5, 
                0.8872983, 0.8872983, 0.5, 0.1127017, 0.1127017, 
                0.8872983, 0.8872983, 0.5, 0.1127017, 0.5, 
                0.8872983, 0.8872983, 0.5, 0.1127017, 0.8872983, 
                0.8872983, 0.8872983, 0.5, 0.5, 0.1127017, 
                0.8872983, 0.8872983, 0.5, 0.5, 0.5, 
                0.8872983, 0.8872983, 0.5, 0.5, 0.8872983, 
                0.8872983, 0.8872983, 0.5, 0.8872983, 0.1127017, 
                0.8872983, 0.8872983, 0.5, 0.8872983, 0.5, 
                0.8872983, 0.8872983, 0.5, 0.8872983, 0.8872983, 
                0.8872983, 0.8872983, 0.8872983, 0.1127017, 0.5, 
                0.8872983, 0.8872983, 0.8872983, 0.5, 0.1127017, 
                0.8872983, 0.8872983, 0.8872983, 0.5, 0.5, 
                0.8872983, 0.8872983, 0.8872983, 0.5, 0.8872983, 
                0.8872983, 0.8872983, 0.8872983, 0.8872983, 0.5, 
                0.8872983, 0.9802456, 0.5, 0.5, 0.5, 
                0.9802456, 0.1127017, 0.5, 0.5, 0.5, 
                0.9802456, 0.5, 0.1127017, 0.5, 0.5, 
                0.9802456, 0.5, 0.5, 0.1127017, 0.5, 
                0.9802456, 0.5, 0.5, 0.5, 0.1127017, 
                0.9802456, 0.5, 0.5, 0.5, 0.5, 
                0.9802456, 0.5, 0.5, 0.5, 0.8872983, 
                0.9802456, 0.5, 0.5, 0.8872983, 0.5, 
                0.9802456, 0.5, 0.8872983, 0.5, 0.5, 
                0.9802456, 0.8872983, 0.5, 0.5, 0.5
            }),
            /* w = */ std::vector<double>
            ({
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                -0.0639565823650983, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                -0.00238149919981761, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                -0.0023814991998176, 
                 0.00595374371284889, 
                -0.0023814991998176, 
                -0.0597364221037197, 
                -0.0023814991998176, 
                 0.00595374371284889, 
                -0.0023814991998176, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                -0.00238149919981761, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.0557496489238249, 
                 0.0145355859497386, 
                 0.00595374371284889, 
                -0.00238149919981761, 
                 0.00595374371284889, 
                -0.0023814991998176, 
                -0.0597364221037198, 
                -0.0023814991998176, 
                 0.00595374371284889, 
                -0.00238149919981761, 
                 0.00595374371284889, 
                 0.0557496489238249, 
                 0.0145355859497386, 
                -0.0023814991998176, 
                -0.0597364221037197, 
                -0.0023814991998176, 
                 0.0557496489238249, 
                 0.0145355859497386, 
                -0.0597364221037198, 
                 0.0557496489238249, 
                -0.0835103067498917, 
                 0.0557496489238249, 
                -0.0597364221037198, 
                 0.0145355859497386, 
                 0.0557496489238249, 
                -0.0023814991998176, 
                -0.0597364221037197, 
                -0.0023814991998176, 
                 0.0145355859497386, 
                 0.0557496489238249, 
                 0.00595374371284889, 
                -0.00238149919981761, 
                 0.00595374371284889, 
                -0.0023814991998176, 
                -0.0597364221037198, 
                -0.0023814991998176, 
                 0.00595374371284889, 
                -0.00238149919981761, 
                 0.00595374371284889, 
                 0.0145355859497386, 
                 0.0557496489238249, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                -0.00238149919981761, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                -0.0023814991998176, 
                 0.00595374371284889, 
                -0.0023814991998176, 
                -0.0597364221037197, 
                -0.0023814991998176, 
                 0.00595374371284889, 
                -0.0023814991998176, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                -0.00238149919981761, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.0145355859497386, 
                 0.0557496489238249, 
                 0.0557496489238249, 
                 0.0557496489238249, 
                 0.0557496489238249, 
                -0.245298471320727, 
                 0.0557496489238249, 
                 0.0557496489238249, 
                 0.0557496489238249, 
                 0.0557496489238249, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                -0.0639565823650983, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.00595374371284889, 
                -0.00238149919981761, 
                 0.00595374371284889, 
                -0.00238149919981761, 
                -0.0597364221037197, 
                -0.00238149919981761, 
                 0.00595374371284889, 
                -0.00238149919981761, 
                 0.00595374371284889, 
                 0.0557496489238249, 
                 0.0145355859497386, 
                -0.0023814991998176, 
                -0.0597364221037197, 
                -0.0023814991998176, 
                 0.0557496489238249, 
                 0.0145355859497386, 
                -0.0597364221037197, 
                 0.0557496489238249, 
                -0.0835103067498925, 
                 0.0557496489238249, 
                -0.0597364221037197, 
                 0.0145355859497386, 
                 0.0557496489238249, 
                -0.0023814991998176, 
                -0.0597364221037197, 
                -0.0023814991998176, 
                 0.0145355859497386, 
                 0.0557496489238249, 
                 0.00595374371284889, 
                -0.00238149919981761, 
                 0.00595374371284889, 
                -0.00238149919981761, 
                -0.0597364221037197, 
                -0.00238149919981761, 
                 0.00595374371284889, 
                -0.00238149919981761, 
                 0.00595374371284889, 
                 0.0145355859497386, 
                 0.0557496489238249, 
                 0.0557496489238249, 
                 0.0557496489238249, 
                -0.245298471320727, 
                 0.0557496489238249, 
                 0.0557496489238249, 
                 0.0557496489238249, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                -0.0639565823650983, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                -0.0023814991998176, 
                -0.0597364221037198, 
                -0.0023814991998176, 
                 0.0557496489238249, 
                 0.0145355859497386, 
                -0.0597364221037196, 
                 0.0557496489238249, 
                -0.0835103067498914, 
                 0.0557496489238249, 
                -0.0597364221037196, 
                 0.0145355859497386, 
                 0.0557496489238249, 
                -0.0023814991998176, 
                -0.0597364221037198, 
                -0.0023814991998176, 
                 0.0145355859497386, 
                 0.0557496489238249, 
                 0.0557496489238249, 
                -0.245298471320727, 
                 0.0557496489238249, 
                 0.0557496489238249, 
                 0.0145355859497386, 
                -0.0639565823650983, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                -0.0597364221037197, 
                 0.0557496489238249, 
                -0.0835103067498916, 
                 0.0557496489238249, 
                -0.0597364221037197, 
                 0.0145355859497386, 
                 0.0557496489238249, 
                -0.245298471320727, 
                 0.0557496489238249, 
                -0.0639565823650983, 
                -0.0835103067498903, 
                -0.245298471320727, 
                 1.40851213757837, 
                -0.245298471320727, 
                -0.0835103067498903, 
                -0.0639565823650983, 
                 0.0557496489238249, 
                -0.245298471320727, 
                 0.0557496489238249, 
                 0.0145355859497386, 
                -0.0597364221037197, 
                 0.0557496489238249, 
                -0.0835103067498916, 
                 0.0557496489238249, 
                -0.0597364221037197, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                -0.0639565823650983, 
                 0.0145355859497386, 
                 0.0557496489238249, 
                 0.0557496489238249, 
                -0.245298471320727, 
                 0.0557496489238249, 
                 0.0557496489238249, 
                 0.0145355859497386, 
                -0.0023814991998176, 
                -0.0597364221037198, 
                -0.0023814991998176, 
                 0.0557496489238249, 
                 0.0145355859497386, 
                -0.0597364221037196, 
                 0.0557496489238249, 
                -0.0835103067498914, 
                 0.0557496489238249, 
                -0.0597364221037196, 
                 0.0145355859497386, 
                 0.0557496489238249, 
                -0.0023814991998176, 
                -0.0597364221037198, 
                -0.0023814991998176, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                -0.0639565823650983, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0557496489238249, 
                 0.0557496489238249, 
                 0.0557496489238249, 
                -0.245298471320727, 
                 0.0557496489238249, 
                 0.0557496489238249, 
                 0.0557496489238249, 
                 0.0145355859497386, 
                 0.00595374371284889, 
                -0.00238149919981761, 
                 0.00595374371284889, 
                -0.00238149919981761, 
                -0.0597364221037197, 
                -0.00238149919981761, 
                 0.00595374371284889, 
                -0.00238149919981761, 
                 0.00595374371284889, 
                 0.0557496489238249, 
                 0.0145355859497386, 
                -0.0023814991998176, 
                -0.0597364221037197, 
                -0.0023814991998176, 
                 0.0557496489238249, 
                 0.0145355859497386, 
                -0.0597364221037197, 
                 0.0557496489238249, 
                -0.0835103067498925, 
                 0.0557496489238249, 
                -0.0597364221037197, 
                 0.0145355859497386, 
                 0.0557496489238249, 
                -0.0023814991998176, 
                -0.0597364221037197, 
                -0.0023814991998176, 
                 0.0145355859497386, 
                 0.0557496489238249, 
                 0.00595374371284889, 
                -0.00238149919981761, 
                 0.00595374371284889, 
                -0.00238149919981761, 
                -0.0597364221037197, 
                -0.00238149919981761, 
                 0.00595374371284889, 
                -0.00238149919981761, 
                 0.00595374371284889, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                -0.0639565823650983, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0557496489238249, 
                 0.0557496489238249, 
                 0.0557496489238249, 
                 0.0557496489238249, 
                -0.245298471320727, 
                 0.0557496489238249, 
                 0.0557496489238249, 
                 0.0557496489238249, 
                 0.0557496489238249, 
                 0.0145355859497386, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                -0.00238149919981761, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                -0.0023814991998176, 
                 0.00595374371284889, 
                -0.0023814991998176, 
                -0.0597364221037197, 
                -0.0023814991998176, 
                 0.00595374371284889, 
                -0.0023814991998176, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                -0.00238149919981761, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.0557496489238249, 
                 0.0145355859497386, 
                 0.00595374371284889, 
                -0.00238149919981761, 
                 0.00595374371284889, 
                -0.0023814991998176, 
                -0.0597364221037198, 
                -0.0023814991998176, 
                 0.00595374371284889, 
                -0.00238149919981761, 
                 0.00595374371284889, 
                 0.0557496489238249, 
                 0.0145355859497386, 
                -0.0023814991998176, 
                -0.0597364221037197, 
                -0.0023814991998176, 
                 0.0557496489238249, 
                 0.0145355859497386, 
                -0.0597364221037198, 
                 0.0557496489238249, 
                -0.0835103067498917, 
                 0.0557496489238249, 
                -0.0597364221037198, 
                 0.0145355859497386, 
                 0.0557496489238249, 
                -0.0023814991998176, 
                -0.0597364221037197, 
                -0.0023814991998176, 
                 0.0145355859497386, 
                 0.0557496489238249, 
                 0.00595374371284889, 
                -0.00238149919981761, 
                 0.00595374371284889, 
                -0.0023814991998176, 
                -0.0597364221037198, 
                -0.0023814991998176, 
                 0.00595374371284889, 
                -0.00238149919981761, 
                 0.00595374371284889, 
                 0.0145355859497386, 
                 0.0557496489238249, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                -0.00238149919981761, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                -0.0023814991998176, 
                 0.00595374371284889, 
                -0.0023814991998176, 
                -0.0597364221037197, 
                -0.0023814991998176, 
                 0.00595374371284889, 
                -0.0023814991998176, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                -0.00238149919981761, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                -0.0639565823650983, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0145355859497386
            })
        }
    },
    {
        d6l1n1,
        {
            /* dim = */ 6,
            /* n = */ 1,
            /* x = */ std::vector<double>
            ({
                0.5, 0.5, 0.5, 0.5, 0.5, 0.5
            }),
            /* w = */ std::vector<double>
            ({
                 1
            })
        }
    },
    {
        d6l2n13,
        {
            /* dim = */ 6,
            /* n = */ 13,
            /* x = */ std::vector<double>
            ({
                0.1127017, 0.5, 0.5, 0.5, 0.5, 0.5, 
                0.5, 0.1127017, 0.5, 0.5, 0.5, 0.5, 
                0.5, 0.5, 0.1127017, 0.5, 0.5, 0.5, 
                0.5, 0.5, 0.5, 0.1127017, 0.5, 0.5, 
                0.5, 0.5, 0.5, 0.5, 0.1127017, 0.5, 
                0.5, 0.5, 0.5, 0.5, 0.5, 0.1127017, 
                0.5, 0.5, 0.5, 0.5, 0.5, 0.5, 
                0.5, 0.5, 0.5, 0.5, 0.5, 0.8872983, 
                0.5, 0.5, 0.5, 0.5, 0.8872983, 0.5, 
                0.5, 0.5, 0.5, 0.8872983, 0.5, 0.5, 
                0.5, 0.5, 0.8872983, 0.5, 0.5, 0.5, 
                0.5, 0.8872983, 0.5, 0.5, 0.5, 0.5, 
                0.8872983, 0.5, 0.5, 0.5, 0.5, 0.5
            }),
            /* w = */ std::vector<double>
            ({
                 0.2777778, 
                 0.2777778, 
                 0.2777778, 
                 0.2777778, 
                 0.2777778, 
                 0.2777778, 
                -2.3333336, 
                 0.2777778, 
                 0.2777778, 
                 0.2777778, 
                 0.2777778, 
                 0.2777778, 
                 0.2777778
            })
        }
    },
    {
        d6l3n73,
        {
            /* dim = */ 6,
            /* n = */ 73,
            /* x = */ std::vector<double>
            ({
                0.1127017, 0.1127017, 0.5, 0.5, 0.5, 0.5, 
                0.1127017, 0.5, 0.1127017, 0.5, 0.5, 0.5, 
                0.1127017, 0.5, 0.5, 0.1127017, 0.5, 0.5, 
                0.1127017, 0.5, 0.5, 0.5, 0.1127017, 0.5, 
                0.1127017, 0.5, 0.5, 0.5, 0.5, 0.1127017, 
                0.1127017, 0.5, 0.5, 0.5, 0.5, 0.5, 
                0.1127017, 0.5, 0.5, 0.5, 0.5, 0.8872983, 
                0.1127017, 0.5, 0.5, 0.5, 0.8872983, 0.5, 
                0.1127017, 0.5, 0.5, 0.8872983, 0.5, 0.5, 
                0.1127017, 0.5, 0.8872983, 0.5, 0.5, 0.5, 
                0.1127017, 0.8872983, 0.5, 0.5, 0.5, 0.5, 
                0.5, 0.1127017, 0.1127017, 0.5, 0.5, 0.5, 
                0.5, 0.1127017, 0.5, 0.1127017, 0.5, 0.5, 
                0.5, 0.1127017, 0.5, 0.5, 0.1127017, 0.5, 
                0.5, 0.1127017, 0.5, 0.5, 0.5, 0.1127017, 
                0.5, 0.1127017, 0.5, 0.5, 0.5, 0.5, 
                0.5, 0.1127017, 0.5, 0.5, 0.5, 0.8872983, 
                0.5, 0.1127017, 0.5, 0.5, 0.8872983, 0.5, 
                0.5, 0.1127017, 0.5, 0.8872983, 0.5, 0.5, 
                0.5, 0.1127017, 0.8872983, 0.5, 0.5, 0.5, 
                0.5, 0.5, 0.1127017, 0.1127017, 0.5, 0.5, 
                0.5, 0.5, 0.1127017, 0.5, 0.1127017, 0.5, 
                0.5, 0.5, 0.1127017, 0.5, 0.5, 0.1127017, 
                0.5, 0.5, 0.1127017, 0.5, 0.5, 0.5, 
                0.5, 0.5, 0.1127017, 0.5, 0.5, 0.8872983, 
                0.5, 0.5, 0.1127017, 0.5, 0.8872983, 0.5, 
                0.5, 0.5, 0.1127017, 0.8872983, 0.5, 0.5, 
                0.5, 0.5, 0.5, 0.1127017, 0.1127017, 0.5, 
                0.5, 0.5, 0.5, 0.1127017, 0.5, 0.1127017, 
                0.5, 0.5, 0.5, 0.1127017, 0.5, 0.5, 
                0.5, 0.5, 0.5, 0.1127017, 0.5, 0.8872983, 
                0.5, 0.5, 0.5, 0.1127017, 0.8872983, 0.5, 
                0.5, 0.5, 0.5, 0.5, 0.1127017, 0.1127017, 
                0.5, 0.5, 0.5, 0.5, 0.1127017, 0.5, 
                0.5, 0.5, 0.5, 0.5, 0.1127017, 0.8872983, 
                0.5, 0.5, 0.5, 0.5, 0.5, 0.1127017, 
                0.5, 0.5, 0.5, 0.5, 0.5, 0.5, 
                0.5, 0.5, 0.5, 0.5, 0.5, 0.8872983, 
                0.5, 0.5, 0.5, 0.5, 0.8872983, 0.1127017, 
                0.5, 0.5, 0.5, 0.5, 0.8872983, 0.5, 
                0.5, 0.5, 0.5, 0.5, 0.8872983, 0.8872983, 
                0.5, 0.5, 0.5, 0.8872983, 0.1127017, 0.5, 
                0.5, 0.5, 0.5, 0.8872983, 0.5, 0.1127017, 
                0.5, 0.5, 0.5, 0.8872983, 0.5, 0.5, 
                0.5, 0.5, 0.5, 0.8872983, 0.5, 0.8872983, 
                0.5, 0.5, 0.5, 0.8872983, 0.8872983, 0.5, 
                0.5, 0.5, 0.8872983, 0.1127017, 0.5, 0.5, 
                0.5, 0.5, 0.8872983, 0.5, 0.1127017, 0.5, 
                0.5, 0.5, 0.8872983, 0.5, 0.5, 0.1127017, 
                0.5, 0.5, 0.8872983, 0.5, 0.5, 0.5, 
                0.5, 0.5, 0.8872983, 0.5, 0.5, 0.8872983, 
                0.5, 0.5, 0.8872983, 0.5, 0.8872983, 0.5, 
                0.5, 0.5, 0.8872983, 0.8872983, 0.5, 0.5, 
                0.5, 0.8872983, 0.1127017, 0.5, 0.5, 0.5, 
                0.5, 0.8872983, 0.5, 0.1127017, 0.5, 0.5, 
                0.5, 0.8872983, 0.5, 0.5, 0.1127017, 0.5, 
                0.5, 0.8872983, 0.5, 0.5, 0.5, 0.1127017, 
                0.5, 0.8872983, 0.5, 0.5, 0.5, 0.5, 
                0.5, 0.8872983, 0.5, 0.5, 0.5, 0.8872983, 
                0.5, 0.8872983, 0.5, 0.5, 0.8872983, 0.5, 
                0.5, 0.8872983, 0.5, 0.8872983, 0.5, 0.5, 
                0.5, 0.8872983, 0.8872983, 0.5, 0.5, 0.5, 
                0.8872983, 0.1127017, 0.5, 0.5, 0.5, 0.5, 
                0.8872983, 0.5, 0.1127017, 0.5, 0.5, 0.5, 
                0.8872983, 0.5, 0.5, 0.1127017, 0.5, 0.5, 
                0.8872983, 0.5, 0.5, 0.5, 0.1127017, 0.5, 
                0.8872983, 0.5, 0.5, 0.5, 0.5, 0.1127017, 
                0.8872983, 0.5, 0.5, 0.5, 0.5, 0.5, 
                0.8872983, 0.5, 0.5, 0.5, 0.5, 0.8872983, 
                0.8872983, 0.5, 0.5, 0.5, 0.8872983, 0.5, 
                0.8872983, 0.5, 0.5, 0.8872983, 0.5, 0.5, 
                0.8872983, 0.5, 0.8872983, 0.5, 0.5, 0.5, 
                0.8872983, 0.8872983, 0.5, 0.5, 0.5, 0.5
            }),
            /* w = */ std::vector<double>
            ({
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                -0.4938272617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                -0.4938272617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                -0.4938272617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                -0.4938272617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                -0.4938272617284, 
                 0.07716050617284, 
                -0.4938272617284, 
                 2.2962967703704, 
                -0.4938272617284, 
                 0.07716050617284, 
                -0.4938272617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                -0.4938272617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                -0.4938272617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                -0.4938272617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                -0.4938272617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284, 
                 0.07716050617284
            })
        }
    },
    {
        d6l4n257,
        {
            /* dim = */ 6,
            /* n = */ 257,
            /* x = */ std::vector<double>
            ({
                0.0197543999999999, 0.5, 0.5, 0.5, 0.5, 0.5, 
                0.1127017, 0.1127017, 0.1127017, 0.5, 0.5, 0.5, 
                0.1127017, 0.1127017, 0.5, 0.1127017, 0.5, 0.5, 
                0.1127017, 0.1127017, 0.5, 0.5, 0.1127017, 0.5, 
                0.1127017, 0.1127017, 0.5, 0.5, 0.5, 0.1127017, 
                0.1127017, 0.1127017, 0.5, 0.5, 0.5, 0.5, 
                0.1127017, 0.1127017, 0.5, 0.5, 0.5, 0.8872983, 
                0.1127017, 0.1127017, 0.5, 0.5, 0.8872983, 0.5, 
                0.1127017, 0.1127017, 0.5, 0.8872983, 0.5, 0.5, 
                0.1127017, 0.1127017, 0.8872983, 0.5, 0.5, 0.5, 
                0.1127017, 0.5, 0.1127017, 0.1127017, 0.5, 0.5, 
                0.1127017, 0.5, 0.1127017, 0.5, 0.1127017, 0.5, 
                0.1127017, 0.5, 0.1127017, 0.5, 0.5, 0.1127017, 
                0.1127017, 0.5, 0.1127017, 0.5, 0.5, 0.5, 
                0.1127017, 0.5, 0.1127017, 0.5, 0.5, 0.8872983, 
                0.1127017, 0.5, 0.1127017, 0.5, 0.8872983, 0.5, 
                0.1127017, 0.5, 0.1127017, 0.8872983, 0.5, 0.5, 
                0.1127017, 0.5, 0.5, 0.1127017, 0.1127017, 0.5, 
                0.1127017, 0.5, 0.5, 0.1127017, 0.5, 0.1127017, 
                0.1127017, 0.5, 0.5, 0.1127017, 0.5, 0.5, 
                0.1127017, 0.5, 0.5, 0.1127017, 0.5, 0.8872983, 
                0.1127017, 0.5, 0.5, 0.1127017, 0.8872983, 0.5, 
                0.1127017, 0.5, 0.5, 0.5, 0.1127017, 0.1127017, 
                0.1127017, 0.5, 0.5, 0.5, 0.1127017, 0.5, 
                0.1127017, 0.5, 0.5, 0.5, 0.1127017, 0.8872983, 
                0.1127017, 0.5, 0.5, 0.5, 0.5, 0.1127017, 
                0.1127017, 0.5, 0.5, 0.5, 0.5, 0.5, 
                0.1127017, 0.5, 0.5, 0.5, 0.5, 0.8872983, 
                0.1127017, 0.5, 0.5, 0.5, 0.8872983, 0.1127017, 
                0.1127017, 0.5, 0.5, 0.5, 0.8872983, 0.5, 
                0.1127017, 0.5, 0.5, 0.5, 0.8872983, 0.8872983, 
                0.1127017, 0.5, 0.5, 0.8872983, 0.1127017, 0.5, 
                0.1127017, 0.5, 0.5, 0.8872983, 0.5, 0.1127017, 
                0.1127017, 0.5, 0.5, 0.8872983, 0.5, 0.5, 
                0.1127017, 0.5, 0.5, 0.8872983, 0.5, 0.8872983, 
                0.1127017, 0.5, 0.5, 0.8872983, 0.8872983, 0.5, 
                0.1127017, 0.5, 0.8872983, 0.1127017, 0.5, 0.5, 
                0.1127017, 0.5, 0.8872983, 0.5, 0.1127017, 0.5, 
                0.1127017, 0.5, 0.8872983, 0.5, 0.5, 0.1127017, 
                0.1127017, 0.5, 0.8872983, 0.5, 0.5, 0.5, 
                0.1127017, 0.5, 0.8872983, 0.5, 0.5, 0.8872983, 
                0.1127017, 0.5, 0.8872983, 0.5, 0.8872983, 0.5, 
                0.1127017, 0.5, 0.8872983, 0.8872983, 0.5, 0.5, 
                0.1127017, 0.8872983, 0.1127017, 0.5, 0.5, 0.5, 
                0.1127017, 0.8872983, 0.5, 0.1127017, 0.5, 0.5, 
                0.1127017, 0.8872983, 0.5, 0.5, 0.1127017, 0.5, 
                0.1127017, 0.8872983, 0.5, 0.5, 0.5, 0.1127017, 
                0.1127017, 0.8872983, 0.5, 0.5, 0.5, 0.5, 
                0.1127017, 0.8872983, 0.5, 0.5, 0.5, 0.8872983, 
                0.1127017, 0.8872983, 0.5, 0.5, 0.8872983, 0.5, 
                0.1127017, 0.8872983, 0.5, 0.8872983, 0.5, 0.5, 
                0.1127017, 0.8872983, 0.8872983, 0.5, 0.5, 0.5, 
                0.2828781, 0.5, 0.5, 0.5, 0.5, 0.5, 
                0.5, 0.0197543999999999, 0.5, 0.5, 0.5, 0.5, 
                0.5, 0.1127017, 0.1127017, 0.1127017, 0.5, 0.5, 
                0.5, 0.1127017, 0.1127017, 0.5, 0.1127017, 0.5, 
                0.5, 0.1127017, 0.1127017, 0.5, 0.5, 0.1127017, 
                0.5, 0.1127017, 0.1127017, 0.5, 0.5, 0.5, 
                0.5, 0.1127017, 0.1127017, 0.5, 0.5, 0.8872983, 
                0.5, 0.1127017, 0.1127017, 0.5, 0.8872983, 0.5, 
                0.5, 0.1127017, 0.1127017, 0.8872983, 0.5, 0.5, 
                0.5, 0.1127017, 0.5, 0.1127017, 0.1127017, 0.5, 
                0.5, 0.1127017, 0.5, 0.1127017, 0.5, 0.1127017, 
                0.5, 0.1127017, 0.5, 0.1127017, 0.5, 0.5, 
                0.5, 0.1127017, 0.5, 0.1127017, 0.5, 0.8872983, 
                0.5, 0.1127017, 0.5, 0.1127017, 0.8872983, 0.5, 
                0.5, 0.1127017, 0.5, 0.5, 0.1127017, 0.1127017, 
                0.5, 0.1127017, 0.5, 0.5, 0.1127017, 0.5, 
                0.5, 0.1127017, 0.5, 0.5, 0.1127017, 0.8872983, 
                0.5, 0.1127017, 0.5, 0.5, 0.5, 0.1127017, 
                0.5, 0.1127017, 0.5, 0.5, 0.5, 0.5, 
                0.5, 0.1127017, 0.5, 0.5, 0.5, 0.8872983, 
                0.5, 0.1127017, 0.5, 0.5, 0.8872983, 0.1127017, 
                0.5, 0.1127017, 0.5, 0.5, 0.8872983, 0.5, 
                0.5, 0.1127017, 0.5, 0.5, 0.8872983, 0.8872983, 
                0.5, 0.1127017, 0.5, 0.8872983, 0.1127017, 0.5, 
                0.5, 0.1127017, 0.5, 0.8872983, 0.5, 0.1127017, 
                0.5, 0.1127017, 0.5, 0.8872983, 0.5, 0.5, 
                0.5, 0.1127017, 0.5, 0.8872983, 0.5, 0.8872983, 
                0.5, 0.1127017, 0.5, 0.8872983, 0.8872983, 0.5, 
                0.5, 0.1127017, 0.8872983, 0.1127017, 0.5, 0.5, 
                0.5, 0.1127017, 0.8872983, 0.5, 0.1127017, 0.5, 
                0.5, 0.1127017, 0.8872983, 0.5, 0.5, 0.1127017, 
                0.5, 0.1127017, 0.8872983, 0.5, 0.5, 0.5, 
                0.5, 0.1127017, 0.8872983, 0.5, 0.5, 0.8872983, 
                0.5, 0.1127017, 0.8872983, 0.5, 0.8872983, 0.5, 
                0.5, 0.1127017, 0.8872983, 0.8872983, 0.5, 0.5, 
                0.5, 0.2828781, 0.5, 0.5, 0.5, 0.5, 
                0.5, 0.5, 0.0197543999999999, 0.5, 0.5, 0.5, 
                0.5, 0.5, 0.1127017, 0.1127017, 0.1127017, 0.5, 
                0.5, 0.5, 0.1127017, 0.1127017, 0.5, 0.1127017, 
                0.5, 0.5, 0.1127017, 0.1127017, 0.5, 0.5, 
                0.5, 0.5, 0.1127017, 0.1127017, 0.5, 0.8872983, 
                0.5, 0.5, 0.1127017, 0.1127017, 0.8872983, 0.5, 
                0.5, 0.5, 0.1127017, 0.5, 0.1127017, 0.1127017, 
                0.5, 0.5, 0.1127017, 0.5, 0.1127017, 0.5, 
                0.5, 0.5, 0.1127017, 0.5, 0.1127017, 0.8872983, 
                0.5, 0.5, 0.1127017, 0.5, 0.5, 0.1127017, 
                0.5, 0.5, 0.1127017, 0.5, 0.5, 0.5, 
                0.5, 0.5, 0.1127017, 0.5, 0.5, 0.8872983, 
                0.5, 0.5, 0.1127017, 0.5, 0.8872983, 0.1127017, 
                0.5, 0.5, 0.1127017, 0.5, 0.8872983, 0.5, 
                0.5, 0.5, 0.1127017, 0.5, 0.8872983, 0.8872983, 
                0.5, 0.5, 0.1127017, 0.8872983, 0.1127017, 0.5, 
                0.5, 0.5, 0.1127017, 0.8872983, 0.5, 0.1127017, 
                0.5, 0.5, 0.1127017, 0.8872983, 0.5, 0.5, 
                0.5, 0.5, 0.1127017, 0.8872983, 0.5, 0.8872983, 
                0.5, 0.5, 0.1127017, 0.8872983, 0.8872983, 0.5, 
                0.5, 0.5, 0.2828781, 0.5, 0.5, 0.5, 
                0.5, 0.5, 0.5, 0.0197543999999999, 0.5, 0.5, 
                0.5, 0.5, 0.5, 0.1127017, 0.1127017, 0.1127017, 
                0.5, 0.5, 0.5, 0.1127017, 0.1127017, 0.5, 
                0.5, 0.5, 0.5, 0.1127017, 0.1127017, 0.8872983, 
                0.5, 0.5, 0.5, 0.1127017, 0.5, 0.1127017, 
                0.5, 0.5, 0.5, 0.1127017, 0.5, 0.5, 
                0.5, 0.5, 0.5, 0.1127017, 0.5, 0.8872983, 
                0.5, 0.5, 0.5, 0.1127017, 0.8872983, 0.1127017, 
                0.5, 0.5, 0.5, 0.1127017, 0.8872983, 0.5, 
                0.5, 0.5, 0.5, 0.1127017, 0.8872983, 0.8872983, 
                0.5, 0.5, 0.5, 0.2828781, 0.5, 0.5, 
                0.5, 0.5, 0.5, 0.5, 0.0197543999999999, 0.5, 
                0.5, 0.5, 0.5, 0.5, 0.1127017, 0.1127017, 
                0.5, 0.5, 0.5, 0.5, 0.1127017, 0.5, 
                0.5, 0.5, 0.5, 0.5, 0.1127017, 0.8872983, 
                0.5, 0.5, 0.5, 0.5, 0.2828781, 0.5, 
                0.5, 0.5, 0.5, 0.5, 0.5, 0.0197543999999999, 
                0.5, 0.5, 0.5, 0.5, 0.5, 0.1127017, 
                0.5, 0.5, 0.5, 0.5, 0.5, 0.2828781, 
                0.5, 0.5, 0.5, 0.5, 0.5, 0.5, 
                0.5, 0.5, 0.5, 0.5, 0.5, 0.7171219, 
                0.5, 0.5, 0.5, 0.5, 0.5, 0.8872983, 
                0.5, 0.5, 0.5, 0.5, 0.5, 0.9802456, 
                0.5, 0.5, 0.5, 0.5, 0.7171219, 0.5, 
                0.5, 0.5, 0.5, 0.5, 0.8872983, 0.1127017, 
                0.5, 0.5, 0.5, 0.5, 0.8872983, 0.5, 
                0.5, 0.5, 0.5, 0.5, 0.8872983, 0.8872983, 
                0.5, 0.5, 0.5, 0.5, 0.9802456, 0.5, 
                0.5, 0.5, 0.5, 0.7171219, 0.5, 0.5, 
                0.5, 0.5, 0.5, 0.8872983, 0.1127017, 0.1127017, 
                0.5, 0.5, 0.5, 0.8872983, 0.1127017, 0.5, 
                0.5, 0.5, 0.5, 0.8872983, 0.1127017, 0.8872983, 
                0.5, 0.5, 0.5, 0.8872983, 0.5, 0.1127017, 
                0.5, 0.5, 0.5, 0.8872983, 0.5, 0.5, 
                0.5, 0.5, 0.5, 0.8872983, 0.5, 0.8872983, 
                0.5, 0.5, 0.5, 0.8872983, 0.8872983, 0.1127017, 
                0.5, 0.5, 0.5, 0.8872983, 0.8872983, 0.5, 
                0.5, 0.5, 0.5, 0.8872983, 0.8872983, 0.8872983, 
                0.5, 0.5, 0.5, 0.9802456, 0.5, 0.5, 
                0.5, 0.5, 0.7171219, 0.5, 0.5, 0.5, 
                0.5, 0.5, 0.8872983, 0.1127017, 0.1127017, 0.5, 
                0.5, 0.5, 0.8872983, 0.1127017, 0.5, 0.1127017, 
                0.5, 0.5, 0.8872983, 0.1127017, 0.5, 0.5, 
                0.5, 0.5, 0.8872983, 0.1127017, 0.5, 0.8872983, 
                0.5, 0.5, 0.8872983, 0.1127017, 0.8872983, 0.5, 
                0.5, 0.5, 0.8872983, 0.5, 0.1127017, 0.1127017, 
                0.5, 0.5, 0.8872983, 0.5, 0.1127017, 0.5, 
                0.5, 0.5, 0.8872983, 0.5, 0.1127017, 0.8872983, 
                0.5, 0.5, 0.8872983, 0.5, 0.5, 0.1127017, 
                0.5, 0.5, 0.8872983, 0.5, 0.5, 0.5, 
                0.5, 0.5, 0.8872983, 0.5, 0.5, 0.8872983, 
                0.5, 0.5, 0.8872983, 0.5, 0.8872983, 0.1127017, 
                0.5, 0.5, 0.8872983, 0.5, 0.8872983, 0.5, 
                0.5, 0.5, 0.8872983, 0.5, 0.8872983, 0.8872983, 
                0.5, 0.5, 0.8872983, 0.8872983, 0.1127017, 0.5, 
                0.5, 0.5, 0.8872983, 0.8872983, 0.5, 0.1127017, 
                0.5, 0.5, 0.8872983, 0.8872983, 0.5, 0.5, 
                0.5, 0.5, 0.8872983, 0.8872983, 0.5, 0.8872983, 
                0.5, 0.5, 0.8872983, 0.8872983, 0.8872983, 0.5, 
                0.5, 0.5, 0.9802456, 0.5, 0.5, 0.5, 
                0.5, 0.7171219, 0.5, 0.5, 0.5, 0.5, 
                0.5, 0.8872983, 0.1127017, 0.1127017, 0.5, 0.5, 
                0.5, 0.8872983, 0.1127017, 0.5, 0.1127017, 0.5, 
                0.5, 0.8872983, 0.1127017, 0.5, 0.5, 0.1127017, 
                0.5, 0.8872983, 0.1127017, 0.5, 0.5, 0.5, 
                0.5, 0.8872983, 0.1127017, 0.5, 0.5, 0.8872983, 
                0.5, 0.8872983, 0.1127017, 0.5, 0.8872983, 0.5, 
                0.5, 0.8872983, 0.1127017, 0.8872983, 0.5, 0.5, 
                0.5, 0.8872983, 0.5, 0.1127017, 0.1127017, 0.5, 
                0.5, 0.8872983, 0.5, 0.1127017, 0.5, 0.1127017, 
                0.5, 0.8872983, 0.5, 0.1127017, 0.5, 0.5, 
                0.5, 0.8872983, 0.5, 0.1127017, 0.5, 0.8872983, 
                0.5, 0.8872983, 0.5, 0.1127017, 0.8872983, 0.5, 
                0.5, 0.8872983, 0.5, 0.5, 0.1127017, 0.1127017, 
                0.5, 0.8872983, 0.5, 0.5, 0.1127017, 0.5, 
                0.5, 0.8872983, 0.5, 0.5, 0.1127017, 0.8872983, 
                0.5, 0.8872983, 0.5, 0.5, 0.5, 0.1127017, 
                0.5, 0.8872983, 0.5, 0.5, 0.5, 0.5, 
                0.5, 0.8872983, 0.5, 0.5, 0.5, 0.8872983, 
                0.5, 0.8872983, 0.5, 0.5, 0.8872983, 0.1127017, 
                0.5, 0.8872983, 0.5, 0.5, 0.8872983, 0.5, 
                0.5, 0.8872983, 0.5, 0.5, 0.8872983, 0.8872983, 
                0.5, 0.8872983, 0.5, 0.8872983, 0.1127017, 0.5, 
                0.5, 0.8872983, 0.5, 0.8872983, 0.5, 0.1127017, 
                0.5, 0.8872983, 0.5, 0.8872983, 0.5, 0.5, 
                0.5, 0.8872983, 0.5, 0.8872983, 0.5, 0.8872983, 
                0.5, 0.8872983, 0.5, 0.8872983, 0.8872983, 0.5, 
                0.5, 0.8872983, 0.8872983, 0.1127017, 0.5, 0.5, 
                0.5, 0.8872983, 0.8872983, 0.5, 0.1127017, 0.5, 
                0.5, 0.8872983, 0.8872983, 0.5, 0.5, 0.1127017, 
                0.5, 0.8872983, 0.8872983, 0.5, 0.5, 0.5, 
                0.5, 0.8872983, 0.8872983, 0.5, 0.5, 0.8872983, 
                0.5, 0.8872983, 0.8872983, 0.5, 0.8872983, 0.5, 
                0.5, 0.8872983, 0.8872983, 0.8872983, 0.5, 0.5, 
                0.5, 0.9802456, 0.5, 0.5, 0.5, 0.5, 
                0.7171219, 0.5, 0.5, 0.5, 0.5, 0.5, 
                0.8872983, 0.1127017, 0.1127017, 0.5, 0.5, 0.5, 
                0.8872983, 0.1127017, 0.5, 0.1127017, 0.5, 0.5, 
                0.8872983, 0.1127017, 0.5, 0.5, 0.1127017, 0.5, 
                0.8872983, 0.1127017, 0.5, 0.5, 0.5, 0.1127017, 
                0.8872983, 0.1127017, 0.5, 0.5, 0.5, 0.5, 
                0.8872983, 0.1127017, 0.5, 0.5, 0.5, 0.8872983, 
                0.8872983, 0.1127017, 0.5, 0.5, 0.8872983, 0.5, 
                0.8872983, 0.1127017, 0.5, 0.8872983, 0.5, 0.5, 
                0.8872983, 0.1127017, 0.8872983, 0.5, 0.5, 0.5, 
                0.8872983, 0.5, 0.1127017, 0.1127017, 0.5, 0.5, 
                0.8872983, 0.5, 0.1127017, 0.5, 0.1127017, 0.5, 
                0.8872983, 0.5, 0.1127017, 0.5, 0.5, 0.1127017, 
                0.8872983, 0.5, 0.1127017, 0.5, 0.5, 0.5, 
                0.8872983, 0.5, 0.1127017, 0.5, 0.5, 0.8872983, 
                0.8872983, 0.5, 0.1127017, 0.5, 0.8872983, 0.5, 
                0.8872983, 0.5, 0.1127017, 0.8872983, 0.5, 0.5, 
                0.8872983, 0.5, 0.5, 0.1127017, 0.1127017, 0.5, 
                0.8872983, 0.5, 0.5, 0.1127017, 0.5, 0.1127017, 
                0.8872983, 0.5, 0.5, 0.1127017, 0.5, 0.5, 
                0.8872983, 0.5, 0.5, 0.1127017, 0.5, 0.8872983, 
                0.8872983, 0.5, 0.5, 0.1127017, 0.8872983, 0.5, 
                0.8872983, 0.5, 0.5, 0.5, 0.1127017, 0.1127017, 
                0.8872983, 0.5, 0.5, 0.5, 0.1127017, 0.5, 
                0.8872983, 0.5, 0.5, 0.5, 0.1127017, 0.8872983, 
                0.8872983, 0.5, 0.5, 0.5, 0.5, 0.1127017, 
                0.8872983, 0.5, 0.5, 0.5, 0.5, 0.5, 
                0.8872983, 0.5, 0.5, 0.5, 0.5, 0.8872983, 
                0.8872983, 0.5, 0.5, 0.5, 0.8872983, 0.1127017, 
                0.8872983, 0.5, 0.5, 0.5, 0.8872983, 0.5, 
                0.8872983, 0.5, 0.5, 0.5, 0.8872983, 0.8872983, 
                0.8872983, 0.5, 0.5, 0.8872983, 0.1127017, 0.5, 
                0.8872983, 0.5, 0.5, 0.8872983, 0.5, 0.1127017, 
                0.8872983, 0.5, 0.5, 0.8872983, 0.5, 0.5, 
                0.8872983, 0.5, 0.5, 0.8872983, 0.5, 0.8872983, 
                0.8872983, 0.5, 0.5, 0.8872983, 0.8872983, 0.5, 
                0.8872983, 0.5, 0.8872983, 0.1127017, 0.5, 0.5, 
                0.8872983, 0.5, 0.8872983, 0.5, 0.1127017, 0.5, 
                0.8872983, 0.5, 0.8872983, 0.5, 0.5, 0.1127017, 
                0.8872983, 0.5, 0.8872983, 0.5, 0.5, 0.5, 
                0.8872983, 0.5, 0.8872983, 0.5, 0.5, 0.8872983, 
                0.8872983, 0.5, 0.8872983, 0.5, 0.8872983, 0.5, 
                0.8872983, 0.5, 0.8872983, 0.8872983, 0.5, 0.5, 
                0.8872983, 0.8872983, 0.1127017, 0.5, 0.5, 0.5, 
                0.8872983, 0.8872983, 0.5, 0.1127017, 0.5, 0.5, 
                0.8872983, 0.8872983, 0.5, 0.5, 0.1127017, 0.5, 
                0.8872983, 0.8872983, 0.5, 0.5, 0.5, 0.1127017, 
                0.8872983, 0.8872983, 0.5, 0.5, 0.5, 0.5, 
                0.8872983, 0.8872983, 0.5, 0.5, 0.5, 0.8872983, 
                0.8872983, 0.8872983, 0.5, 0.5, 0.8872983, 0.5, 
                0.8872983, 0.8872983, 0.5, 0.8872983, 0.5, 0.5, 
                0.8872983, 0.8872983, 0.8872983, 0.5, 0.5, 0.5, 
                0.9802456, 0.5, 0.5, 0.5, 0.5, 0.5
            }),
            /* w = */ std::vector<double>
            ({
                 0.0523281052328106, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                -0.0943072990397835, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                -0.0943072990397835, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                -0.0943072990397835, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                -0.0943072990397835, 
                 0.021433475651578, 
                -0.0943072990397835, 
                 0.219977977759118, 
                -0.0943072990397835, 
                 0.021433475651578, 
                -0.0943072990397835, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                -0.0943072990397835, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                -0.0943072990397835, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                -0.0943072990397835, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.200698720069872, 
                 0.0523281052328106, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                -0.0943072990397834, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                -0.0943072990397835, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                -0.0943072990397835, 
                 0.021433475651578, 
                -0.0943072990397834, 
                 0.219977977759118, 
                -0.0943072990397834, 
                 0.021433475651578, 
                -0.0943072990397835, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                -0.0943072990397835, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                -0.0943072990397834, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.200698720069872, 
                 0.0523281052328106, 
                 0.021433475651578, 
                 0.021433475651578, 
                -0.0943072990397834, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                -0.0943072990397835, 
                 0.021433475651578, 
                -0.0943072990397835, 
                 0.219977977759118, 
                -0.0943072990397835, 
                 0.021433475651578, 
                -0.0943072990397835, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                -0.0943072990397834, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.200698720069872, 
                 0.0523281052328106, 
                 0.021433475651578, 
                -0.0943072990397834, 
                 0.021433475651578, 
                -0.0943072990397834, 
                 0.219977977759118, 
                -0.0943072990397834, 
                 0.021433475651578, 
                -0.0943072990397834, 
                 0.021433475651578, 
                 0.200698720069872, 
                 0.0523281052328106, 
                -0.0943072990397835, 
                 0.219977977759118, 
                -0.0943072990397835, 
                 0.200698720069872, 
                 0.0523281052328106, 
                 0.219977977759118, 
                 0.200698720069872, 
                -2.44697579860708, 
                 0.200698720069872, 
                 0.219977977759118, 
                 0.0523281052328106, 
                 0.200698720069872, 
                -0.0943072990397835, 
                 0.219977977759118, 
                -0.0943072990397835, 
                 0.0523281052328106, 
                 0.200698720069872, 
                 0.021433475651578, 
                -0.0943072990397834, 
                 0.021433475651578, 
                -0.0943072990397834, 
                 0.219977977759118, 
                -0.0943072990397834, 
                 0.021433475651578, 
                -0.0943072990397834, 
                 0.021433475651578, 
                 0.0523281052328106, 
                 0.200698720069872, 
                 0.021433475651578, 
                 0.021433475651578, 
                -0.0943072990397834, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                -0.0943072990397835, 
                 0.021433475651578, 
                -0.0943072990397835, 
                 0.219977977759118, 
                -0.0943072990397835, 
                 0.021433475651578, 
                -0.0943072990397835, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                -0.0943072990397834, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.0523281052328106, 
                 0.200698720069872, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                -0.0943072990397834, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                -0.0943072990397835, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                -0.0943072990397835, 
                 0.021433475651578, 
                -0.0943072990397834, 
                 0.219977977759118, 
                -0.0943072990397834, 
                 0.021433475651578, 
                -0.0943072990397835, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                -0.0943072990397835, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                -0.0943072990397834, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.0523281052328106, 
                 0.200698720069872, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                -0.0943072990397835, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                -0.0943072990397835, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                -0.0943072990397835, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                -0.0943072990397835, 
                 0.021433475651578, 
                -0.0943072990397835, 
                 0.219977977759118, 
                -0.0943072990397835, 
                 0.021433475651578, 
                -0.0943072990397835, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                -0.0943072990397835, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                -0.0943072990397835, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                -0.0943072990397835, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.021433475651578, 
                 0.0523281052328106
            })
        }
    },
    {
        d6l5n737,
        {
            /* dim = */ 6,
            /* n = */ 737,
            /* x = */ std::vector<double>
            ({
                0.0197543999999999, 0.1127017, 0.5, 0.5, 0.5, 0.5, 
                0.0197543999999999, 0.5, 0.1127017, 0.5, 0.5, 0.5, 
                0.0197543999999999, 0.5, 0.5, 0.1127017, 0.5, 0.5, 
                0.0197543999999999, 0.5, 0.5, 0.5, 0.1127017, 0.5, 
                0.0197543999999999, 0.5, 0.5, 0.5, 0.5, 0.1127017, 
                0.0197543999999999, 0.5, 0.5, 0.5, 0.5, 0.5, 
                0.0197543999999999, 0.5, 0.5, 0.5, 0.5, 0.8872983, 
                0.0197543999999999, 0.5, 0.5, 0.5, 0.8872983, 0.5, 
                0.0197543999999999, 0.5, 0.5, 0.8872983, 0.5, 0.5, 
                0.0197543999999999, 0.5, 0.8872983, 0.5, 0.5, 0.5, 
                0.0197543999999999, 0.8872983, 0.5, 0.5, 0.5, 0.5, 
                0.1127017, 0.0197543999999999, 0.5, 0.5, 0.5, 0.5, 
                0.1127017, 0.1127017, 0.1127017, 0.1127017, 0.5, 0.5, 
                0.1127017, 0.1127017, 0.1127017, 0.5, 0.1127017, 0.5, 
                0.1127017, 0.1127017, 0.1127017, 0.5, 0.5, 0.1127017, 
                0.1127017, 0.1127017, 0.1127017, 0.5, 0.5, 0.5, 
                0.1127017, 0.1127017, 0.1127017, 0.5, 0.5, 0.8872983, 
                0.1127017, 0.1127017, 0.1127017, 0.5, 0.8872983, 0.5, 
                0.1127017, 0.1127017, 0.1127017, 0.8872983, 0.5, 0.5, 
                0.1127017, 0.1127017, 0.5, 0.1127017, 0.1127017, 0.5, 
                0.1127017, 0.1127017, 0.5, 0.1127017, 0.5, 0.1127017, 
                0.1127017, 0.1127017, 0.5, 0.1127017, 0.5, 0.5, 
                0.1127017, 0.1127017, 0.5, 0.1127017, 0.5, 0.8872983, 
                0.1127017, 0.1127017, 0.5, 0.1127017, 0.8872983, 0.5, 
                0.1127017, 0.1127017, 0.5, 0.5, 0.1127017, 0.1127017, 
                0.1127017, 0.1127017, 0.5, 0.5, 0.1127017, 0.5, 
                0.1127017, 0.1127017, 0.5, 0.5, 0.1127017, 0.8872983, 
                0.1127017, 0.1127017, 0.5, 0.5, 0.5, 0.1127017, 
                0.1127017, 0.1127017, 0.5, 0.5, 0.5, 0.5, 
                0.1127017, 0.1127017, 0.5, 0.5, 0.5, 0.8872983, 
                0.1127017, 0.1127017, 0.5, 0.5, 0.8872983, 0.1127017, 
                0.1127017, 0.1127017, 0.5, 0.5, 0.8872983, 0.5, 
                0.1127017, 0.1127017, 0.5, 0.5, 0.8872983, 0.8872983, 
                0.1127017, 0.1127017, 0.5, 0.8872983, 0.1127017, 0.5, 
                0.1127017, 0.1127017, 0.5, 0.8872983, 0.5, 0.1127017, 
                0.1127017, 0.1127017, 0.5, 0.8872983, 0.5, 0.5, 
                0.1127017, 0.1127017, 0.5, 0.8872983, 0.5, 0.8872983, 
                0.1127017, 0.1127017, 0.5, 0.8872983, 0.8872983, 0.5, 
                0.1127017, 0.1127017, 0.8872983, 0.1127017, 0.5, 0.5, 
                0.1127017, 0.1127017, 0.8872983, 0.5, 0.1127017, 0.5, 
                0.1127017, 0.1127017, 0.8872983, 0.5, 0.5, 0.1127017, 
                0.1127017, 0.1127017, 0.8872983, 0.5, 0.5, 0.5, 
                0.1127017, 0.1127017, 0.8872983, 0.5, 0.5, 0.8872983, 
                0.1127017, 0.1127017, 0.8872983, 0.5, 0.8872983, 0.5, 
                0.1127017, 0.1127017, 0.8872983, 0.8872983, 0.5, 0.5, 
                0.1127017, 0.2828781, 0.5, 0.5, 0.5, 0.5, 
                0.1127017, 0.5, 0.0197543999999999, 0.5, 0.5, 0.5, 
                0.1127017, 0.5, 0.1127017, 0.1127017, 0.1127017, 0.5, 
                0.1127017, 0.5, 0.1127017, 0.1127017, 0.5, 0.1127017, 
                0.1127017, 0.5, 0.1127017, 0.1127017, 0.5, 0.5, 
                0.1127017, 0.5, 0.1127017, 0.1127017, 0.5, 0.8872983, 
                0.1127017, 0.5, 0.1127017, 0.1127017, 0.8872983, 0.5, 
                0.1127017, 0.5, 0.1127017, 0.5, 0.1127017, 0.1127017, 
                0.1127017, 0.5, 0.1127017, 0.5, 0.1127017, 0.5, 
                0.1127017, 0.5, 0.1127017, 0.5, 0.1127017, 0.8872983, 
                0.1127017, 0.5, 0.1127017, 0.5, 0.5, 0.1127017, 
                0.1127017, 0.5, 0.1127017, 0.5, 0.5, 0.5, 
                0.1127017, 0.5, 0.1127017, 0.5, 0.5, 0.8872983, 
                0.1127017, 0.5, 0.1127017, 0.5, 0.8872983, 0.1127017, 
                0.1127017, 0.5, 0.1127017, 0.5, 0.8872983, 0.5, 
                0.1127017, 0.5, 0.1127017, 0.5, 0.8872983, 0.8872983, 
                0.1127017, 0.5, 0.1127017, 0.8872983, 0.1127017, 0.5, 
                0.1127017, 0.5, 0.1127017, 0.8872983, 0.5, 0.1127017, 
                0.1127017, 0.5, 0.1127017, 0.8872983, 0.5, 0.5, 
                0.1127017, 0.5, 0.1127017, 0.8872983, 0.5, 0.8872983, 
                0.1127017, 0.5, 0.1127017, 0.8872983, 0.8872983, 0.5, 
                0.1127017, 0.5, 0.2828781, 0.5, 0.5, 0.5, 
                0.1127017, 0.5, 0.5, 0.0197543999999999, 0.5, 0.5, 
                0.1127017, 0.5, 0.5, 0.1127017, 0.1127017, 0.1127017, 
                0.1127017, 0.5, 0.5, 0.1127017, 0.1127017, 0.5, 
                0.1127017, 0.5, 0.5, 0.1127017, 0.1127017, 0.8872983, 
                0.1127017, 0.5, 0.5, 0.1127017, 0.5, 0.1127017, 
                0.1127017, 0.5, 0.5, 0.1127017, 0.5, 0.5, 
                0.1127017, 0.5, 0.5, 0.1127017, 0.5, 0.8872983, 
                0.1127017, 0.5, 0.5, 0.1127017, 0.8872983, 0.1127017, 
                0.1127017, 0.5, 0.5, 0.1127017, 0.8872983, 0.5, 
                0.1127017, 0.5, 0.5, 0.1127017, 0.8872983, 0.8872983, 
                0.1127017, 0.5, 0.5, 0.2828781, 0.5, 0.5, 
                0.1127017, 0.5, 0.5, 0.5, 0.0197543999999999, 0.5, 
                0.1127017, 0.5, 0.5, 0.5, 0.1127017, 0.1127017, 
                0.1127017, 0.5, 0.5, 0.5, 0.1127017, 0.5, 
                0.1127017, 0.5, 0.5, 0.5, 0.1127017, 0.8872983, 
                0.1127017, 0.5, 0.5, 0.5, 0.2828781, 0.5, 
                0.1127017, 0.5, 0.5, 0.5, 0.5, 0.0197543999999999, 
                0.1127017, 0.5, 0.5, 0.5, 0.5, 0.1127017, 
                0.1127017, 0.5, 0.5, 0.5, 0.5, 0.2828781, 
                0.1127017, 0.5, 0.5, 0.5, 0.5, 0.5, 
                0.1127017, 0.5, 0.5, 0.5, 0.5, 0.7171219, 
                0.1127017, 0.5, 0.5, 0.5, 0.5, 0.8872983, 
                0.1127017, 0.5, 0.5, 0.5, 0.5, 0.9802456, 
                0.1127017, 0.5, 0.5, 0.5, 0.7171219, 0.5, 
                0.1127017, 0.5, 0.5, 0.5, 0.8872983, 0.1127017, 
                0.1127017, 0.5, 0.5, 0.5, 0.8872983, 0.5, 
                0.1127017, 0.5, 0.5, 0.5, 0.8872983, 0.8872983, 
                0.1127017, 0.5, 0.5, 0.5, 0.9802456, 0.5, 
                0.1127017, 0.5, 0.5, 0.7171219, 0.5, 0.5, 
                0.1127017, 0.5, 0.5, 0.8872983, 0.1127017, 0.1127017, 
                0.1127017, 0.5, 0.5, 0.8872983, 0.1127017, 0.5, 
                0.1127017, 0.5, 0.5, 0.8872983, 0.1127017, 0.8872983, 
                0.1127017, 0.5, 0.5, 0.8872983, 0.5, 0.1127017, 
                0.1127017, 0.5, 0.5, 0.8872983, 0.5, 0.5, 
                0.1127017, 0.5, 0.5, 0.8872983, 0.5, 0.8872983, 
                0.1127017, 0.5, 0.5, 0.8872983, 0.8872983, 0.1127017, 
                0.1127017, 0.5, 0.5, 0.8872983, 0.8872983, 0.5, 
                0.1127017, 0.5, 0.5, 0.8872983, 0.8872983, 0.8872983, 
                0.1127017, 0.5, 0.5, 0.9802456, 0.5, 0.5, 
                0.1127017, 0.5, 0.7171219, 0.5, 0.5, 0.5, 
                0.1127017, 0.5, 0.8872983, 0.1127017, 0.1127017, 0.5, 
                0.1127017, 0.5, 0.8872983, 0.1127017, 0.5, 0.1127017, 
                0.1127017, 0.5, 0.8872983, 0.1127017, 0.5, 0.5, 
                0.1127017, 0.5, 0.8872983, 0.1127017, 0.5, 0.8872983, 
                0.1127017, 0.5, 0.8872983, 0.1127017, 0.8872983, 0.5, 
                0.1127017, 0.5, 0.8872983, 0.5, 0.1127017, 0.1127017, 
                0.1127017, 0.5, 0.8872983, 0.5, 0.1127017, 0.5, 
                0.1127017, 0.5, 0.8872983, 0.5, 0.1127017, 0.8872983, 
                0.1127017, 0.5, 0.8872983, 0.5, 0.5, 0.1127017, 
                0.1127017, 0.5, 0.8872983, 0.5, 0.5, 0.5, 
                0.1127017, 0.5, 0.8872983, 0.5, 0.5, 0.8872983, 
                0.1127017, 0.5, 0.8872983, 0.5, 0.8872983, 0.1127017, 
                0.1127017, 0.5, 0.8872983, 0.5, 0.8872983, 0.5, 
                0.1127017, 0.5, 0.8872983, 0.5, 0.8872983, 0.8872983, 
                0.1127017, 0.5, 0.8872983, 0.8872983, 0.1127017, 0.5, 
                0.1127017, 0.5, 0.8872983, 0.8872983, 0.5, 0.1127017, 
                0.1127017, 0.5, 0.8872983, 0.8872983, 0.5, 0.5, 
                0.1127017, 0.5, 0.8872983, 0.8872983, 0.5, 0.8872983, 
                0.1127017, 0.5, 0.8872983, 0.8872983, 0.8872983, 0.5, 
                0.1127017, 0.5, 0.9802456, 0.5, 0.5, 0.5, 
                0.1127017, 0.7171219, 0.5, 0.5, 0.5, 0.5, 
                0.1127017, 0.8872983, 0.1127017, 0.1127017, 0.5, 0.5, 
                0.1127017, 0.8872983, 0.1127017, 0.5, 0.1127017, 0.5, 
                0.1127017, 0.8872983, 0.1127017, 0.5, 0.5, 0.1127017, 
                0.1127017, 0.8872983, 0.1127017, 0.5, 0.5, 0.5, 
                0.1127017, 0.8872983, 0.1127017, 0.5, 0.5, 0.8872983, 
                0.1127017, 0.8872983, 0.1127017, 0.5, 0.8872983, 0.5, 
                0.1127017, 0.8872983, 0.1127017, 0.8872983, 0.5, 0.5, 
                0.1127017, 0.8872983, 0.5, 0.1127017, 0.1127017, 0.5, 
                0.1127017, 0.8872983, 0.5, 0.1127017, 0.5, 0.1127017, 
                0.1127017, 0.8872983, 0.5, 0.1127017, 0.5, 0.5, 
                0.1127017, 0.8872983, 0.5, 0.1127017, 0.5, 0.8872983, 
                0.1127017, 0.8872983, 0.5, 0.1127017, 0.8872983, 0.5, 
                0.1127017, 0.8872983, 0.5, 0.5, 0.1127017, 0.1127017, 
                0.1127017, 0.8872983, 0.5, 0.5, 0.1127017, 0.5, 
                0.1127017, 0.8872983, 0.5, 0.5, 0.1127017, 0.8872983, 
                0.1127017, 0.8872983, 0.5, 0.5, 0.5, 0.1127017, 
                0.1127017, 0.8872983, 0.5, 0.5, 0.5, 0.5, 
                0.1127017, 0.8872983, 0.5, 0.5, 0.5, 0.8872983, 
                0.1127017, 0.8872983, 0.5, 0.5, 0.8872983, 0.1127017, 
                0.1127017, 0.8872983, 0.5, 0.5, 0.8872983, 0.5, 
                0.1127017, 0.8872983, 0.5, 0.5, 0.8872983, 0.8872983, 
                0.1127017, 0.8872983, 0.5, 0.8872983, 0.1127017, 0.5, 
                0.1127017, 0.8872983, 0.5, 0.8872983, 0.5, 0.1127017, 
                0.1127017, 0.8872983, 0.5, 0.8872983, 0.5, 0.5, 
                0.1127017, 0.8872983, 0.5, 0.8872983, 0.5, 0.8872983, 
                0.1127017, 0.8872983, 0.5, 0.8872983, 0.8872983, 0.5, 
                0.1127017, 0.8872983, 0.8872983, 0.1127017, 0.5, 0.5, 
                0.1127017, 0.8872983, 0.8872983, 0.5, 0.1127017, 0.5, 
                0.1127017, 0.8872983, 0.8872983, 0.5, 0.5, 0.1127017, 
                0.1127017, 0.8872983, 0.8872983, 0.5, 0.5, 0.5, 
                0.1127017, 0.8872983, 0.8872983, 0.5, 0.5, 0.8872983, 
                0.1127017, 0.8872983, 0.8872983, 0.5, 0.8872983, 0.5, 
                0.1127017, 0.8872983, 0.8872983, 0.8872983, 0.5, 0.5, 
                0.1127017, 0.9802456, 0.5, 0.5, 0.5, 0.5, 
                0.2828781, 0.1127017, 0.5, 0.5, 0.5, 0.5, 
                0.2828781, 0.5, 0.1127017, 0.5, 0.5, 0.5, 
                0.2828781, 0.5, 0.5, 0.1127017, 0.5, 0.5, 
                0.2828781, 0.5, 0.5, 0.5, 0.1127017, 0.5, 
                0.2828781, 0.5, 0.5, 0.5, 0.5, 0.1127017, 
                0.2828781, 0.5, 0.5, 0.5, 0.5, 0.5, 
                0.2828781, 0.5, 0.5, 0.5, 0.5, 0.8872983, 
                0.2828781, 0.5, 0.5, 0.5, 0.8872983, 0.5, 
                0.2828781, 0.5, 0.5, 0.8872983, 0.5, 0.5, 
                0.2828781, 0.5, 0.8872983, 0.5, 0.5, 0.5, 
                0.2828781, 0.8872983, 0.5, 0.5, 0.5, 0.5, 
                0.5, 0.0197543999999999, 0.1127017, 0.5, 0.5, 0.5, 
                0.5, 0.0197543999999999, 0.5, 0.1127017, 0.5, 0.5, 
                0.5, 0.0197543999999999, 0.5, 0.5, 0.1127017, 0.5, 
                0.5, 0.0197543999999999, 0.5, 0.5, 0.5, 0.1127017, 
                0.5, 0.0197543999999999, 0.5, 0.5, 0.5, 0.5, 
                0.5, 0.0197543999999999, 0.5, 0.5, 0.5, 0.8872983, 
                0.5, 0.0197543999999999, 0.5, 0.5, 0.8872983, 0.5, 
                0.5, 0.0197543999999999, 0.5, 0.8872983, 0.5, 0.5, 
                0.5, 0.0197543999999999, 0.8872983, 0.5, 0.5, 0.5, 
                0.5, 0.1127017, 0.0197543999999999, 0.5, 0.5, 0.5, 
                0.5, 0.1127017, 0.1127017, 0.1127017, 0.1127017, 0.5, 
                0.5, 0.1127017, 0.1127017, 0.1127017, 0.5, 0.1127017, 
                0.5, 0.1127017, 0.1127017, 0.1127017, 0.5, 0.5, 
                0.5, 0.1127017, 0.1127017, 0.1127017, 0.5, 0.8872983, 
                0.5, 0.1127017, 0.1127017, 0.1127017, 0.8872983, 0.5, 
                0.5, 0.1127017, 0.1127017, 0.5, 0.1127017, 0.1127017, 
                0.5, 0.1127017, 0.1127017, 0.5, 0.1127017, 0.5, 
                0.5, 0.1127017, 0.1127017, 0.5, 0.1127017, 0.8872983, 
                0.5, 0.1127017, 0.1127017, 0.5, 0.5, 0.1127017, 
                0.5, 0.1127017, 0.1127017, 0.5, 0.5, 0.5, 
                0.5, 0.1127017, 0.1127017, 0.5, 0.5, 0.8872983, 
                0.5, 0.1127017, 0.1127017, 0.5, 0.8872983, 0.1127017, 
                0.5, 0.1127017, 0.1127017, 0.5, 0.8872983, 0.5, 
                0.5, 0.1127017, 0.1127017, 0.5, 0.8872983, 0.8872983, 
                0.5, 0.1127017, 0.1127017, 0.8872983, 0.1127017, 0.5, 
                0.5, 0.1127017, 0.1127017, 0.8872983, 0.5, 0.1127017, 
                0.5, 0.1127017, 0.1127017, 0.8872983, 0.5, 0.5, 
                0.5, 0.1127017, 0.1127017, 0.8872983, 0.5, 0.8872983, 
                0.5, 0.1127017, 0.1127017, 0.8872983, 0.8872983, 0.5, 
                0.5, 0.1127017, 0.2828781, 0.5, 0.5, 0.5, 
                0.5, 0.1127017, 0.5, 0.0197543999999999, 0.5, 0.5, 
                0.5, 0.1127017, 0.5, 0.1127017, 0.1127017, 0.1127017, 
                0.5, 0.1127017, 0.5, 0.1127017, 0.1127017, 0.5, 
                0.5, 0.1127017, 0.5, 0.1127017, 0.1127017, 0.8872983, 
                0.5, 0.1127017, 0.5, 0.1127017, 0.5, 0.1127017, 
                0.5, 0.1127017, 0.5, 0.1127017, 0.5, 0.5, 
                0.5, 0.1127017, 0.5, 0.1127017, 0.5, 0.8872983, 
                0.5, 0.1127017, 0.5, 0.1127017, 0.8872983, 0.1127017, 
                0.5, 0.1127017, 0.5, 0.1127017, 0.8872983, 0.5, 
                0.5, 0.1127017, 0.5, 0.1127017, 0.8872983, 0.8872983, 
                0.5, 0.1127017, 0.5, 0.2828781, 0.5, 0.5, 
                0.5, 0.1127017, 0.5, 0.5, 0.0197543999999999, 0.5, 
                0.5, 0.1127017, 0.5, 0.5, 0.1127017, 0.1127017, 
                0.5, 0.1127017, 0.5, 0.5, 0.1127017, 0.5, 
                0.5, 0.1127017, 0.5, 0.5, 0.1127017, 0.8872983, 
                0.5, 0.1127017, 0.5, 0.5, 0.2828781, 0.5, 
                0.5, 0.1127017, 0.5, 0.5, 0.5, 0.0197543999999999, 
                0.5, 0.1127017, 0.5, 0.5, 0.5, 0.1127017, 
                0.5, 0.1127017, 0.5, 0.5, 0.5, 0.2828781, 
                0.5, 0.1127017, 0.5, 0.5, 0.5, 0.5, 
                0.5, 0.1127017, 0.5, 0.5, 0.5, 0.7171219, 
                0.5, 0.1127017, 0.5, 0.5, 0.5, 0.8872983, 
                0.5, 0.1127017, 0.5, 0.5, 0.5, 0.9802456, 
                0.5, 0.1127017, 0.5, 0.5, 0.7171219, 0.5, 
                0.5, 0.1127017, 0.5, 0.5, 0.8872983, 0.1127017, 
                0.5, 0.1127017, 0.5, 0.5, 0.8872983, 0.5, 
                0.5, 0.1127017, 0.5, 0.5, 0.8872983, 0.8872983, 
                0.5, 0.1127017, 0.5, 0.5, 0.9802456, 0.5, 
                0.5, 0.1127017, 0.5, 0.7171219, 0.5, 0.5, 
                0.5, 0.1127017, 0.5, 0.8872983, 0.1127017, 0.1127017, 
                0.5, 0.1127017, 0.5, 0.8872983, 0.1127017, 0.5, 
                0.5, 0.1127017, 0.5, 0.8872983, 0.1127017, 0.8872983, 
                0.5, 0.1127017, 0.5, 0.8872983, 0.5, 0.1127017, 
                0.5, 0.1127017, 0.5, 0.8872983, 0.5, 0.5, 
                0.5, 0.1127017, 0.5, 0.8872983, 0.5, 0.8872983, 
                0.5, 0.1127017, 0.5, 0.8872983, 0.8872983, 0.1127017, 
                0.5, 0.1127017, 0.5, 0.8872983, 0.8872983, 0.5, 
                0.5, 0.1127017, 0.5, 0.8872983, 0.8872983, 0.8872983, 
                0.5, 0.1127017, 0.5, 0.9802456, 0.5, 0.5, 
                0.5, 0.1127017, 0.7171219, 0.5, 0.5, 0.5, 
                0.5, 0.1127017, 0.8872983, 0.1127017, 0.1127017, 0.5, 
                0.5, 0.1127017, 0.8872983, 0.1127017, 0.5, 0.1127017, 
                0.5, 0.1127017, 0.8872983, 0.1127017, 0.5, 0.5, 
                0.5, 0.1127017, 0.8872983, 0.1127017, 0.5, 0.8872983, 
                0.5, 0.1127017, 0.8872983, 0.1127017, 0.8872983, 0.5, 
                0.5, 0.1127017, 0.8872983, 0.5, 0.1127017, 0.1127017, 
                0.5, 0.1127017, 0.8872983, 0.5, 0.1127017, 0.5, 
                0.5, 0.1127017, 0.8872983, 0.5, 0.1127017, 0.8872983, 
                0.5, 0.1127017, 0.8872983, 0.5, 0.5, 0.1127017, 
                0.5, 0.1127017, 0.8872983, 0.5, 0.5, 0.5, 
                0.5, 0.1127017, 0.8872983, 0.5, 0.5, 0.8872983, 
                0.5, 0.1127017, 0.8872983, 0.5, 0.8872983, 0.1127017, 
                0.5, 0.1127017, 0.8872983, 0.5, 0.8872983, 0.5, 
                0.5, 0.1127017, 0.8872983, 0.5, 0.8872983, 0.8872983, 
                0.5, 0.1127017, 0.8872983, 0.8872983, 0.1127017, 0.5, 
                0.5, 0.1127017, 0.8872983, 0.8872983, 0.5, 0.1127017, 
                0.5, 0.1127017, 0.8872983, 0.8872983, 0.5, 0.5, 
                0.5, 0.1127017, 0.8872983, 0.8872983, 0.5, 0.8872983, 
                0.5, 0.1127017, 0.8872983, 0.8872983, 0.8872983, 0.5, 
                0.5, 0.1127017, 0.9802456, 0.5, 0.5, 0.5, 
                0.5, 0.2828781, 0.1127017, 0.5, 0.5, 0.5, 
                0.5, 0.2828781, 0.5, 0.1127017, 0.5, 0.5, 
                0.5, 0.2828781, 0.5, 0.5, 0.1127017, 0.5, 
                0.5, 0.2828781, 0.5, 0.5, 0.5, 0.1127017, 
                0.5, 0.2828781, 0.5, 0.5, 0.5, 0.5, 
                0.5, 0.2828781, 0.5, 0.5, 0.5, 0.8872983, 
                0.5, 0.2828781, 0.5, 0.5, 0.8872983, 0.5, 
                0.5, 0.2828781, 0.5, 0.8872983, 0.5, 0.5, 
                0.5, 0.2828781, 0.8872983, 0.5, 0.5, 0.5, 
                0.5, 0.5, 0.0197543999999999, 0.1127017, 0.5, 0.5, 
                0.5, 0.5, 0.0197543999999999, 0.5, 0.1127017, 0.5, 
                0.5, 0.5, 0.0197543999999999, 0.5, 0.5, 0.1127017, 
                0.5, 0.5, 0.0197543999999999, 0.5, 0.5, 0.5, 
                0.5, 0.5, 0.0197543999999999, 0.5, 0.5, 0.8872983, 
                0.5, 0.5, 0.0197543999999999, 0.5, 0.8872983, 0.5, 
                0.5, 0.5, 0.0197543999999999, 0.8872983, 0.5, 0.5, 
                0.5, 0.5, 0.1127017, 0.0197543999999999, 0.5, 0.5, 
                0.5, 0.5, 0.1127017, 0.1127017, 0.1127017, 0.1127017, 
                0.5, 0.5, 0.1127017, 0.1127017, 0.1127017, 0.5, 
                0.5, 0.5, 0.1127017, 0.1127017, 0.1127017, 0.8872983, 
                0.5, 0.5, 0.1127017, 0.1127017, 0.5, 0.1127017, 
                0.5, 0.5, 0.1127017, 0.1127017, 0.5, 0.5, 
                0.5, 0.5, 0.1127017, 0.1127017, 0.5, 0.8872983, 
                0.5, 0.5, 0.1127017, 0.1127017, 0.8872983, 0.1127017, 
                0.5, 0.5, 0.1127017, 0.1127017, 0.8872983, 0.5, 
                0.5, 0.5, 0.1127017, 0.1127017, 0.8872983, 0.8872983, 
                0.5, 0.5, 0.1127017, 0.2828781, 0.5, 0.5, 
                0.5, 0.5, 0.1127017, 0.5, 0.0197543999999999, 0.5, 
                0.5, 0.5, 0.1127017, 0.5, 0.1127017, 0.1127017, 
                0.5, 0.5, 0.1127017, 0.5, 0.1127017, 0.5, 
                0.5, 0.5, 0.1127017, 0.5, 0.1127017, 0.8872983, 
                0.5, 0.5, 0.1127017, 0.5, 0.2828781, 0.5, 
                0.5, 0.5, 0.1127017, 0.5, 0.5, 0.0197543999999999, 
                0.5, 0.5, 0.1127017, 0.5, 0.5, 0.1127017, 
                0.5, 0.5, 0.1127017, 0.5, 0.5, 0.2828781, 
                0.5, 0.5, 0.1127017, 0.5, 0.5, 0.5, 
                0.5, 0.5, 0.1127017, 0.5, 0.5, 0.7171219, 
                0.5, 0.5, 0.1127017, 0.5, 0.5, 0.8872983, 
                0.5, 0.5, 0.1127017, 0.5, 0.5, 0.9802456, 
                0.5, 0.5, 0.1127017, 0.5, 0.7171219, 0.5, 
                0.5, 0.5, 0.1127017, 0.5, 0.8872983, 0.1127017, 
                0.5, 0.5, 0.1127017, 0.5, 0.8872983, 0.5, 
                0.5, 0.5, 0.1127017, 0.5, 0.8872983, 0.8872983, 
                0.5, 0.5, 0.1127017, 0.5, 0.9802456, 0.5, 
                0.5, 0.5, 0.1127017, 0.7171219, 0.5, 0.5, 
                0.5, 0.5, 0.1127017, 0.8872983, 0.1127017, 0.1127017, 
                0.5, 0.5, 0.1127017, 0.8872983, 0.1127017, 0.5, 
                0.5, 0.5, 0.1127017, 0.8872983, 0.1127017, 0.8872983, 
                0.5, 0.5, 0.1127017, 0.8872983, 0.5, 0.1127017, 
                0.5, 0.5, 0.1127017, 0.8872983, 0.5, 0.5, 
                0.5, 0.5, 0.1127017, 0.8872983, 0.5, 0.8872983, 
                0.5, 0.5, 0.1127017, 0.8872983, 0.8872983, 0.1127017, 
                0.5, 0.5, 0.1127017, 0.8872983, 0.8872983, 0.5, 
                0.5, 0.5, 0.1127017, 0.8872983, 0.8872983, 0.8872983, 
                0.5, 0.5, 0.1127017, 0.9802456, 0.5, 0.5, 
                0.5, 0.5, 0.2828781, 0.1127017, 0.5, 0.5, 
                0.5, 0.5, 0.2828781, 0.5, 0.1127017, 0.5, 
                0.5, 0.5, 0.2828781, 0.5, 0.5, 0.1127017, 
                0.5, 0.5, 0.2828781, 0.5, 0.5, 0.5, 
                0.5, 0.5, 0.2828781, 0.5, 0.5, 0.8872983, 
                0.5, 0.5, 0.2828781, 0.5, 0.8872983, 0.5, 
                0.5, 0.5, 0.2828781, 0.8872983, 0.5, 0.5, 
                0.5, 0.5, 0.5, 0.0197543999999999, 0.1127017, 0.5, 
                0.5, 0.5, 0.5, 0.0197543999999999, 0.5, 0.1127017, 
                0.5, 0.5, 0.5, 0.0197543999999999, 0.5, 0.5, 
                0.5, 0.5, 0.5, 0.0197543999999999, 0.5, 0.8872983, 
                0.5, 0.5, 0.5, 0.0197543999999999, 0.8872983, 0.5, 
                0.5, 0.5, 0.5, 0.1127017, 0.0197543999999999, 0.5, 
                0.5, 0.5, 0.5, 0.1127017, 0.1127017, 0.1127017, 
                0.5, 0.5, 0.5, 0.1127017, 0.1127017, 0.5, 
                0.5, 0.5, 0.5, 0.1127017, 0.1127017, 0.8872983, 
                0.5, 0.5, 0.5, 0.1127017, 0.2828781, 0.5, 
                0.5, 0.5, 0.5, 0.1127017, 0.5, 0.0197543999999999, 
                0.5, 0.5, 0.5, 0.1127017, 0.5, 0.1127017, 
                0.5, 0.5, 0.5, 0.1127017, 0.5, 0.2828781, 
                0.5, 0.5, 0.5, 0.1127017, 0.5, 0.5, 
                0.5, 0.5, 0.5, 0.1127017, 0.5, 0.7171219, 
                0.5, 0.5, 0.5, 0.1127017, 0.5, 0.8872983, 
                0.5, 0.5, 0.5, 0.1127017, 0.5, 0.9802456, 
                0.5, 0.5, 0.5, 0.1127017, 0.7171219, 0.5, 
                0.5, 0.5, 0.5, 0.1127017, 0.8872983, 0.1127017, 
                0.5, 0.5, 0.5, 0.1127017, 0.8872983, 0.5, 
                0.5, 0.5, 0.5, 0.1127017, 0.8872983, 0.8872983, 
                0.5, 0.5, 0.5, 0.1127017, 0.9802456, 0.5, 
                0.5, 0.5, 0.5, 0.2828781, 0.1127017, 0.5, 
                0.5, 0.5, 0.5, 0.2828781, 0.5, 0.1127017, 
                0.5, 0.5, 0.5, 0.2828781, 0.5, 0.5, 
                0.5, 0.5, 0.5, 0.2828781, 0.5, 0.8872983, 
                0.5, 0.5, 0.5, 0.2828781, 0.8872983, 0.5, 
                0.5, 0.5, 0.5, 0.5, 0.0197543999999999, 0.1127017, 
                0.5, 0.5, 0.5, 0.5, 0.0197543999999999, 0.5, 
                0.5, 0.5, 0.5, 0.5, 0.0197543999999999, 0.8872983, 
                0.5, 0.5, 0.5, 0.5, 0.1127017, 0.0197543999999999, 
                0.5, 0.5, 0.5, 0.5, 0.1127017, 0.1127017, 
                0.5, 0.5, 0.5, 0.5, 0.1127017, 0.2828781, 
                0.5, 0.5, 0.5, 0.5, 0.1127017, 0.5, 
                0.5, 0.5, 0.5, 0.5, 0.1127017, 0.7171219, 
                0.5, 0.5, 0.5, 0.5, 0.1127017, 0.8872983, 
                0.5, 0.5, 0.5, 0.5, 0.1127017, 0.9802456, 
                0.5, 0.5, 0.5, 0.5, 0.2828781, 0.1127017, 
                0.5, 0.5, 0.5, 0.5, 0.2828781, 0.5, 
                0.5, 0.5, 0.5, 0.5, 0.2828781, 0.8872983, 
                0.5, 0.5, 0.5, 0.5, 0.5, 0.0197543999999999, 
                0.5, 0.5, 0.5, 0.5, 0.5, 0.1127017, 
                0.5, 0.5, 0.5, 0.5, 0.5, 0.2828781, 
                0.5, 0.5, 0.5, 0.5, 0.5, 0.5, 
                0.5, 0.5, 0.5, 0.5, 0.5, 0.7171219, 
                0.5, 0.5, 0.5, 0.5, 0.5, 0.8872983, 
                0.5, 0.5, 0.5, 0.5, 0.5, 0.9802456, 
                0.5, 0.5, 0.5, 0.5, 0.7171219, 0.1127017, 
                0.5, 0.5, 0.5, 0.5, 0.7171219, 0.5, 
                0.5, 0.5, 0.5, 0.5, 0.7171219, 0.8872983, 
                0.5, 0.5, 0.5, 0.5, 0.8872983, 0.0197543999999999, 
                0.5, 0.5, 0.5, 0.5, 0.8872983, 0.1127017, 
                0.5, 0.5, 0.5, 0.5, 0.8872983, 0.2828781, 
                0.5, 0.5, 0.5, 0.5, 0.8872983, 0.5, 
                0.5, 0.5, 0.5, 0.5, 0.8872983, 0.7171219, 
                0.5, 0.5, 0.5, 0.5, 0.8872983, 0.8872983, 
                0.5, 0.5, 0.5, 0.5, 0.8872983, 0.9802456, 
                0.5, 0.5, 0.5, 0.5, 0.9802456, 0.1127017, 
                0.5, 0.5, 0.5, 0.5, 0.9802456, 0.5, 
                0.5, 0.5, 0.5, 0.5, 0.9802456, 0.8872983, 
                0.5, 0.5, 0.5, 0.7171219, 0.1127017, 0.5, 
                0.5, 0.5, 0.5, 0.7171219, 0.5, 0.1127017, 
                0.5, 0.5, 0.5, 0.7171219, 0.5, 0.5, 
                0.5, 0.5, 0.5, 0.7171219, 0.5, 0.8872983, 
                0.5, 0.5, 0.5, 0.7171219, 0.8872983, 0.5, 
                0.5, 0.5, 0.5, 0.8872983, 0.0197543999999999, 0.5, 
                0.5, 0.5, 0.5, 0.8872983, 0.1127017, 0.1127017, 
                0.5, 0.5, 0.5, 0.8872983, 0.1127017, 0.5, 
                0.5, 0.5, 0.5, 0.8872983, 0.1127017, 0.8872983, 
                0.5, 0.5, 0.5, 0.8872983, 0.2828781, 0.5, 
                0.5, 0.5, 0.5, 0.8872983, 0.5, 0.0197543999999999, 
                0.5, 0.5, 0.5, 0.8872983, 0.5, 0.1127017, 
                0.5, 0.5, 0.5, 0.8872983, 0.5, 0.2828781, 
                0.5, 0.5, 0.5, 0.8872983, 0.5, 0.5, 
                0.5, 0.5, 0.5, 0.8872983, 0.5, 0.7171219, 
                0.5, 0.5, 0.5, 0.8872983, 0.5, 0.8872983, 
                0.5, 0.5, 0.5, 0.8872983, 0.5, 0.9802456, 
                0.5, 0.5, 0.5, 0.8872983, 0.7171219, 0.5, 
                0.5, 0.5, 0.5, 0.8872983, 0.8872983, 0.1127017, 
                0.5, 0.5, 0.5, 0.8872983, 0.8872983, 0.5, 
                0.5, 0.5, 0.5, 0.8872983, 0.8872983, 0.8872983, 
                0.5, 0.5, 0.5, 0.8872983, 0.9802456, 0.5, 
                0.5, 0.5, 0.5, 0.9802456, 0.1127017, 0.5, 
                0.5, 0.5, 0.5, 0.9802456, 0.5, 0.1127017, 
                0.5, 0.5, 0.5, 0.9802456, 0.5, 0.5, 
                0.5, 0.5, 0.5, 0.9802456, 0.5, 0.8872983, 
                0.5, 0.5, 0.5, 0.9802456, 0.8872983, 0.5, 
                0.5, 0.5, 0.7171219, 0.1127017, 0.5, 0.5, 
                0.5, 0.5, 0.7171219, 0.5, 0.1127017, 0.5, 
                0.5, 0.5, 0.7171219, 0.5, 0.5, 0.1127017, 
                0.5, 0.5, 0.7171219, 0.5, 0.5, 0.5, 
                0.5, 0.5, 0.7171219, 0.5, 0.5, 0.8872983, 
                0.5, 0.5, 0.7171219, 0.5, 0.8872983, 0.5, 
                0.5, 0.5, 0.7171219, 0.8872983, 0.5, 0.5, 
                0.5, 0.5, 0.8872983, 0.0197543999999999, 0.5, 0.5, 
                0.5, 0.5, 0.8872983, 0.1127017, 0.1127017, 0.1127017, 
                0.5, 0.5, 0.8872983, 0.1127017, 0.1127017, 0.5, 
                0.5, 0.5, 0.8872983, 0.1127017, 0.1127017, 0.8872983, 
                0.5, 0.5, 0.8872983, 0.1127017, 0.5, 0.1127017, 
                0.5, 0.5, 0.8872983, 0.1127017, 0.5, 0.5, 
                0.5, 0.5, 0.8872983, 0.1127017, 0.5, 0.8872983, 
                0.5, 0.5, 0.8872983, 0.1127017, 0.8872983, 0.1127017, 
                0.5, 0.5, 0.8872983, 0.1127017, 0.8872983, 0.5, 
                0.5, 0.5, 0.8872983, 0.1127017, 0.8872983, 0.8872983, 
                0.5, 0.5, 0.8872983, 0.2828781, 0.5, 0.5, 
                0.5, 0.5, 0.8872983, 0.5, 0.0197543999999999, 0.5, 
                0.5, 0.5, 0.8872983, 0.5, 0.1127017, 0.1127017, 
                0.5, 0.5, 0.8872983, 0.5, 0.1127017, 0.5, 
                0.5, 0.5, 0.8872983, 0.5, 0.1127017, 0.8872983, 
                0.5, 0.5, 0.8872983, 0.5, 0.2828781, 0.5, 
                0.5, 0.5, 0.8872983, 0.5, 0.5, 0.0197543999999999, 
                0.5, 0.5, 0.8872983, 0.5, 0.5, 0.1127017, 
                0.5, 0.5, 0.8872983, 0.5, 0.5, 0.2828781, 
                0.5, 0.5, 0.8872983, 0.5, 0.5, 0.5, 
                0.5, 0.5, 0.8872983, 0.5, 0.5, 0.7171219, 
                0.5, 0.5, 0.8872983, 0.5, 0.5, 0.8872983, 
                0.5, 0.5, 0.8872983, 0.5, 0.5, 0.9802456, 
                0.5, 0.5, 0.8872983, 0.5, 0.7171219, 0.5, 
                0.5, 0.5, 0.8872983, 0.5, 0.8872983, 0.1127017, 
                0.5, 0.5, 0.8872983, 0.5, 0.8872983, 0.5, 
                0.5, 0.5, 0.8872983, 0.5, 0.8872983, 0.8872983, 
                0.5, 0.5, 0.8872983, 0.5, 0.9802456, 0.5, 
                0.5, 0.5, 0.8872983, 0.7171219, 0.5, 0.5, 
                0.5, 0.5, 0.8872983, 0.8872983, 0.1127017, 0.1127017, 
                0.5, 0.5, 0.8872983, 0.8872983, 0.1127017, 0.5, 
                0.5, 0.5, 0.8872983, 0.8872983, 0.1127017, 0.8872983, 
                0.5, 0.5, 0.8872983, 0.8872983, 0.5, 0.1127017, 
                0.5, 0.5, 0.8872983, 0.8872983, 0.5, 0.5, 
                0.5, 0.5, 0.8872983, 0.8872983, 0.5, 0.8872983, 
                0.5, 0.5, 0.8872983, 0.8872983, 0.8872983, 0.1127017, 
                0.5, 0.5, 0.8872983, 0.8872983, 0.8872983, 0.5, 
                0.5, 0.5, 0.8872983, 0.8872983, 0.8872983, 0.8872983, 
                0.5, 0.5, 0.8872983, 0.9802456, 0.5, 0.5, 
                0.5, 0.5, 0.9802456, 0.1127017, 0.5, 0.5, 
                0.5, 0.5, 0.9802456, 0.5, 0.1127017, 0.5, 
                0.5, 0.5, 0.9802456, 0.5, 0.5, 0.1127017, 
                0.5, 0.5, 0.9802456, 0.5, 0.5, 0.5, 
                0.5, 0.5, 0.9802456, 0.5, 0.5, 0.8872983, 
                0.5, 0.5, 0.9802456, 0.5, 0.8872983, 0.5, 
                0.5, 0.5, 0.9802456, 0.8872983, 0.5, 0.5, 
                0.5, 0.7171219, 0.1127017, 0.5, 0.5, 0.5, 
                0.5, 0.7171219, 0.5, 0.1127017, 0.5, 0.5, 
                0.5, 0.7171219, 0.5, 0.5, 0.1127017, 0.5, 
                0.5, 0.7171219, 0.5, 0.5, 0.5, 0.1127017, 
                0.5, 0.7171219, 0.5, 0.5, 0.5, 0.5, 
                0.5, 0.7171219, 0.5, 0.5, 0.5, 0.8872983, 
                0.5, 0.7171219, 0.5, 0.5, 0.8872983, 0.5, 
                0.5, 0.7171219, 0.5, 0.8872983, 0.5, 0.5, 
                0.5, 0.7171219, 0.8872983, 0.5, 0.5, 0.5, 
                0.5, 0.8872983, 0.0197543999999999, 0.5, 0.5, 0.5, 
                0.5, 0.8872983, 0.1127017, 0.1127017, 0.1127017, 0.5, 
                0.5, 0.8872983, 0.1127017, 0.1127017, 0.5, 0.1127017, 
                0.5, 0.8872983, 0.1127017, 0.1127017, 0.5, 0.5, 
                0.5, 0.8872983, 0.1127017, 0.1127017, 0.5, 0.8872983, 
                0.5, 0.8872983, 0.1127017, 0.1127017, 0.8872983, 0.5, 
                0.5, 0.8872983, 0.1127017, 0.5, 0.1127017, 0.1127017, 
                0.5, 0.8872983, 0.1127017, 0.5, 0.1127017, 0.5, 
                0.5, 0.8872983, 0.1127017, 0.5, 0.1127017, 0.8872983, 
                0.5, 0.8872983, 0.1127017, 0.5, 0.5, 0.1127017, 
                0.5, 0.8872983, 0.1127017, 0.5, 0.5, 0.5, 
                0.5, 0.8872983, 0.1127017, 0.5, 0.5, 0.8872983, 
                0.5, 0.8872983, 0.1127017, 0.5, 0.8872983, 0.1127017, 
                0.5, 0.8872983, 0.1127017, 0.5, 0.8872983, 0.5, 
                0.5, 0.8872983, 0.1127017, 0.5, 0.8872983, 0.8872983, 
                0.5, 0.8872983, 0.1127017, 0.8872983, 0.1127017, 0.5, 
                0.5, 0.8872983, 0.1127017, 0.8872983, 0.5, 0.1127017, 
                0.5, 0.8872983, 0.1127017, 0.8872983, 0.5, 0.5, 
                0.5, 0.8872983, 0.1127017, 0.8872983, 0.5, 0.8872983, 
                0.5, 0.8872983, 0.1127017, 0.8872983, 0.8872983, 0.5, 
                0.5, 0.8872983, 0.2828781, 0.5, 0.5, 0.5, 
                0.5, 0.8872983, 0.5, 0.0197543999999999, 0.5, 0.5, 
                0.5, 0.8872983, 0.5, 0.1127017, 0.1127017, 0.1127017, 
                0.5, 0.8872983, 0.5, 0.1127017, 0.1127017, 0.5, 
                0.5, 0.8872983, 0.5, 0.1127017, 0.1127017, 0.8872983, 
                0.5, 0.8872983, 0.5, 0.1127017, 0.5, 0.1127017, 
                0.5, 0.8872983, 0.5, 0.1127017, 0.5, 0.5, 
                0.5, 0.8872983, 0.5, 0.1127017, 0.5, 0.8872983, 
                0.5, 0.8872983, 0.5, 0.1127017, 0.8872983, 0.1127017, 
                0.5, 0.8872983, 0.5, 0.1127017, 0.8872983, 0.5, 
                0.5, 0.8872983, 0.5, 0.1127017, 0.8872983, 0.8872983, 
                0.5, 0.8872983, 0.5, 0.2828781, 0.5, 0.5, 
                0.5, 0.8872983, 0.5, 0.5, 0.0197543999999999, 0.5, 
                0.5, 0.8872983, 0.5, 0.5, 0.1127017, 0.1127017, 
                0.5, 0.8872983, 0.5, 0.5, 0.1127017, 0.5, 
                0.5, 0.8872983, 0.5, 0.5, 0.1127017, 0.8872983, 
                0.5, 0.8872983, 0.5, 0.5, 0.2828781, 0.5, 
                0.5, 0.8872983, 0.5, 0.5, 0.5, 0.0197543999999999, 
                0.5, 0.8872983, 0.5, 0.5, 0.5, 0.1127017, 
                0.5, 0.8872983, 0.5, 0.5, 0.5, 0.2828781, 
                0.5, 0.8872983, 0.5, 0.5, 0.5, 0.5, 
                0.5, 0.8872983, 0.5, 0.5, 0.5, 0.7171219, 
                0.5, 0.8872983, 0.5, 0.5, 0.5, 0.8872983, 
                0.5, 0.8872983, 0.5, 0.5, 0.5, 0.9802456, 
                0.5, 0.8872983, 0.5, 0.5, 0.7171219, 0.5, 
                0.5, 0.8872983, 0.5, 0.5, 0.8872983, 0.1127017, 
                0.5, 0.8872983, 0.5, 0.5, 0.8872983, 0.5, 
                0.5, 0.8872983, 0.5, 0.5, 0.8872983, 0.8872983, 
                0.5, 0.8872983, 0.5, 0.5, 0.9802456, 0.5, 
                0.5, 0.8872983, 0.5, 0.7171219, 0.5, 0.5, 
                0.5, 0.8872983, 0.5, 0.8872983, 0.1127017, 0.1127017, 
                0.5, 0.8872983, 0.5, 0.8872983, 0.1127017, 0.5, 
                0.5, 0.8872983, 0.5, 0.8872983, 0.1127017, 0.8872983, 
                0.5, 0.8872983, 0.5, 0.8872983, 0.5, 0.1127017, 
                0.5, 0.8872983, 0.5, 0.8872983, 0.5, 0.5, 
                0.5, 0.8872983, 0.5, 0.8872983, 0.5, 0.8872983, 
                0.5, 0.8872983, 0.5, 0.8872983, 0.8872983, 0.1127017, 
                0.5, 0.8872983, 0.5, 0.8872983, 0.8872983, 0.5, 
                0.5, 0.8872983, 0.5, 0.8872983, 0.8872983, 0.8872983, 
                0.5, 0.8872983, 0.5, 0.9802456, 0.5, 0.5, 
                0.5, 0.8872983, 0.7171219, 0.5, 0.5, 0.5, 
                0.5, 0.8872983, 0.8872983, 0.1127017, 0.1127017, 0.5, 
                0.5, 0.8872983, 0.8872983, 0.1127017, 0.5, 0.1127017, 
                0.5, 0.8872983, 0.8872983, 0.1127017, 0.5, 0.5, 
                0.5, 0.8872983, 0.8872983, 0.1127017, 0.5, 0.8872983, 
                0.5, 0.8872983, 0.8872983, 0.1127017, 0.8872983, 0.5, 
                0.5, 0.8872983, 0.8872983, 0.5, 0.1127017, 0.1127017, 
                0.5, 0.8872983, 0.8872983, 0.5, 0.1127017, 0.5, 
                0.5, 0.8872983, 0.8872983, 0.5, 0.1127017, 0.8872983, 
                0.5, 0.8872983, 0.8872983, 0.5, 0.5, 0.1127017, 
                0.5, 0.8872983, 0.8872983, 0.5, 0.5, 0.5, 
                0.5, 0.8872983, 0.8872983, 0.5, 0.5, 0.8872983, 
                0.5, 0.8872983, 0.8872983, 0.5, 0.8872983, 0.1127017, 
                0.5, 0.8872983, 0.8872983, 0.5, 0.8872983, 0.5, 
                0.5, 0.8872983, 0.8872983, 0.5, 0.8872983, 0.8872983, 
                0.5, 0.8872983, 0.8872983, 0.8872983, 0.1127017, 0.5, 
                0.5, 0.8872983, 0.8872983, 0.8872983, 0.5, 0.1127017, 
                0.5, 0.8872983, 0.8872983, 0.8872983, 0.5, 0.5, 
                0.5, 0.8872983, 0.8872983, 0.8872983, 0.5, 0.8872983, 
                0.5, 0.8872983, 0.8872983, 0.8872983, 0.8872983, 0.5, 
                0.5, 0.8872983, 0.9802456, 0.5, 0.5, 0.5, 
                0.5, 0.9802456, 0.1127017, 0.5, 0.5, 0.5, 
                0.5, 0.9802456, 0.5, 0.1127017, 0.5, 0.5, 
                0.5, 0.9802456, 0.5, 0.5, 0.1127017, 0.5, 
                0.5, 0.9802456, 0.5, 0.5, 0.5, 0.1127017, 
                0.5, 0.9802456, 0.5, 0.5, 0.5, 0.5, 
                0.5, 0.9802456, 0.5, 0.5, 0.5, 0.8872983, 
                0.5, 0.9802456, 0.5, 0.5, 0.8872983, 0.5, 
                0.5, 0.9802456, 0.5, 0.8872983, 0.5, 0.5, 
                0.5, 0.9802456, 0.8872983, 0.5, 0.5, 0.5, 
                0.7171219, 0.1127017, 0.5, 0.5, 0.5, 0.5, 
                0.7171219, 0.5, 0.1127017, 0.5, 0.5, 0.5, 
                0.7171219, 0.5, 0.5, 0.1127017, 0.5, 0.5, 
                0.7171219, 0.5, 0.5, 0.5, 0.1127017, 0.5, 
                0.7171219, 0.5, 0.5, 0.5, 0.5, 0.1127017, 
                0.7171219, 0.5, 0.5, 0.5, 0.5, 0.5, 
                0.7171219, 0.5, 0.5, 0.5, 0.5, 0.8872983, 
                0.7171219, 0.5, 0.5, 0.5, 0.8872983, 0.5, 
                0.7171219, 0.5, 0.5, 0.8872983, 0.5, 0.5, 
                0.7171219, 0.5, 0.8872983, 0.5, 0.5, 0.5, 
                0.7171219, 0.8872983, 0.5, 0.5, 0.5, 0.5, 
                0.8872983, 0.0197543999999999, 0.5, 0.5, 0.5, 0.5, 
                0.8872983, 0.1127017, 0.1127017, 0.1127017, 0.5, 0.5, 
                0.8872983, 0.1127017, 0.1127017, 0.5, 0.1127017, 0.5, 
                0.8872983, 0.1127017, 0.1127017, 0.5, 0.5, 0.1127017, 
                0.8872983, 0.1127017, 0.1127017, 0.5, 0.5, 0.5, 
                0.8872983, 0.1127017, 0.1127017, 0.5, 0.5, 0.8872983, 
                0.8872983, 0.1127017, 0.1127017, 0.5, 0.8872983, 0.5, 
                0.8872983, 0.1127017, 0.1127017, 0.8872983, 0.5, 0.5, 
                0.8872983, 0.1127017, 0.5, 0.1127017, 0.1127017, 0.5, 
                0.8872983, 0.1127017, 0.5, 0.1127017, 0.5, 0.1127017, 
                0.8872983, 0.1127017, 0.5, 0.1127017, 0.5, 0.5, 
                0.8872983, 0.1127017, 0.5, 0.1127017, 0.5, 0.8872983, 
                0.8872983, 0.1127017, 0.5, 0.1127017, 0.8872983, 0.5, 
                0.8872983, 0.1127017, 0.5, 0.5, 0.1127017, 0.1127017, 
                0.8872983, 0.1127017, 0.5, 0.5, 0.1127017, 0.5, 
                0.8872983, 0.1127017, 0.5, 0.5, 0.1127017, 0.8872983, 
                0.8872983, 0.1127017, 0.5, 0.5, 0.5, 0.1127017, 
                0.8872983, 0.1127017, 0.5, 0.5, 0.5, 0.5, 
                0.8872983, 0.1127017, 0.5, 0.5, 0.5, 0.8872983, 
                0.8872983, 0.1127017, 0.5, 0.5, 0.8872983, 0.1127017, 
                0.8872983, 0.1127017, 0.5, 0.5, 0.8872983, 0.5, 
                0.8872983, 0.1127017, 0.5, 0.5, 0.8872983, 0.8872983, 
                0.8872983, 0.1127017, 0.5, 0.8872983, 0.1127017, 0.5, 
                0.8872983, 0.1127017, 0.5, 0.8872983, 0.5, 0.1127017, 
                0.8872983, 0.1127017, 0.5, 0.8872983, 0.5, 0.5, 
                0.8872983, 0.1127017, 0.5, 0.8872983, 0.5, 0.8872983, 
                0.8872983, 0.1127017, 0.5, 0.8872983, 0.8872983, 0.5, 
                0.8872983, 0.1127017, 0.8872983, 0.1127017, 0.5, 0.5, 
                0.8872983, 0.1127017, 0.8872983, 0.5, 0.1127017, 0.5, 
                0.8872983, 0.1127017, 0.8872983, 0.5, 0.5, 0.1127017, 
                0.8872983, 0.1127017, 0.8872983, 0.5, 0.5, 0.5, 
                0.8872983, 0.1127017, 0.8872983, 0.5, 0.5, 0.8872983, 
                0.8872983, 0.1127017, 0.8872983, 0.5, 0.8872983, 0.5, 
                0.8872983, 0.1127017, 0.8872983, 0.8872983, 0.5, 0.5, 
                0.8872983, 0.2828781, 0.5, 0.5, 0.5, 0.5, 
                0.8872983, 0.5, 0.0197543999999999, 0.5, 0.5, 0.5, 
                0.8872983, 0.5, 0.1127017, 0.1127017, 0.1127017, 0.5, 
                0.8872983, 0.5, 0.1127017, 0.1127017, 0.5, 0.1127017, 
                0.8872983, 0.5, 0.1127017, 0.1127017, 0.5, 0.5, 
                0.8872983, 0.5, 0.1127017, 0.1127017, 0.5, 0.8872983, 
                0.8872983, 0.5, 0.1127017, 0.1127017, 0.8872983, 0.5, 
                0.8872983, 0.5, 0.1127017, 0.5, 0.1127017, 0.1127017, 
                0.8872983, 0.5, 0.1127017, 0.5, 0.1127017, 0.5, 
                0.8872983, 0.5, 0.1127017, 0.5, 0.1127017, 0.8872983, 
                0.8872983, 0.5, 0.1127017, 0.5, 0.5, 0.1127017, 
                0.8872983, 0.5, 0.1127017, 0.5, 0.5, 0.5, 
                0.8872983, 0.5, 0.1127017, 0.5, 0.5, 0.8872983, 
                0.8872983, 0.5, 0.1127017, 0.5, 0.8872983, 0.1127017, 
                0.8872983, 0.5, 0.1127017, 0.5, 0.8872983, 0.5, 
                0.8872983, 0.5, 0.1127017, 0.5, 0.8872983, 0.8872983, 
                0.8872983, 0.5, 0.1127017, 0.8872983, 0.1127017, 0.5, 
                0.8872983, 0.5, 0.1127017, 0.8872983, 0.5, 0.1127017, 
                0.8872983, 0.5, 0.1127017, 0.8872983, 0.5, 0.5, 
                0.8872983, 0.5, 0.1127017, 0.8872983, 0.5, 0.8872983, 
                0.8872983, 0.5, 0.1127017, 0.8872983, 0.8872983, 0.5, 
                0.8872983, 0.5, 0.2828781, 0.5, 0.5, 0.5, 
                0.8872983, 0.5, 0.5, 0.0197543999999999, 0.5, 0.5, 
                0.8872983, 0.5, 0.5, 0.1127017, 0.1127017, 0.1127017, 
                0.8872983, 0.5, 0.5, 0.1127017, 0.1127017, 0.5, 
                0.8872983, 0.5, 0.5, 0.1127017, 0.1127017, 0.8872983, 
                0.8872983, 0.5, 0.5, 0.1127017, 0.5, 0.1127017, 
                0.8872983, 0.5, 0.5, 0.1127017, 0.5, 0.5, 
                0.8872983, 0.5, 0.5, 0.1127017, 0.5, 0.8872983, 
                0.8872983, 0.5, 0.5, 0.1127017, 0.8872983, 0.1127017, 
                0.8872983, 0.5, 0.5, 0.1127017, 0.8872983, 0.5, 
                0.8872983, 0.5, 0.5, 0.1127017, 0.8872983, 0.8872983, 
                0.8872983, 0.5, 0.5, 0.2828781, 0.5, 0.5, 
                0.8872983, 0.5, 0.5, 0.5, 0.0197543999999999, 0.5, 
                0.8872983, 0.5, 0.5, 0.5, 0.1127017, 0.1127017, 
                0.8872983, 0.5, 0.5, 0.5, 0.1127017, 0.5, 
                0.8872983, 0.5, 0.5, 0.5, 0.1127017, 0.8872983, 
                0.8872983, 0.5, 0.5, 0.5, 0.2828781, 0.5, 
                0.8872983, 0.5, 0.5, 0.5, 0.5, 0.0197543999999999, 
                0.8872983, 0.5, 0.5, 0.5, 0.5, 0.1127017, 
                0.8872983, 0.5, 0.5, 0.5, 0.5, 0.2828781, 
                0.8872983, 0.5, 0.5, 0.5, 0.5, 0.5, 
                0.8872983, 0.5, 0.5, 0.5, 0.5, 0.7171219, 
                0.8872983, 0.5, 0.5, 0.5, 0.5, 0.8872983, 
                0.8872983, 0.5, 0.5, 0.5, 0.5, 0.9802456, 
                0.8872983, 0.5, 0.5, 0.5, 0.7171219, 0.5, 
                0.8872983, 0.5, 0.5, 0.5, 0.8872983, 0.1127017, 
                0.8872983, 0.5, 0.5, 0.5, 0.8872983, 0.5, 
                0.8872983, 0.5, 0.5, 0.5, 0.8872983, 0.8872983, 
                0.8872983, 0.5, 0.5, 0.5, 0.9802456, 0.5, 
                0.8872983, 0.5, 0.5, 0.7171219, 0.5, 0.5, 
                0.8872983, 0.5, 0.5, 0.8872983, 0.1127017, 0.1127017, 
                0.8872983, 0.5, 0.5, 0.8872983, 0.1127017, 0.5, 
                0.8872983, 0.5, 0.5, 0.8872983, 0.1127017, 0.8872983, 
                0.8872983, 0.5, 0.5, 0.8872983, 0.5, 0.1127017, 
                0.8872983, 0.5, 0.5, 0.8872983, 0.5, 0.5, 
                0.8872983, 0.5, 0.5, 0.8872983, 0.5, 0.8872983, 
                0.8872983, 0.5, 0.5, 0.8872983, 0.8872983, 0.1127017, 
                0.8872983, 0.5, 0.5, 0.8872983, 0.8872983, 0.5, 
                0.8872983, 0.5, 0.5, 0.8872983, 0.8872983, 0.8872983, 
                0.8872983, 0.5, 0.5, 0.9802456, 0.5, 0.5, 
                0.8872983, 0.5, 0.7171219, 0.5, 0.5, 0.5, 
                0.8872983, 0.5, 0.8872983, 0.1127017, 0.1127017, 0.5, 
                0.8872983, 0.5, 0.8872983, 0.1127017, 0.5, 0.1127017, 
                0.8872983, 0.5, 0.8872983, 0.1127017, 0.5, 0.5, 
                0.8872983, 0.5, 0.8872983, 0.1127017, 0.5, 0.8872983, 
                0.8872983, 0.5, 0.8872983, 0.1127017, 0.8872983, 0.5, 
                0.8872983, 0.5, 0.8872983, 0.5, 0.1127017, 0.1127017, 
                0.8872983, 0.5, 0.8872983, 0.5, 0.1127017, 0.5, 
                0.8872983, 0.5, 0.8872983, 0.5, 0.1127017, 0.8872983, 
                0.8872983, 0.5, 0.8872983, 0.5, 0.5, 0.1127017, 
                0.8872983, 0.5, 0.8872983, 0.5, 0.5, 0.5, 
                0.8872983, 0.5, 0.8872983, 0.5, 0.5, 0.8872983, 
                0.8872983, 0.5, 0.8872983, 0.5, 0.8872983, 0.1127017, 
                0.8872983, 0.5, 0.8872983, 0.5, 0.8872983, 0.5, 
                0.8872983, 0.5, 0.8872983, 0.5, 0.8872983, 0.8872983, 
                0.8872983, 0.5, 0.8872983, 0.8872983, 0.1127017, 0.5, 
                0.8872983, 0.5, 0.8872983, 0.8872983, 0.5, 0.1127017, 
                0.8872983, 0.5, 0.8872983, 0.8872983, 0.5, 0.5, 
                0.8872983, 0.5, 0.8872983, 0.8872983, 0.5, 0.8872983, 
                0.8872983, 0.5, 0.8872983, 0.8872983, 0.8872983, 0.5, 
                0.8872983, 0.5, 0.9802456, 0.5, 0.5, 0.5, 
                0.8872983, 0.7171219, 0.5, 0.5, 0.5, 0.5, 
                0.8872983, 0.8872983, 0.1127017, 0.1127017, 0.5, 0.5, 
                0.8872983, 0.8872983, 0.1127017, 0.5, 0.1127017, 0.5, 
                0.8872983, 0.8872983, 0.1127017, 0.5, 0.5, 0.1127017, 
                0.8872983, 0.8872983, 0.1127017, 0.5, 0.5, 0.5, 
                0.8872983, 0.8872983, 0.1127017, 0.5, 0.5, 0.8872983, 
                0.8872983, 0.8872983, 0.1127017, 0.5, 0.8872983, 0.5, 
                0.8872983, 0.8872983, 0.1127017, 0.8872983, 0.5, 0.5, 
                0.8872983, 0.8872983, 0.5, 0.1127017, 0.1127017, 0.5, 
                0.8872983, 0.8872983, 0.5, 0.1127017, 0.5, 0.1127017, 
                0.8872983, 0.8872983, 0.5, 0.1127017, 0.5, 0.5, 
                0.8872983, 0.8872983, 0.5, 0.1127017, 0.5, 0.8872983, 
                0.8872983, 0.8872983, 0.5, 0.1127017, 0.8872983, 0.5, 
                0.8872983, 0.8872983, 0.5, 0.5, 0.1127017, 0.1127017, 
                0.8872983, 0.8872983, 0.5, 0.5, 0.1127017, 0.5, 
                0.8872983, 0.8872983, 0.5, 0.5, 0.1127017, 0.8872983, 
                0.8872983, 0.8872983, 0.5, 0.5, 0.5, 0.1127017, 
                0.8872983, 0.8872983, 0.5, 0.5, 0.5, 0.5, 
                0.8872983, 0.8872983, 0.5, 0.5, 0.5, 0.8872983, 
                0.8872983, 0.8872983, 0.5, 0.5, 0.8872983, 0.1127017, 
                0.8872983, 0.8872983, 0.5, 0.5, 0.8872983, 0.5, 
                0.8872983, 0.8872983, 0.5, 0.5, 0.8872983, 0.8872983, 
                0.8872983, 0.8872983, 0.5, 0.8872983, 0.1127017, 0.5, 
                0.8872983, 0.8872983, 0.5, 0.8872983, 0.5, 0.1127017, 
                0.8872983, 0.8872983, 0.5, 0.8872983, 0.5, 0.5, 
                0.8872983, 0.8872983, 0.5, 0.8872983, 0.5, 0.8872983, 
                0.8872983, 0.8872983, 0.5, 0.8872983, 0.8872983, 0.5, 
                0.8872983, 0.8872983, 0.8872983, 0.1127017, 0.5, 0.5, 
                0.8872983, 0.8872983, 0.8872983, 0.5, 0.1127017, 0.5, 
                0.8872983, 0.8872983, 0.8872983, 0.5, 0.5, 0.1127017, 
                0.8872983, 0.8872983, 0.8872983, 0.5, 0.5, 0.5, 
                0.8872983, 0.8872983, 0.8872983, 0.5, 0.5, 0.8872983, 
                0.8872983, 0.8872983, 0.8872983, 0.5, 0.8872983, 0.5, 
                0.8872983, 0.8872983, 0.8872983, 0.8872983, 0.5, 0.5, 
                0.8872983, 0.9802456, 0.5, 0.5, 0.5, 0.5, 
                0.9802456, 0.1127017, 0.5, 0.5, 0.5, 0.5, 
                0.9802456, 0.5, 0.1127017, 0.5, 0.5, 0.5, 
                0.9802456, 0.5, 0.5, 0.1127017, 0.5, 0.5, 
                0.9802456, 0.5, 0.5, 0.5, 0.1127017, 0.5, 
                0.9802456, 0.5, 0.5, 0.5, 0.5, 0.1127017, 
                0.9802456, 0.5, 0.5, 0.5, 0.5, 0.5, 
                0.9802456, 0.5, 0.5, 0.5, 0.5, 0.8872983, 
                0.9802456, 0.5, 0.5, 0.5, 0.8872983, 0.5, 
                0.9802456, 0.5, 0.5, 0.8872983, 0.5, 0.5, 
                0.9802456, 0.5, 0.8872983, 0.5, 0.5, 0.5, 
                0.9802456, 0.8872983, 0.5, 0.5, 0.5, 0.5
            }),
            /* w = */ std::vector<double>
            ({
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                -0.0930277542645755, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                -0.031158448852689, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.055749648923825, 
                 0.0145355859497386, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                -0.0311584488526887, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.055749648923825, 
                 0.0145355859497386, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                -0.0311584488526889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                 0.055749648923825, 
                 0.0145355859497386, 
                -0.0142889866255154, 
                -0.031158448852689, 
                -0.0142889866255154, 
                 0.055749648923825, 
                 0.0145355859497386, 
                -0.0311584488526889, 
                 0.055749648923825, 
                -0.161763878791639, 
                 0.055749648923825, 
                -0.0311584488526889, 
                 0.0145355859497386, 
                 0.055749648923825, 
                -0.0142889866255154, 
                -0.031158448852689, 
                -0.0142889866255154, 
                 0.0145355859497386, 
                 0.055749648923825, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                -0.0311584488526889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                 0.0145355859497386, 
                 0.055749648923825, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                -0.0311584488526887, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.0145355859497386, 
                 0.055749648923825, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                -0.031158448852689, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.0145355859497386, 
                 0.055749648923825, 
                 0.055749648923825, 
                 0.055749648923825, 
                 0.055749648923825, 
                 0.055749648923825, 
                -0.356797769168378, 
                 0.055749648923825, 
                 0.055749648923825, 
                 0.055749648923825, 
                 0.055749648923825, 
                 0.055749648923825, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                -0.0930277542645755, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                -0.031158448852689, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.055749648923825, 
                 0.0145355859497386, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                -0.0311584488526889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                 0.055749648923825, 
                 0.0145355859497386, 
                -0.0142889866255154, 
                -0.0311584488526888, 
                -0.0142889866255154, 
                 0.055749648923825, 
                 0.0145355859497386, 
                -0.031158448852689, 
                 0.055749648923825, 
                -0.161763878791642, 
                 0.055749648923825, 
                -0.031158448852689, 
                 0.0145355859497386, 
                 0.055749648923825, 
                -0.0142889866255154, 
                -0.0311584488526888, 
                -0.0142889866255154, 
                 0.0145355859497386, 
                 0.055749648923825, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                -0.0311584488526889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                 0.0145355859497386, 
                 0.055749648923825, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                -0.031158448852689, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.0145355859497386, 
                 0.055749648923825, 
                 0.055749648923825, 
                 0.055749648923825, 
                 0.055749648923825, 
                -0.356797769168378, 
                 0.055749648923825, 
                 0.055749648923825, 
                 0.055749648923825, 
                 0.055749648923825, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                -0.0930277542645755, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                -0.031158448852689, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                 0.055749648923825, 
                 0.0145355859497386, 
                -0.0142889866255154, 
                -0.031158448852689, 
                -0.0142889866255154, 
                 0.055749648923825, 
                 0.0145355859497386, 
                -0.031158448852689, 
                 0.055749648923825, 
                -0.161763878791641, 
                 0.055749648923825, 
                -0.031158448852689, 
                 0.0145355859497386, 
                 0.055749648923825, 
                -0.0142889866255154, 
                -0.031158448852689, 
                -0.0142889866255154, 
                 0.0145355859497386, 
                 0.055749648923825, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                -0.031158448852689, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                 0.0145355859497386, 
                 0.055749648923825, 
                 0.055749648923825, 
                 0.055749648923825, 
                -0.356797769168378, 
                 0.055749648923825, 
                 0.055749648923825, 
                 0.055749648923825, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                -0.0930277542645755, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                -0.0142889866255154, 
                -0.0311584488526889, 
                -0.0142889866255154, 
                 0.055749648923825, 
                 0.0145355859497386, 
                -0.0311584488526887, 
                 0.055749648923825, 
                -0.161763878791641, 
                 0.055749648923825, 
                -0.0311584488526887, 
                 0.0145355859497386, 
                 0.055749648923825, 
                -0.0142889866255154, 
                -0.0311584488526889, 
                -0.0142889866255154, 
                 0.0145355859497386, 
                 0.055749648923825, 
                 0.055749648923825, 
                -0.356797769168378, 
                 0.055749648923825, 
                 0.055749648923825, 
                 0.0145355859497386, 
                -0.0930277542645755, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                -0.031158448852689, 
                 0.055749648923825, 
                -0.161763878791643, 
                 0.055749648923825, 
                -0.031158448852689, 
                 0.0145355859497386, 
                 0.055749648923825, 
                -0.356797769168378, 
                 0.055749648923825, 
                -0.0930277542645755, 
                -0.16176387879164, 
                -0.356797769168378, 
                 2.63169094202756, 
                -0.356797769168378, 
                -0.16176387879164, 
                -0.0930277542645755, 
                 0.055749648923825, 
                -0.356797769168378, 
                 0.055749648923825, 
                 0.0145355859497386, 
                -0.031158448852689, 
                 0.055749648923825, 
                -0.161763878791643, 
                 0.055749648923825, 
                -0.031158448852689, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                -0.0930277542645755, 
                 0.0145355859497386, 
                 0.055749648923825, 
                 0.055749648923825, 
                -0.356797769168378, 
                 0.055749648923825, 
                 0.055749648923825, 
                 0.0145355859497386, 
                -0.0142889866255154, 
                -0.0311584488526889, 
                -0.0142889866255154, 
                 0.055749648923825, 
                 0.0145355859497386, 
                -0.0311584488526887, 
                 0.055749648923825, 
                -0.161763878791641, 
                 0.055749648923825, 
                -0.0311584488526887, 
                 0.0145355859497386, 
                 0.055749648923825, 
                -0.0142889866255154, 
                -0.0311584488526889, 
                -0.0142889866255154, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                -0.0930277542645755, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.055749648923825, 
                 0.055749648923825, 
                 0.055749648923825, 
                -0.356797769168378, 
                 0.055749648923825, 
                 0.055749648923825, 
                 0.055749648923825, 
                 0.0145355859497386, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                -0.031158448852689, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                 0.055749648923825, 
                 0.0145355859497386, 
                -0.0142889866255154, 
                -0.031158448852689, 
                -0.0142889866255154, 
                 0.055749648923825, 
                 0.0145355859497386, 
                -0.031158448852689, 
                 0.055749648923825, 
                -0.161763878791641, 
                 0.055749648923825, 
                -0.031158448852689, 
                 0.0145355859497386, 
                 0.055749648923825, 
                -0.0142889866255154, 
                -0.031158448852689, 
                -0.0142889866255154, 
                 0.0145355859497386, 
                 0.055749648923825, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                -0.031158448852689, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                -0.0930277542645755, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.055749648923825, 
                 0.055749648923825, 
                 0.055749648923825, 
                 0.055749648923825, 
                -0.356797769168378, 
                 0.055749648923825, 
                 0.055749648923825, 
                 0.055749648923825, 
                 0.055749648923825, 
                 0.0145355859497386, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                -0.031158448852689, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.055749648923825, 
                 0.0145355859497386, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                -0.0311584488526889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                 0.055749648923825, 
                 0.0145355859497386, 
                -0.0142889866255154, 
                -0.0311584488526888, 
                -0.0142889866255154, 
                 0.055749648923825, 
                 0.0145355859497386, 
                -0.031158448852689, 
                 0.055749648923825, 
                -0.161763878791642, 
                 0.055749648923825, 
                -0.031158448852689, 
                 0.0145355859497386, 
                 0.055749648923825, 
                -0.0142889866255154, 
                -0.0311584488526888, 
                -0.0142889866255154, 
                 0.0145355859497386, 
                 0.055749648923825, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                -0.0311584488526889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                 0.0145355859497386, 
                 0.055749648923825, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                -0.031158448852689, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                -0.0930277542645755, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.055749648923825, 
                 0.055749648923825, 
                 0.055749648923825, 
                 0.055749648923825, 
                 0.055749648923825, 
                -0.356797769168378, 
                 0.055749648923825, 
                 0.055749648923825, 
                 0.055749648923825, 
                 0.055749648923825, 
                 0.055749648923825, 
                 0.0145355859497386, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                -0.031158448852689, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.055749648923825, 
                 0.0145355859497386, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                -0.0311584488526887, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.055749648923825, 
                 0.0145355859497386, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                -0.0311584488526889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                 0.055749648923825, 
                 0.0145355859497386, 
                -0.0142889866255154, 
                -0.031158448852689, 
                -0.0142889866255154, 
                 0.055749648923825, 
                 0.0145355859497386, 
                -0.0311584488526889, 
                 0.055749648923825, 
                -0.161763878791639, 
                 0.055749648923825, 
                -0.0311584488526889, 
                 0.0145355859497386, 
                 0.055749648923825, 
                -0.0142889866255154, 
                -0.031158448852689, 
                -0.0142889866255154, 
                 0.0145355859497386, 
                 0.055749648923825, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                -0.0311584488526889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                 0.0145355859497386, 
                 0.055749648923825, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                -0.0311584488526887, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.0145355859497386, 
                 0.055749648923825, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                -0.031158448852689, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                -0.0142889866255154, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.00595374371284889, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                -0.0930277542645755, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0145355859497386, 
                 0.0145355859497386
            })
        }
    }
};
